#include "canvas_button.h"

CanvasButton* canvas_button_new (GnomeCanvasGroup* parent, CanvasButtonDesc* desc)
{
	// --------------------------------------------------------------------
	// Creates new CanvasButton with default layout.
	// --------------------------------------------------------------------
	CanvasButton *button;
	GnomeCanvasPoints *points;
	
	// --------------------------------------------------------------------
	// CanvasButton initialization.
	// --------------------------------------------------------------------
	button = g_new (CanvasButton, 1);
	button->desc = desc;
	
	button->tip = NULL;
	button->tip_desc = NULL;
	
	button->x = 0.0;
	button->y = 0.0;
	button->hidden = false;
	button->enabled = true;
	button->pressed = false;
	
	button->has_mouse_hover = false;
	button->border_shown = button->desc->border_show;
	button->has_icon = false;		// this is going to be set to true only when an icon is set
	button->icon_visible = false;	// -- // --
	
	button->image = NULL;
	button->image_clicked = NULL;
	button->image_disabled = NULL;
	button->image_icon = NULL;
	button->image_icon_clicked = NULL;
	button->image_icon_disabled = NULL;
	
	// --------------------------------------------------------------------
	// Handle various button types
	// --------------------------------------------------------------------
	// switch - not done yet, TODO later
	
	// --------------------------------------------------------------------
	// Canvas items creation.
	// --------------------------------------------------------------------
	button->root_grp = gnome_canvas_item_new (
						parent,
						gnome_canvas_group_get_type (),
						"x", button->x,
						"y", button->y,
						NULL);
	if (!button->desc->toggle_button) // TODO else option --> IMPL callback for toggle buttons
		g_signal_connect (G_OBJECT(button->root_grp), "event", G_CALLBACK(OnCanvasButtonSwitchEvent), button);
	if (button->desc->plastic_on_motion)
		g_signal_connect (G_OBJECT(button->root_grp), "event", G_CALLBACK(OnCanvasButtonEnterLeaveEvent), button);
	
	// create button background (not for image button)
	if (button->desc->type == ButtonTypeImage)
	{
		// do nothing
	}
	else
	{
		button->rect = gnome_canvas_item_new (GNOME_CANVAS_GROUP(button->root_grp),
					gnome_canvas_rect_get_type (),
					"x1", 0.0,
					"y1", 0.0,
					"x2", button->desc->width,
					"y2", button->desc->height,
					"fill-color", button->desc->background_color, //"#AAAAFF", // nejaka modra
					NULL);
	}
	
	// border bottom-right
	points = gnome_canvas_points_new (3);
	points->coords[0] = button->desc->width;
	points->coords[1] = 0.0;
	points->coords[2] = button->desc->width;
	points->coords[3] = button->desc->height;
	points->coords[4] = 0.0;
	points->coords[5] = button->desc->height;
	
	button->border_se = gnome_canvas_item_new (GNOME_CANVAS_GROUP(button->root_grp),
					gnome_canvas_line_get_type(),
					"points", points,
					"fill-color", button->desc->border_color,
					NULL);
	gnome_canvas_points_free (points);
	
	// border top-left
	points = gnome_canvas_points_new (3);
	points->coords[0] = 0.0;
	points->coords[1] = button->desc->height;
	points->coords[2] = 0.0;
	points->coords[3] = 0.0;
	points->coords[4] = button->desc->width;
	points->coords[5] = 0.0;
	
	button->border_nw = gnome_canvas_item_new (GNOME_CANVAS_GROUP(button->root_grp),
					gnome_canvas_line_get_type(),
					"points", points,
					"fill-color", button->desc->shadow_color,
					NULL);
	gnome_canvas_points_free (points);
	
	// --------------------------------------------------------------------
	// Post-initialization.
	// --------------------------------------------------------------------
	// init borders for property plastic_on_motion
	if (button->desc->plastic_on_motion) 
	{
		button->border_shown = false; 
		canvas_button_border_hide (button);
	}
	else button->border_shown = true; // by default
	
	// set border visibility
	if (!button->desc->border_show)
	{
		button->border_shown = false;
		canvas_button_border_hide (button);
	}
	
	return button;				
}
void canvas_button_destroy (CanvasButton *button)
{
	g_free (button->desc->label_text);
	gtk_object_destroy (GTK_OBJECT (button->root_grp));
	g_free (button->desc);
	
	if (button->tip != NULL)
		canvas_tip_destroy (button->tip);
	if (button->tip_desc != NULL)
		
		g_free (button->tip_desc);
	
	g_free (button);
}

void canvas_button_move_at (CanvasButton *button, double x, double y)
{
	// --------------------------------------------------------------------
	// Moves component to [x,y] coords, relative to its parent
	// --------------------------------------------------------------------
	double x1, y1, x2, y2;
	gnome_canvas_item_get_bounds (button->root_grp, &x1, &y1, &x2, &y2);
	gnome_canvas_item_move (button->root_grp, -x1, -y1);
	gnome_canvas_item_move (button->root_grp, x, y);
	
	button->x = x;
	button->y = y;
}
void canvas_button_hide (CanvasButton *button)
{
	gnome_canvas_item_hide (button->root_grp);
	button->hidden = false;
}
void canvas_button_show (CanvasButton *button)
{
	gnome_canvas_item_show (button->root_grp);
	button->hidden = true;
}
void canvas_button_set_text (CanvasButton *button, char *text)
{
	// --------------------------------------------------------------------
	// Sets text of the button.
	// For a reason gnome_canvas_item_get_bounds doesn't work, if the borders
	// of the button are invisible. Therefore, we set it visible if necessary,
	// then get item's bounds and we hide it again (if necessary).
	// --------------------------------------------------------------------
	button->desc->label_text = g_strdup_printf ("%s", text);
	
	double x, y, xx, yy, ww, hh;
	// get bounds of button, borders must be visible (see method comment)
	if (!button->border_shown) { gnome_canvas_item_show (button->border_nw); gnome_canvas_item_show (button->border_se); }
	gnome_canvas_item_get_bounds (button->root_grp, &x, &y, &xx, &yy);
	if (!button->border_shown) { gnome_canvas_item_hide (button->border_nw); gnome_canvas_item_hide (button->border_se); }
	ww = xx - x; hh = yy - y;
	
	//g_print ("x,y, xx,yy, ww, hh, x+ww/2, y+hh/2: %4.1f, %4.1f, %4.1f, %4.1f, %4.1f, %4.1f, %4.1f, %4.1f\n", x, y, xx, yy, ww, hh, ww/2, hh/2);
	
	button->label = gnome_canvas_item_new (
							GNOME_CANVAS_GROUP (button->root_grp),
							gnome_canvas_text_get_type (),
							"x", ww/2, "y", hh/2,
							"text", text,
							"fill-color", button->desc->label_color,
							"size", button->desc->label_text_size,
							"anchor", GTK_ANCHOR_CENTER,
							"justification", GTK_JUSTIFY_CENTER,
							NULL);
}
void canvas_button_set_image (CanvasButton *button, GdkPixbuf *pic, GdkPixbuf *pic_clicked, GdkPixbuf *pic_disabled)
{
	// --------------------------------------------------------------------
	// Sets image of image button.
	// --------------------------------------------------------------------
	if (pic == NULL) g_error ("Error in function 'canvas_button_set_image' .. GdkPixbuf *pic == NULL!");
	
	if (button->image != NULL) gtk_object_destroy (GTK_OBJECT (button->image));
	button->image = gnome_canvas_item_new (
					GNOME_CANVAS_GROUP (button->root_grp),
					gnome_canvas_pixbuf_get_type (),
					"x", 0.0, "y", 0.0,
					"pixbuf", pic,
					"anchor", GTK_ANCHOR_NORTH_WEST,
					NULL);
	
	if (button->image_clicked != NULL) gtk_object_destroy (GTK_OBJECT (button->image_clicked));
	if (pic_clicked != NULL)
	{
		button->image_clicked = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (button->root_grp),
						gnome_canvas_pixbuf_get_type (),
						"x", 0.0, "y", 0.0,
						"pixbuf", pic_clicked,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						NULL);
		gnome_canvas_item_hide (button->image_clicked);
	}
	else
	{
		button->image_clicked = NULL;
	}
					
	if (button->image_disabled != NULL) gtk_object_destroy (GTK_OBJECT (button->image_disabled));
	if (pic_disabled != NULL)
	{
		button->image_disabled = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP (button->root_grp),
						gnome_canvas_pixbuf_get_type (),
						"x", 0.0, "y", 0.0,
						"pixbuf", pic_disabled,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						NULL);
		gnome_canvas_item_hide (button->image_disabled);
	}
	else
	{
		button->image_disabled = NULL;
	}
}
void canvas_button_set_icon (CanvasButton *button, GdkPixbuf *pic, GdkPixbuf *pic_disabled)
{
	// --------------------------------------------------------------------
	// Sets icon for canvas button.
	// --------------------------------------------------------------------
	if (pic == NULL)
	{
		g_warning ("Error in function 'canvas_button_set_image' .. GdkPixbuf *pic == NULL!");
		return;
	}
	if (button->image_icon != NULL) gtk_object_destroy (GTK_OBJECT (button->image_icon));
	
	// get center of the button
	double x, y, xx, yy, ww, hh;
	// get bounds of button, borders must be visible (see method comment for canvas_button_set_text ())
	if (!button->border_shown) { gnome_canvas_item_show (button->border_nw); gnome_canvas_item_show (button->border_se); }
	gnome_canvas_item_get_bounds (button->root_grp, &x, &y, &xx, &yy);
	if (!button->border_shown) { gnome_canvas_item_hide (button->border_nw); gnome_canvas_item_hide (button->border_se); }
	ww = xx - x; hh = yy - y;
	
	// create icon
	button->image_icon = gnome_canvas_item_new (
					GNOME_CANVAS_GROUP (button->root_grp),
					gnome_canvas_pixbuf_get_type (),
					"x", ww/2, "y", hh/2,
					"pixbuf", pic,
					"anchor", GTK_ANCHOR_CENTER,
					NULL);
	
	// create disabled icon
	if (pic_disabled == NULL)
	{
		// no icon for disabled button provided, we must stick with normal icon
		button->image_icon_disabled = NULL;
	}
	else
	{
		button->image_icon_disabled = gnome_canvas_item_new (
					GNOME_CANVAS_GROUP (button->root_grp),
					gnome_canvas_pixbuf_get_type (),
					"x", ww/2, "y", hh/2,
					"pixbuf", pic_disabled,
					"anchor", GTK_ANCHOR_CENTER,
					NULL);
		gnome_canvas_item_hide (button->image_icon_disabled);
	}
	
	button->has_icon = true;
	button->icon_visible = true;
}
void canvas_button_set_tooltip (CanvasButton *button, CanvasTipDesc* desc_tt)
{
	// --------------------------------------------------------------------
	// Sets button as not pressed.
	// --------------------------------------------------------------------
	button->tip_desc = desc_tt;
	button->has_tooltip = true;
	g_signal_connect (G_OBJECT(button->root_grp), "event", G_CALLBACK(OnCanvasButtonTooltipEvent), button);
}
void canvas_button_up (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Sets button as not pressed.
	// --------------------------------------------------------------------
	g_object_set (button->border_se, "fill-color", button->desc->border_color, NULL);
	g_object_set (button->border_nw, "fill-color", button->desc->shadow_color, NULL);
	
	if (button->desc->type == ButtonTypeImage)
	{
		if (button->image_clicked != NULL)
		{
			gnome_canvas_item_hide (button->image_clicked);
			gnome_canvas_item_show (button->image);
			
		}
	}
	
	button->pressed = false;
}
void canvas_button_down (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Sets button as pressed down.
	// --------------------------------------------------------------------
	g_object_set (button->border_se, "fill-color", button->desc->shadow_color, NULL);
	g_object_set (button->border_nw, "fill-color", button->desc->border_color, NULL);

	
	if (button->desc->type == ButtonTypeImage)
	{
		if (button->image_clicked != NULL)
		{
			gnome_canvas_item_show (button->image_clicked);
			gnome_canvas_item_hide (button->image);
			
		}
	}
	
	button->pressed = true;
}
void canvas_button_enable (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Enables disabled button.
	// --------------------------------------------------------------------
	if (button->enabled) return;
	button->enabled = true;
	if (button->pressed) canvas_button_down (button); else canvas_button_up (button);
	
	if (button->desc->label_text != NULL)
		g_object_set (button->label, "fill-color", button->desc->label_color, NULL);
	
	if (button->desc->type == ButtonTypeLabel) // LABEL Button
	{
		
		if ((button->desc->plastic_on_motion) && (!button->has_mouse_hover))
			canvas_button_border_hide (button);
		else
			canvas_button_border_show (button);
	}
	else if (button->desc->type == ButtonTypeImage) // IMAGE Button
	{
		if (button->image_disabled != NULL)
			gnome_canvas_item_hide (button->image_disabled);
		gnome_canvas_item_show (button->image);
	}
	
	if (button->has_icon && button->icon_visible)
	{
		if (button->image_icon_disabled != NULL)
		{
			gnome_canvas_item_hide (button->image_icon_disabled);
		}
		else
		{
			if (button->image_icon != NULL) gnome_canvas_item_show (button->image_icon);
		}
	}
}
void canvas_button_disable (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Disabled enabled button.
	// --------------------------------------------------------------------
	if (!button->enabled) return;
	button->enabled = false;
	button->pressed = false; // TODO possible conflicts
	
	if (button->desc->label_text != NULL)
		g_object_set (button->label, "fill-color", button->desc->label_color_alt, NULL);
	
	if (button->desc->type == ButtonTypeLabel) // LABEL Button
	{
		g_object_set (button->border_nw, "fill-color", button->desc->shadow_color, NULL);
		g_object_set (button->border_se, "fill-color", button->desc->shadow_color, NULL);
		canvas_button_border_show (button);
	}
	else if (button->desc->type == ButtonTypeImage) // IMAGE Button
	{
		gnome_canvas_item_hide (button->image);
		if (button->image_disabled != NULL)
			gnome_canvas_item_show (button->image_disabled);
	}
	
	if (button->has_icon)
	{
		if (button->image_icon_disabled != NULL)
		{
			gnome_canvas_item_show (button->image_icon_disabled);
		}
		else
		{
			// no icon for disabled image, just hide icon
			if (button->image_icon != NULL)	gnome_canvas_item_hide (button->image_icon);
		}
	}
}
void canvas_button_border_show (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Shows borders of the button.
	// --------------------------------------------------------------------
	gnome_canvas_item_show (button->border_nw);
	gnome_canvas_item_show (button->border_se);
	button->border_shown = true;
}
void canvas_button_border_hide (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Hides borders of the button.
	// --------------------------------------------------------------------
	gnome_canvas_item_hide (button->border_nw);
	gnome_canvas_item_hide (button->border_se);
	button->border_shown = false;
}

GnomeCanvasItem* canvas_button_get_parent (CanvasButton *button)
{
	// --------------------------------------------------------------------
	// Returns parent of the button. This reference should not be freed!
	// --------------------------------------------------------------------
	GnomeCanvasItem *parent;
	g_object_get (button->root_grp, "parent", &parent, NULL);
	return parent;
}

// EVENTS
gint OnCanvasButtonSwitchEvent (GnomeCanvasItem* button, GdkEvent* event, CanvasButton* canvas_button)
{
	// --------------------------------------------------------------------
	// This event toggles button.
	// If button is disabled, do nothing.
	// --------------------------------------------------------------------
	if (!canvas_button->enabled) return false;
	
	switch (event->type)
	{
		case GDK_BUTTON_PRESS:
		{
			//g_print ("Button clicked!\n");
			canvas_button_down (canvas_button);
			return false;
		}
		case GDK_BUTTON_RELEASE:
		{
			//g_print ("Button released!\n");
			canvas_button_up (canvas_button);
			return false;
		}
		default:
			return false;
	}
}
gint OnCanvasButtonEnterLeaveEvent (GnomeCanvasItem* button, GdkEvent* event, CanvasButton* canvas_button)
{
	// --------------------------------------------------------------------
	// This event highlights button as mouse enters/leaves it.
	// It has effect only if the button is enabled, however we need to keep
	// track of mouse enter/leave events even if it is disabled.
	// Note that this eventhandler always propagates this event further.
	// --------------------------------------------------------------------
	
	switch (event->type)
	{
		case GDK_ENTER_NOTIFY:
		{
			if (canvas_button->enabled)	canvas_button_border_show (canvas_button);
			canvas_button->has_mouse_hover = true;
			return false;
		}
		
		case GDK_LEAVE_NOTIFY:
		{
			if (canvas_button->enabled)	canvas_button_border_hide (canvas_button);
			canvas_button->has_mouse_hover = false;
			return false;
		}
		
		default:
			return false;
	}
}
gint OnCanvasButtonTooltipEvent (GnomeCanvasItem* button_item, GdkEvent* event, CanvasButton* button)
{
	// --------------------------------------------------------------------
	// Create/destroy tooltips on enter/leave notify.
	// --------------------------------------------------------------------
	if (!button->has_tooltip) return false;
	
	switch (event->type)
	{
		case GDK_ENTER_NOTIFY:
		{
			double event_x, event_y;
			event_x = event->crossing.x;
			event_y = event->crossing.y;
			GnomeCanvasItem *parent = canvas_button_get_parent (button);
			
			button->tip = canvas_tip_new (GNOME_CANVAS_GROUP (parent), button->tip_desc);
			
			double tx, ty;
			g_object_get (button->tip->root_grp, "x", &tx, "y", &ty, NULL);
			gnome_canvas_item_i2w (button->tip->root_grp, &tx, &ty);
			gnome_canvas_item_move (button->tip->root_grp, event_x - tx, event_y - ty + 18.0);
			
			break;
		}
		case GDK_LEAVE_NOTIFY:
		{
			canvas_tip_destroy (button->tip);
			button->tip = NULL;
			break;
		}
		default:
			return false;
	}
	
	return false;
}







