#include "canvas_tips.h"

CanvasTip* canvas_tip_new (GnomeCanvasGroup *parent, CanvasTipDesc* tip_desc)
{
	// --------------------------------------------------------------------
	// Creates CanvasTip.
	// --------------------------------------------------------------------
	// check for existence of content
	if (tip_desc->label == NULL && tip_desc->img == NULL)
	{
		g_warning ("canvas_tip_new: label == NULL && image == NULL");
		return NULL;
	}
	
	CanvasTip *tip;
	
	tip = g_new (CanvasTip, 1);
	tip->text = NULL;
	tip->image = NULL;
	
	// create tooltip root item
	tip->root_grp = gnome_canvas_item_new (
						parent,
						gnome_canvas_group_get_type (),
						"x", 0.0, "y", 0.0,
						NULL);
	
	// create text item					
	double x1, y1, x2, y2, t_h = 0, t_w = 0;
	if (tip_desc->label != NULL)
	{
		tip->text = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP(tip->root_grp),
						gnome_canvas_text_get_type (),
						"x", 0.0, "y", 0.0,
						"text", tip_desc->label,
						"fill-color", tip_desc->font_color, //TIP_FONT_COLOR,
						"size", tip_desc->font_size, //TIP_FONT_SIZE,
						"anchor", GTK_ANCHOR_CENTER,
						"justification", GTK_JUSTIFY_CENTER,
						NULL);
		// get bounding box and center text
		gnome_canvas_item_get_bounds (tip->text, &x1, &y1, &x2, &y2);
		t_w = x2 - x1; t_h = y2 - y1;
	}
	
	// create image
	double i_h = 0, i_w = 0;
	if (tip_desc->img != NULL)
	{
		tip->image = gnome_canvas_item_new (
						GNOME_CANVAS_GROUP(tip->root_grp),
						gnome_canvas_pixbuf_get_type (),
						"x", 0.0, "y", 0.0,
						"pixbuf", tip_desc->img,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						NULL);
		gnome_canvas_item_get_bounds (tip->image, &x1, &y1, &x2, &y2);
		i_w = x2 - x1; i_h = y2 - y1;
	}
	else tip->image = NULL;
	
	// get greater values
	t_w = max (t_w, i_w);
	t_h = max (t_h, i_h);
	
	// position text
	if (tip->text != NULL)
	gnome_canvas_item_move (tip->text, t_w/2 + tip_desc->padding_width, t_h/2 + tip_desc->padding_height);
	
	// create background rect item after the text was created, so we know size of text
	tip->background = gnome_canvas_item_new (
					GNOME_CANVAS_GROUP(tip->root_grp),
					gnome_canvas_rect_get_type (),
					"x1", 0.0, "y1", 0.0,
					"x2", t_w + 2*tip_desc->padding_width, //TIP_PADDING_WIDTH,
					"y2", t_h + 2*tip_desc->padding_height, //TIP_PADDING_HEIGHT,
					"fill-color", tip_desc->fill_color,// TIP_COLOR,
					//"outline-color", TIP_BORDER_COLOR,
					//"width-units", TIP_BORDER_THICK,
					NULL);
	gnome_canvas_item_lower (tip->background, 1);

	// --------------------------------------------------------------------
	// Draw outline (with or without plastic effect - enabled by default)
	// --------------------------------------------------------------------
	GnomeCanvasPoints *points;
	char* tip_border_color;
	
	points = gnome_canvas_points_new (3);
	points->coords[0] = 0.0;
	points->coords[1] = t_h + 2*tip_desc->padding_height;
	points->coords[2] = t_w + 2*tip_desc->padding_width;
	points->coords[3] = t_h + 2*tip_desc->padding_height;
	points->coords[4] = t_w + 2*tip_desc->padding_width;
	points->coords[5] = 0.0;
	
	gnome_canvas_item_new (GNOME_CANVAS_GROUP(tip->root_grp),
							gnome_canvas_line_get_type (),
							"points", points,
							"fill-color", tip_desc->border_color,
							"width-units", tip_desc->border_thick,
							NULL);
	gnome_canvas_points_free (points);
	
	
	points = gnome_canvas_points_new (3);
	points->coords[0] = 0.0;
	points->coords[1] = t_h + 2*tip_desc->padding_height;
	points->coords[2] = 0.0;
	points->coords[3] = 0.0;
	points->coords[4] = t_w + 2*tip_desc->padding_width;
	points->coords[5] = 0.0;
	
	if (tip_desc->border_plastic)
	{
		tip_border_color = tip_desc->border_color_alt;
	}
	else
	{
		tip_border_color = tip_desc->border_color;
	}
	
	gnome_canvas_item_new (GNOME_CANVAS_GROUP(tip->root_grp),
							gnome_canvas_line_get_type (),
							"points", points,
							"fill-color", tip_border_color,
							"width-units", tip_desc->border_thick,
							NULL);
	gnome_canvas_points_free (points);

	
	return tip;
}
void canvas_tip_destroy (CanvasTip *tip)
{
	// --------------------------------------------------------------------
	// Destroys CanvasTip and deallocates its memory.
	// --------------------------------------------------------------------
	if (tip == NULL) return;
	gtk_object_destroy (GTK_OBJECT (tip->root_grp));
	g_free (tip);
}


CanvasTipDesc*	canvas_tip_desc_new ()
{
	// --------------------------------------------------------------------
	// Creates new blank CanvasTipDesc.
	// --------------------------------------------------------------------
	CanvasTipDesc *desc;
	desc = g_new (CanvasTipDesc, 1);
	
	desc->label = NULL;
	desc->font_color = NULL;
	desc->fill_color = NULL;
	desc->border_color = NULL;
	desc->border_color_alt = NULL;
	
	return desc;
}
void			canvas_tip_desc_free (CanvasTipDesc *desc)
{
	// --------------------------------------------------------------------
	// Deallocates memory.
	// --------------------------------------------------------------------
	if (desc == NULL) return;
	
	if (desc->label == NULL) g_free (desc->label);
	if (desc->font_color == NULL) g_free (desc->font_color);
	if (desc->fill_color == NULL) g_free (desc->fill_color);
	if (desc->border_color == NULL) g_free (desc->border_color);
	if (desc->border_color_alt == NULL) g_free (desc->border_color_alt);
	
	g_free (desc);
}
CanvasTipDesc*	canvas_tip_desc_duplicate (CanvasTipDesc *desc)
{
	// --------------------------------------------------------------------
	// Allocates exact duplicate of the given CanvasTipDesc.
	// --------------------------------------------------------------------
	CanvasTipDesc* ndesc;
	
	ndesc = g_new (CanvasTipDesc, 1);
	
	ndesc->label = g_strdup_printf ("%s", desc->label);
	ndesc->font_color = g_strdup_printf ("%s", desc->font_color);
	ndesc->font_size = desc->font_size;
	
	ndesc->fill_color = g_strdup_printf ("%s", desc->fill_color);
	ndesc->border_color = g_strdup_printf ("%s", desc->border_color);
	ndesc->border_color_alt = g_strdup_printf ("%s", desc->border_color_alt);
	ndesc->border_thick = desc->border_thick;
	ndesc->border_plastic = desc->border_plastic;
	
	ndesc->padding_width = desc->padding_width;
	ndesc->padding_height = desc->padding_height;
	
	return ndesc;
}


