extern "C" {
#include "libgcompris/gcompris.h"
}

#include "../src/canvas.h"
#include "glib.h"
#include "glib/gstdio.h"

static GcomprisBoard *gcomprisBoard = NULL;
static GnomeCanvasGroup *boardRootItem = NULL;

// tmp items
static Canvas *tutorcanvas;
static GnomeCanvasItem *stvorec;

static void		 start_board (GcomprisBoard *agcomprisBoard);
static void		 end_board (void);
static gboolean	 is_our_board (GcomprisBoard *gcomprisBoard);
static void		 destroy_all_items(void);

static void		 draw_rectangle ();

static BoardPlugin menu_bp =
{
	NULL,
	NULL,
	"Read Tutor",
	"Reading tutor plugin.",
	"Martin Malar <martinmalar@gmail.com>",
	NULL,
	NULL,
	NULL,
	NULL,
	start_board,
	NULL,
	end_board,
	is_our_board,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

// FROM plugin.h
//typedef struct
//{
//  void *handle;			/* Filled in by gcompris */
//  char *filename;		/* Filled in by gcompris */
//  char *name;			/* The name that describes this board */
//  char *description;		/* The description that describes this board */
//  char *author;			/* The author of this board */
//
//  GcomprisInitBoard    init;	/* Called when the plugin is loaded */
//
//  void (*cleanup) (void);      	/* Called when gcompris exit */
//  void (*about) (void);		/* Show the about box */
//  void (*configure) (void);	/* Show the configuration dialog */
//
//  GcomprisStartBoard   start_board;
//  GcomprisPauseBoard   pause_board;
//  GcomprisEndBoard     end_board;
//
//  GcomprisIsOurBoard   is_our_board; /* Return 1 if the plugin can handle the board file */
//
//  GcomprisKeyPress     key_press;
//  GcomprisOK           ok;
//  GcomprisSetLevel     set_level;
//  GcomprisConfig       config;
//  GcomprisRepeat       repeat;
//
//  GcomprisConfigStart  config_start;
//  GcomprisConfigStop   config_stop;
//
//} BoardPlugin;


/*
 * Main entry point mandatory for each Gcompris's game
 * ---------------------------------------------------
 *
 */
GET_BPLUGIN_INFO(readtutorapi);
 
static void start_board (GcomprisBoard *agcomprisBoard)
{
	if (agcomprisBoard != NULL)
	{
		gcomprisBoard = agcomprisBoard;

		gcomprisBoard->level=1;
		gcomprisBoard->maxlevel=1;
		gcomprisBoard->sublevel=1;
		gcomprisBoard->number_of_sublevel=3; /* Go to next level after this number of 'play' */
		gc_score_start(SCORESTYLE_NOTE,
						gcomprisBoard->width - 220,
						gcomprisBoard->height - 50,
						gcomprisBoard->number_of_sublevel);
		gc_bar_set ((GComprisBarFlags) (NULL)); //(GC_BAR_LEVEL|GC_BAR_OK));
		
		// Load ReadTutor
		draw_rectangle ();
		g_debug ("Creating new canvas class for ReadTutor...");
		tutorcanvas = new Canvas (gcomprisBoard->canvas);
//		g_debug ("Loading story from file: %s", D_STORY_FILE);
//		tutorcanvas->LoadFromFile (D_STORY_FILE);
		g_debug ("Loading Canvas class done..");
		
		gc_cursor_set(GCOMPRIS_DEFAULT_CURSOR); //(GCOMPRIS_LINE_CURSOR);
		
		
		
		/*
		 * Few notes about threads in GCompris.
		 * 
		 * - with all 3 commands uncommented, GCompris crashes or freezes;
		 * - g_thread_init (NULL) crashes GCompris, if gcompris is running
		 *   with sounds;
		 * - g_thread_init (NULL) must be called for readtutor to work properly
		 *   if gcompris -m (with sounds muted) is running;
		 * - gdk_threads_enter crashes GCompris immediately;
		 * - gdk_threads_init must be called.
		 * 
		 * TODO find a solution to how to find out, whether g_thread_init needs
		 * to be executed. By default, I'll leave it off as I assume, gcompris
		 * will be run with sounds.
		 * 
		 */
		
		// This crashes/freezes GCompris
//		g_print ("Pred threads...\n");
//		g_thread_init (NULL);

//		g_print ("Po thread init (pred gdk_threads_init)\n");
		gdk_threads_init ();
//		g_print ("Po gdk_threads_init..\n");


		// This crashes/freezes GCompris
//		gdk_threads_enter ();
//		g_print ("Threads initialized..\n");
		
	}
}

static void end_board ()
{
	if (gcomprisBoard != NULL)
	{
		gc_score_end();
		destroy_all_items();
	}
	gcomprisBoard = NULL;
	gc_cursor_set(GCOMPRIS_DEFAULT_CURSOR);
}

static gboolean is_our_board (GcomprisBoard *gcomprisBoard)
{
	if (gcomprisBoard)
	{
		if (g_strcasecmp (gcomprisBoard->type, "readtutorapi") == 0)
		{
			/* Set the plugin entry */
			gcomprisBoard->plugin = &menu_bp;
	
			return TRUE;
		}
	}
	
	return FALSE;
}

static void destroy_all_items()
{
	if(boardRootItem != NULL)
		gtk_object_destroy (GTK_OBJECT(boardRootItem));
	boardRootItem = NULL;
	
	tutorcanvas->ClearBoard ();
}

static void draw_rectangle ()
{
	boardRootItem = GNOME_CANVAS_GROUP(
				     gnome_canvas_item_new (gnome_canvas_root(gcomprisBoard->canvas),
							    gnome_canvas_group_get_type (),
							    "x", (double) 0,
							    "y", (double) 0,
							    NULL));
							    
	stvorec = gnome_canvas_item_new (
					boardRootItem,
					gnome_canvas_rect_get_type (),
					"x1", 50.0,
					"y1", 50.0,
					"x2", 120.0,
					"y2", 150.0,
					"fill-color", "#AAAAFF",
					NULL);
}

