#include "canvas.h"


Canvas::Canvas (GnomeCanvas* gcanvas)
{
	PrintVersion ();
	if (D_SHOW_DEBUG) g_debug ("Canvas::Canvas(): Running..");
	
	// --------------------------------------------------------------------
	// Configuration
	// --------------------------------------------------------------------
	char* config_file = g_strconcat (g_get_home_dir (), G_DIR_SEPARATOR_S, D_PATH_RESX, G_DIR_SEPARATOR_S, D_CONFIG_FILE, NULL);
	Configure (config_file);
	LoadData ();
	
	// --------------------------------------------------------------------
	// Pre-initialization
	// --------------------------------------------------------------------
	canvas = gcanvas;
	tf_width = TEXTF_WIDTH - 2* TEXTF_PADDING;
	tf_height = TEXTF_HEIGHT - 2* TEXTF_PADDING_VERT;
	tf_words.resize (0);
	tf_swords.resize (0);
	tf_marks.resize (0);
	tf_words_selectable = true; // by default
	selected_word = -1;
	marked_word = -1;
	page = -1;
	story_id = -1;
	button_next = NULL; button_prev = NULL;
	tooltip = NULL;
	
	// GCompris specifics
	
	
	// --------------------------------------------------------------------
	// Loading graphics
	// --------------------------------------------------------------------
	// parameters: gchar* filename, GError **error
	btn_next = gdk_pixbuf_new_from_file (PIC_BTN_NEXT, NULL);
	btn_next_dis = gdk_pixbuf_new_from_file (PIC_BTN_NEXT_DIS, NULL);
	btn_prev = gdk_pixbuf_new_from_file (PIC_BTN_PREV, NULL);
	btn_prev_dis = gdk_pixbuf_new_from_file (PIC_BTN_PREV_DIS, NULL);
	button_std = gdk_pixbuf_new_from_file (PIC_BUTTON_STD, NULL);
	button_alt = gdk_pixbuf_new_from_file (PIC_BUTTON_ALT, NULL);
	button_dis = gdk_pixbuf_new_from_file (PIC_BUTTON_DIS, NULL);
	
	// Icons (kind of static stuff)
	char* tmp_file;
	// cancel button icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "button_unchecked.png", NULL);
	icon_cancel = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "button_unchecked_disabled.png", NULL);
	icon_cancel_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	// open button icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "directory-32x32.png", NULL);
	icon_open = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "directory-disabled-32x32.png", NULL);
	icon_open_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	// play button icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "multimedia-32x32.png", NULL);
	icon_play = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "multimedia-disabled-32x32.png", NULL);
	icon_play_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	// record button icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "mic-30-32x32.png", NULL);
	icon_record = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "mic-30-32x32.png", NULL); // note that disabled icon for microphone is the same as an active icon
	icon_record_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	// read icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "book-32x32.png", NULL);
	icon_read = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "book-disabled-32x32.png", NULL); // note that disabled icon for microphone is the same as an active icon
	icon_read_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	// dictate button icon
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "pencil-32x32.png", NULL);
	icon_dictate = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "pencil-disabled-32x32.png", NULL); // note that disabled icon for microphone is the same as an active icon
	icon_dictate_dis = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	
	// smileys
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_SMILEY1, NULL);
	smile1 = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_SMILEY2, NULL);
	smile2 = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_SMILEY3, NULL);
	smile3 = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_SMILEY4, NULL);
	smile4 = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_SMILEY5, NULL);
	smile5 = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	
	// other static images
	tmp_file = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, "pencilholder.jpg", NULL);
	info_background_image = gdk_pixbuf_new_from_file (tmp_file, NULL);
	g_free (tmp_file);
	
	
	GError *err = NULL;
	background_image = gdk_pixbuf_new_from_file (BACKGROUND_IMAGE_FILENAME, &err);
	if (err != NULL)
	{
		g_print ("Error: '%s'\n", err->message);
		g_error_free (err);
	}
	
	sound = NULL;
	
	// --------------------------------------------------------------------
	// Initialization
	// --------------------------------------------------------------------
	Init ();
	
	// --------------------------------------------------------------------
	// Cleanup
	// --------------------------------------------------------------------
	g_free (config_file);
}
Canvas::~Canvas ()
{
	if (SHOW_MESSAGES) g_print ("Canvas: Running destructor..");
	ClearBoard ();
	g_object_unref (btn_next);
	g_object_unref (btn_prev);
	g_object_unref (btn_next_dis);
	g_object_unref (btn_prev_dis);
	g_object_unref (button_std);
	g_object_unref (button_alt);
	g_object_unref (button_dis);
	g_object_unref (background_image);
	// icon open
	g_object_unref (icon_open);
	g_object_unref (icon_open_dis);
	g_object_unref (icon_cancel);
	g_object_unref (icon_cancel_dis);
	g_object_unref (icon_play);
	g_object_unref (icon_play_dis);
	g_object_unref (icon_record);
	g_object_unref (icon_record_dis);
	g_object_unref (icon_read);
	g_object_unref (icon_read_dis);
	g_object_unref (icon_dictate);
	g_object_unref (icon_dictate_dis);
	g_object_unref (smile1);
	g_object_unref (smile2);
	g_object_unref (smile3);
	g_object_unref (smile4);
	g_object_unref (smile5);
	
	g_object_unref (info_background_image);
	
	// TODO
	// free all allocated strings (for configuration)
	// free all other allocated data
}

// PUBLIC
gint Canvas::LoadFromFile (char *filePath)
{
	// --------------------------------------------------------------------
	// Uses Xml class to read contents of given file.
	// 
	// TODO: Error handling, checkings on file existence and validity,
	// check on what does getStoryWords() actually return.
	// --------------------------------------------------------------------
	char* fileName;
	
	fileName = g_strconcat (filePath, G_DIR_SEPARATOR_S, STORY_FILE, NULL);
	
	Xml xml(0);
	if (!xml.ReadFile(fileName))
	{
		if (SHOW_WARNINGS)
			g_warning ("Error opening file '%s'", fileName);
		return -1;
	}
	Story = xml.getStoryWords ();
	person_data = xml.GetPersonData ();
	
	// now load this structure into textfield
	LoadAuthorInfo (filePath);
	LoadPage (0); // this loads first page of text
	
	g_free (fileName);
	return 0;
}
void Canvas::PrintVersion ()
{
	// --------------------------------------------------------------------
	// Prints current version
	// --------------------------------------------------------------------
	g_print ("%s library v%s build %s", PACKAGE, VERSION, _BUILD);
	#if GCOMPRIS_API
		g_print (" - using GCompris API\n");
	#else
		g_print (" - using standard API\n");
	#endif	
	g_print ("%s - written by %s\n", COPYRIGHT, AUTHOR);
}
void Canvas::ClearBoard ()
{
//	if (SHOW_DEBUG) g_debug ("Canvas: ClearBoard () - clearing all ReadTutor data..");
	
	ClearTextfield ();
	gtk_object_destroy (GTK_OBJECT(board));
	gtk_object_destroy (GTK_OBJECT(board_border));
	sound = NULL;
}

// PROPERTIES
gchar* Canvas::GetVersion ()
{
	// --------------------------------------------------------------------
	// Returns current version
	// --------------------------------------------------------------------
	return VERSION;
}
GnomeCanvas* Canvas::GetCanvas ()
{
	// --------------------------------------------------------------------
	// Provides canvas after it has been initialized
	// --------------------------------------------------------------------
	return canvas;
}
GnomeCanvasItem* Canvas::GetTextField ()
{
	// --------------------------------------------------------------------
	// Returns group for textfield to use with events
	// --------------------------------------------------------------------
	return (GnomeCanvasItem*) textfield;
}

// PRIVATE
void Canvas::Configure (char* configFileName)
{
	// --------------------------------------------------------------------
	// Reads configuration file
	// --------------------------------------------------------------------
	if (D_SHOW_DEBUG) g_debug ("Canvas::Configure running..");
	ConfigHandler config (configFileName);
	
	// system
	SHOW_DEBUG = config.ShowDebug ();
	SHOW_MESSAGES = config.ShowMessages ();
	SHOW_WARNINGS = config.ShowWarnings ();
	SHOW_ERRORS = config.ShowErrors ();
	
	// paths, files
	PATH_RESX = config.PathResx ();
	GLADE_FILE = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, config.GladeFile (), NULL);
	PRG_ICON = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, config.PrgIcon (), NULL);
	CONFIG_FILE = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, config.ConfigFile (), NULL);
	PIC_OPEN_BOOK = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_OPEN_BOOK, NULL);
	PIC_BTN_NEXT = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BTN_NEXT, NULL);
	PIC_BTN_NEXT_DIS = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BTN_NEXT_DIS, NULL);
	PIC_BTN_PREV = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BTN_PREV, NULL);
	PIC_BTN_PREV_DIS = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BTN_PREV_DIS, NULL);
	BACKGROUND_IMAGE_FILENAME = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, config.PicBackgroundImage (), NULL);
	PIC_BUTTON_STD = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BUTTON_STD, NULL);
	PIC_BUTTON_ALT = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BUTTON_ALT, NULL);
	PIC_BUTTON_DIS = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, D_PATH_IMG, G_DIR_SEPARATOR_S, D_PIC_BUTTON_DIS, NULL);
	
	STORY_FILE = config.StoryFile ();
	PICTURE_FILE = config.PictureFile ();

	// canvas
	CANVAS_ANTI_ALIASED = config.CanvasAntiAliased ();
	CANVAS_WIDTH = config.CanvasWidth ();
	CANVAS_HEIGHT = config.CanvasHeight ();
	CANVAS_BORDER = config.CanvasBorder ();
	CANVAS_BORDER_COLOR = config.CanvasBorderColor ();
	CANVAS_BACKGROUND_COLOR = config.CanvasBackgroundColor ();
	
	CANVAS_GRID_SHOW = config.CanvasGridShow ();
	CANVAS_GRID_COLOR = config.CanvasGridColor ();
	CANVAS_GRID_THICK = config.CanvasGridThick ();
	
	// textfield
	TEXTF_DOCK = config.TextfDock ();
	
	TEXTF_DOCK_TOP = config.TextfDockTop ();
	TEXTF_DOCK_BOTTOM = config.TextfDockBottom ();
	TEXTF_DOCK_RIGHT = config.TextfDockRight ();
	TEXTF_DOCK_TOP_MARGIN = config.TextfDockTopMargin ();
	TEXTF_DOCK_BOTTOM_MARGIN = config.TextfDockBottomMargin ();
	TEXTF_DOCK_RIGHT_MARGIN = config.TextfDockRightMargin ();
	
	TEXTF_WIDTH = config.TextfWidth ();
	TEXTF_HEIGHT = config.TextfHeight ();
	TEXTF_X = config.TextfX ();
	TEXTF_Y = config.TextfY ();
//	if (TEXTF_DOCK)
//		TEXTF_HEIGHT = CANVAS_HEIGHT - 2* TEXTF_Y;
//	else
//		TEXTF_HEIGHT = config.TextfHeight ();
//	if (TEXTF_DOCK)
//		TEXTF_X = CANVAS_WIDTH - (TEXTF_Y + TEXTF_WIDTH);
//	else
//		TEXTF_X = config.TextfX ();
	TEXTF_PADDING = config.TextfPadding ();
	TEXTF_PADDING_VERT = config.TextfPaddingVert ();
	TEXTF_BORDER = config.TextfBorder ();
	TEXTF_BORDER_COLOR = config.TextfBorderColor ();
	TEXTF_BORDER_SHOW = config.TextfBorderShow ();
	TEXTF_INNER_SHOW = config.TextfInnerShow ();
	
		// handle docking
		if (TEXTF_DOCK)
		{
			// top
			if (TEXTF_DOCK_TOP)
			{
				TEXTF_Y = TEXTF_DOCK_TOP_MARGIN;
			}
			// bottom
			if (TEXTF_DOCK_BOTTOM)
			{
				TEXTF_HEIGHT = CANVAS_HEIGHT - TEXTF_DOCK_TOP_MARGIN - TEXTF_DOCK_BOTTOM_MARGIN;
			}
			// right-left
			if (TEXTF_DOCK_RIGHT)
			{
				TEXTF_X = CANVAS_WIDTH - TEXTF_WIDTH - TEXTF_DOCK_RIGHT_MARGIN;
			}
		}
	if (SHOW_DEBUG) g_debug (
	"Textfield attributes are as follows:\n"
	"X, Y: %4.1f, %4.1f\n"
	"W, H: %4.1f, %4.1f",
	TEXTF_X, TEXTF_Y, TEXTF_WIDTH, TEXTF_HEIGHT
	);
	
	// (author) info
	INFO_X = config.InfoX ();
	INFO_Y = config.InfoY ();
	INFO_WIDTH = config.InfoWidth ();
	INFO_HEIGHT = config.InfoHeight ();
	INFO_PADDING = config.InfoPadding ();
	INFO_DOCK = config.InfoDock ();
	INFO_DOCK_BOTTOM = config.InfoDockBottom ();
	INFO_DOCK_RIGHT = config.InfoDockRight ();
	INFO_DOCK_LEFT = config.InfoDockLeft ();
	INFO_BORDER = config.InfoBorder ();
	INFO_BORDER_SHOW = config.InfoBorderShow ();
	INFO_BORDER_COLOR = config.InfoBorderColor ();
	
	// text
	TEXT_FAMILY = config.TextFamily ();
	TEXT_SIZE = config.TextSize ();
	TEXT_SIZE_HEADING = config.TextSizeHeading ();
	TEXT_SPACING_VERT = config.TextSpacingVert ();
	TEXT_SPACING_HORIZ = config.TextSpacingHoriz ();
	TEXT_HEADLINE_CENTER = config.TextHeadlineCenter ();
	TEXT_PARAGRAPH_INDEND = D_TEXT_PARAGRAPH_INDEND; // always use default // config.TextParagraphIndend ();
	
	// font colors
	FONT_COLOR = config.FontColor ();
	FONT_HLT_COLOR = config.FontHltColor ();
	FONT_SELECT_COLOR = config.FontSelectColor ();
	FONT_BCOLOR = config.FontBColor ();
	FONT_HLT_BCOLOR = config.FontHltBColor ();
	FONT_SELECT_BCOLOR = config.FontSelectBColor ();
	FONT_SELECT_HLT_COLOR = config.FontSelectHltColor ();
	FONT_SELECT_HLT_BCOLOR = config.FontSelectHltBColor ();
	
	// mark colors
	MARK_COLOR_NONE = D_MARK_COLOR_NONE;
	MARK_COLOR_CURRENT = D_MARK_COLOR_CURRENT;
	MARK_COLOR_PASSED = D_MARK_COLOR_PASSED;
	
	// font aa colors
	FONT_COLOR_AA = config.FontColorAA ();
	FONT_HLT_COLOR_AA = config.FontHltColorAA ();
	FONT_BCOLOR_AA = config.FontBColorAA ();
	FONT_HLT_BCOLOR_AA = config.FontHltBColorAA ();
	
	// tooltips
	TIP_SHOW_TOOLTIPS = config.TipShowTooltips ();
	TIP_COLOR = config.TipColor ();
	TIP_BORDER_COLOR = config.TipBorderColor ();
	TIP_BORDER_COLOR_ALT = config.TipBorderColorAlt ();
	TIP_FONT_COLOR = config.TipFontColor ();
	TIP_BORDER_THICK = config.TipBorderThick ();
	TIP_FONT_SIZE = config.TipFontSize ();
	TIP_PADDING_WIDTH = config.TipPaddingWidth ();
	TIP_PADDING_HEIGHT = config.TipPaddingHeight ();
	TIP_BORDER_PLASTIC = config.TipBorderPlastic ();
	
	// buttons
	BUTTON_BORDER_WIDTH = config.ButtonBorderWidth ();
	BUTTON_BORDER_COLOR = config.ButtonBorderColor ();
	BUTTON_SHADOW_COLOR = config.ButtonShadowColor ();
	BUTTON_BACKGROUND_COLOR = config.ButtonBackgroundColor ();
	BUTTON_TEXT_SIZE = config.ButtonTextSize ();
	BUTTON_TEXT_COLOR = config.ButtonTextColor ();
	BUTTON_TEXT_COLOR_ALT = config.ButtonTextColorAlt ();
	BUTTON_PLASTIC_ON_MOTION = D_BUTTON_PLASTIC_ON_MOTION;
	BUTTON_WIDTH = config.ButtonWidth ();
	BUTTON_HEIGHT = config.ButtonHeight ();
	BUTTON_WIDTH_ICON = D_BUTTON_WIDTH_ICON;
	BUTTON_HEIGHT_ICON = D_BUTTON_HEIGHT_ICON;
	BUTTON_SHOW_IMAGE_BUTTONS = config.ButtonShowImageButtons ();
	BUTTON_SHOW_IMG_LABELS = config.ButtonShowImgLabels ();
	
	// listbox
	LB_X = config.LbX ();
	LB_Y = config.LbY ();
	LB_WIDTH = config.LbWidth ();
	LB_HEIGHT = config.LbHeight ();
	LB_TEXT_SIZE = config.LbTextSize ();
	LB_BORDER_WIDTH = config.LbBorderWidth ();
	LB_TEXT_COLOR = config.LbTextColor ();
	LB_BORDER_COLOR = config.LbBorderColor ();
	LB_PADDING = config.LbPadding ();
	LB_BTN_CANCEL_X = config.LbBtnCancelX ();
	LB_BTN_CANCEL_Y = config.LbBtnCancelY ();

//	LB_X = D_LB_X;
//	LB_Y = D_LB_Y;
//	LB_WIDTH = D_LB_WIDTH;
//	LB_HEIGHT = D_LB_HEIGHT;
//	LB_TEXT_SIZE = D_LB_TEXT_SIZE;
//	LB_BORDER_WIDTH = D_LB_BORDER_WIDTH;
//	LB_TEXT_COLOR = D_LB_TEXT_COLOR;
//	LB_BORDER_COLOR = D_LB_BORDER_COLOR;
//	LB_PADDING = D_LB_PADDING;
//	LB_BTN_CANCEL_X = D_LB_BTN_CANCEL_X;
//	LB_BTN_CANCEL_Y = D_LB_BTN_CANCEL_Y;

	if (SHOW_DEBUG) g_debug ("Canvas::Configure finishing...");
}
ErrorLoadData Canvas::LoadData ()
{
	// --------------------------------------------------------------------
	// Scans resources directory for available articles.
	// --------------------------------------------------------------------
	// resource directory is temporarily PATH_RESX
	GDir *resx_handle;
	GError *errGDir = NULL;
	gchar *full_path;
	const gchar *file_name;
	struct stat st;
	StoryInfo* si;
	
	resx_handle = g_dir_open (PATH_RESX, 0, &errGDir);
	if (errGDir)
	{
		if (SHOW_WARNINGS) g_warning ("Failed to open Resources directory '%s' with message: %s", PATH_RESX, errGDir->message);
		g_error_free (errGDir);
		return eLoadDataFailedOpenDir;
	}
		
	// first, get list of all directories in PATH_RESX
	file_name = g_dir_read_name (resx_handle);
	while (file_name != NULL)
	{
		full_path = g_strconcat (PATH_RESX, G_DIR_SEPARATOR_S, file_name, NULL);
		if (g_stat (full_path, &st) == -1) { if (SHOW_WARNINGS) g_warning ("Error accessing to file '%s'", full_path); }
		else
		{
			if (st.st_mode&S_IFDIR) // if it is directory
			{
				Xml xml(0);
				si = xml.CheckDirectory (full_path, (gchar*) file_name);
				if (si->valid)
				{
					// add this storyinfo into story_list vector (TODO)
					story_list.push_back (si);
					if (SHOW_DEBUG) g_debug ("%s - %s %s: %s", file_name, si->name, si->surname, si->story_name);
				}
				else
				{
					if (SHOW_DEBUG) g_debug ("%s - failed to validate", file_name);
					g_free (si);
				}
			}
		}
		file_name = g_dir_read_name (resx_handle);
	}
	
	g_dir_close (resx_handle);
	
	// tmp at the end, output stored story infos
//	g_print ("Zoznam pribehov:\n");
//	int i;
//	for (i = 0; i < (int) story_list.size(); i++)
//	{
//		g_print ("%s %s - '%s' (%s)\n", story_list[i]->name, story_list[i]->surname, story_list[i]->story_name, story_list[i]->file_name);
//	}
	
	return eLoadDataOk;
}
void Canvas::Init()
{
	// --------------------------------------------------------------------
	// Initialize Gnome Canvas
	// --------------------------------------------------------------------
	if (SHOW_DEBUG) g_debug ("Canvas::Init(): Running...");
	
//	if (CANVAS_ANTI_ALIASED)
//	{
//		gtk_widget_push_visual(gdk_rgb_get_visual());
//		gtk_widget_push_colormap(gdk_rgb_get_cmap());
//		canvas = GNOME_CANVAS (gnome_canvas_new_aa());
//		gtk_widget_pop_colormap();
//		gtk_widget_pop_visual();
//	}
//	else
//	{
//		gtk_widget_push_visual(gdk_imlib_get_visual());
//		//gtk_widget_push_colormap(gdk_imlib_get_colormap()); // doesn't seem to work
//		canvas = GNOME_CANVAS (gnome_canvas_new());
//		gtk_widget_pop_visual();
//		//gtk_widget_pop_colormap();
//	}
	
	gnome_canvas_set_scroll_region (canvas, 
									(double) 0.0,
									(double) 0.0,
									(double) (CANVAS_WIDTH + 2*CANVAS_BORDER),
									(double) (CANVAS_HEIGHT+ 2*CANVAS_BORDER));
	
	// sets minimum window size (use only in vanilla)
	#if !GCOMPRIS_API
		gtk_widget_set_size_request (GTK_WIDGET(canvas),
	 								(gint) (CANVAS_WIDTH + 2*CANVAS_BORDER),
	 								(gint) (CANVAS_HEIGHT+ 2*CANVAS_BORDER));
	#endif
	
	// this connects event signal to the whole gnomecanvas widget
	g_signal_connect (G_OBJECT(canvas), "event", G_CALLBACK(OnCanvasEvent), this);
	
	DrawBackground ();
	DrawTextfield ();
	DrawAuthorInfo ();
	
	if (SHOW_DEBUG) g_debug ("Init: finished - GnomeCanvas was created.");
	
	// test of string-distance function
//	string s1 = "ktory";
//	string s2 = "tory";
//	int dist = sdist (s1, s2);
//	int size = g_utf8_strlen (s1.c_str(), -1);
//	double perc = (double) dist / (double) size * 100.0;
//	perc = min (perc, 100.0);
//	perc = 100.0 - perc;
//	cout << "Dist ( '" << s1 << "'("<<size<<"), '" << s2 << "' ) = " << dist << " (" << perc << "%)" << endl;
	
	// using pre-processor directives (test of negation)
//	#define MOJ_DEFINE true
//	#if !MOJ_DEFINE
//	g_print ("We have our define! (evaluated to true)\n");
//	#else
//	g_print ("We are 'out_define-free'! (evaluated to false)\n");
//	#endif
	
}
void Canvas::DrawBackground()
{
	// --------------------------------------------------------------------
	// Initializes canvas' background
	// --------------------------------------------------------------------
	if (SHOW_DEBUG) g_debug ("Canvas::DrawBackground(): Running...");
	
	// draws border rectangle
	board_border = gnome_canvas_item_new(gnome_canvas_root(canvas),
				gnome_canvas_rect_get_type(),
				"x1", 0.0,
				"y1", 0.0,
				"x2", CANVAS_WIDTH + 2*(double)CANVAS_BORDER,
				"y2", CANVAS_HEIGHT+ 2*(double)CANVAS_BORDER,
				"fill_color", CANVAS_BORDER_COLOR,
				NULL);
	
	
	// inserts group for a board
	board = GNOME_CANVAS_GROUP(
			gnome_canvas_item_new(gnome_canvas_root(canvas),
				gnome_canvas_group_get_type(),
				"x", (double) CANVAS_BORDER,
				"y", (double) CANVAS_BORDER,
				NULL));
	
	// draw background pattern
	double img_w, img_h, canvas_w, canvas_h;
	double x, y;
	img_w = gdk_pixbuf_get_width (background_image);
	img_h = gdk_pixbuf_get_height (background_image);
	canvas_w = CANVAS_WIDTH + 2*((double)CANVAS_BORDER);
	canvas_h = CANVAS_HEIGHT + 2*((double)CANVAS_BORDER);
	
	y = 0;
	while (y < canvas_h)
	{
		x = 0;
		while (x < canvas_w)
		{
			gnome_canvas_item_new (board, //gnome_canvas_root (canvas),
						gnome_canvas_pixbuf_get_type (),
						"x", x, "y", y,
						"pixbuf", background_image,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						NULL);
			
			x += img_w;
		}
		y += img_h;
	}
	
	// draws working board (white rect - disabled as background is now image pattern)
//	gnome_canvas_item_new(board,
//				gnome_canvas_rect_get_type(),
//				"x1", 0.0,
//				"y1", 0.0,
//				"x2", CANVAS_WIDTH,
//				"y2", CANVAS_HEIGHT,
//				"fill_color", CANVAS_BACKGROUND_COLOR,
//				NULL);
	
	// create picture of the open book (textfield background)
	GdkPixbuf* open_book_pic = gdk_pixbuf_new_from_file (PIC_OPEN_BOOK, NULL);
	gnome_canvas_item_new (board,
				gnome_canvas_pixbuf_get_type(),
				"x", TEXTF_X, "y", TEXTF_Y,
				"width", TEXTF_WIDTH, "width-set", true,
				"height", TEXTF_HEIGHT, "height-set", true,
				"pixbuf", open_book_pic,
				"anchor", GTK_ANCHOR_NORTH_WEST,
				NULL);
	g_object_unref (open_book_pic);
	
	// indent rulers (every 100 pixels)
	int i;
	if (CANVAS_GRID_SHOW)
	{
		for (i = 10; i < CANVAS_WIDTH; i += 100)
		{
			DrawSingleLine (board,i, 1,
							i, (int) CANVAS_HEIGHT - 1,
							CANVAS_GRID_COLOR, CANVAS_GRID_THICK);
		}
		for (i = 10; i < CANVAS_HEIGHT; i+= 100)
		{
			DrawSingleLine (board,1, i,
							(int) CANVAS_WIDTH - 1, i,
							CANVAS_GRID_COLOR, CANVAS_GRID_THICK);
		}
	}
	
	
	// --------------------------------------------------------------------
	// BUTTONS:
	// --------------------------------------------------------------------
	
	// button for loading articles 'Open button'
	double top_padding = 10.0;	// buffer space from the top
	double vert_padding = 0.0; // buffer space in between buttons
//	btnLoadStory = ButtonNewOnBoard ("Otvoriť..", 10.0, 10.0);
	btnLoadStory = ButtonImageIconNew ("Otvoriť..", button_std, button_alt, icon_open, icon_open_dis, 10.0, 0*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT(btnLoadStory->root_grp), "event", G_CALLBACK(OnLoadStoryClick), this);
	// disable this button, if no articles were detected
	if (story_list.size () == 0)
		canvas_button_disable (btnLoadStory);

	// draw listbox
	lb_select = ListboxSelectNew (LB_X, LB_Y);
	g_signal_connect (G_OBJECT(lb_select->root_grp), "event", G_CALLBACK(OnLstBoxMouseUp), this);
	canvas_listbox_hide (lb_select);
	// fill in listbox
	if (story_list.size() > 0)
	for (i = 0; i < (int) story_list.size(); i++)
		canvas_listbox_insert (lb_select, story_list[i]->story_name, story_list[i]->file_name, i);
	
	// draw listbox cancel button
//	btnLoadCancel = ButtonNewOnBoard ("Zrušiť", LB_BTN_CANCEL_X, LB_BTN_CANCEL_Y);
	btnLoadCancel = ButtonImageIconNew ("Zrušiť", button_std, button_alt, icon_cancel, icon_cancel_dis, LB_BTN_CANCEL_X, LB_BTN_CANCEL_Y); // 10, 225 (button props: 190x48)
	g_signal_connect (G_OBJECT(btnLoadCancel->root_grp), "event", G_CALLBACK(OnLoadCancelClick), this);
	canvas_button_hide (btnLoadCancel);
	
	// play button
//	btnPlay = ButtonNewOnBoard ("Prehrať", 10.0, 39.0); // 39 = 10 + 24 (vyska tlacitka) + 5 (medzera)
	btnPlay = ButtonImageIconNew ("Prehrať", button_std, button_alt, icon_play, icon_play_dis, 10.0, 1*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT(btnPlay->root_grp), "event", G_CALLBACK(OnPlayClick), this);
	canvas_button_disable (btnPlay);
	// play cancel button
//	btnPlayCancel = ButtonNewOnBoard ("Zastaviť", 10.0, 39.0); // ta ista pozicia ako btnPlay
	btnPlayCancel = ButtonImageIconNew ("Zastaviť", button_std, button_alt, icon_cancel, icon_cancel_dis, 10.0, 1*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT(btnPlayCancel->root_grp), "event", G_CALLBACK(OnPlayCancelClick), this);
	canvas_button_hide (btnPlayCancel);
	
	// push button
//	btnPush = ButtonNewOnBoard ("Nahrať", 10.0, 68.0); // 68 = 39 + 24 + 5 (posledne tlacitko + vyska tlacitka + medzera)
	btnPush = ButtonImageIconNew ("Nahrať", button_std, button_alt, icon_record, icon_record_dis, 10.0, 2*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT (btnPush->root_grp), "event", G_CALLBACK(OnPushMouseDown), this);
	g_signal_connect (G_OBJECT (btnPush->root_grp), "event", G_CALLBACK(OnPushMouseUp), this);
	canvas_button_disable (btnPush);
	
	// read button
	btnRead = ButtonImageIconNew ("Prečítať", button_std, button_alt, icon_read, icon_read_dis, 10.0, 3*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT (btnRead->root_grp), "event", G_CALLBACK(OnReadMouseUp), this);
	canvas_button_disable (btnRead);
	// read cancel button
	btnReadCancel = ButtonImageIconNew ("Zastaviť", button_std, button_alt, icon_cancel, icon_cancel_dis, 10.0, 3*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT (btnReadCancel->root_grp), "event", G_CALLBACK(OnPlayCancelClick), this);
	canvas_button_hide (btnReadCancel);
	
	
	// dictate button
	btnDictate = ButtonImageIconNew ("Diktát", button_std, button_alt, icon_dictate, icon_dictate_dis, 10.0, 4*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	g_signal_connect (G_OBJECT (btnDictate->root_grp), "event", G_CALLBACK(OnDictateMouseUp), this);
	canvas_button_disable (btnDictate);
	// dictate cancel button
	btnDictateCancel = ButtonImageIconNew ("Zastaviť", button_std, button_alt, icon_cancel, icon_cancel_dis, 10.0, 4*(BUTTON_HEIGHT_ICON+vert_padding) + top_padding);
	canvas_button_hide (btnDictateCancel);
	g_signal_connect (G_OBJECT (btnDictateCancel->root_grp), "event", G_CALLBACK(OnDictateCancelClick), this);
	
	// TMP - color palette
	double xx = 250;
	double yy = 460;
	double ww = 50;
	double hh = 50;
	i = 0;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_0, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_20, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_30, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_40, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_50, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_60, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_70, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_80, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_90, NULL); i++;
	gnome_canvas_item_new (board, gnome_canvas_rect_get_type (),
	"x1", xx + i*ww, "y1", yy, "x2", xx + (i+1)*ww, "y2", yy+hh,
	"fill-color", MM_COLOR_95, NULL); i++;
	
}
void Canvas::DrawTextfield ()
{
	// --------------------------------------------------------------------
	// Draws textfield outline
	// --------------------------------------------------------------------
	// insert textfield group
	textf_outline = GNOME_CANVAS_GROUP (
				gnome_canvas_item_new (board,
					gnome_canvas_group_get_type(),
					"x", TEXTF_X,
					"y", TEXTF_Y,
					NULL));
	
	// Draw Outline
	if (TEXTF_BORDER_SHOW)
	{
		// create points
		GnomeCanvasPoints *points;
		points = gnome_canvas_points_new (5);
		// fill in points
		points->coords[0] = 0.0;
		points->coords[1] = 0.0;
		points->coords[2] = TEXTF_WIDTH;
		points->coords[3] = 0.0;
		points->coords[4] = TEXTF_WIDTH;
		points->coords[5] = TEXTF_HEIGHT;
		points->coords[6] = 0.0;
		points->coords[7] = TEXTF_HEIGHT;
		points->coords[8] = 0.0;
		points->coords[9] = 0.0;
		// create item
		gnome_canvas_item_new (textf_outline,
							gnome_canvas_line_get_type(),
							"points", points,
							"fill-color", TEXTF_BORDER_COLOR,
							"width-units", TEXTF_BORDER,
							NULL);
		gnome_canvas_points_free (points);
	}
	
	// Insert textfield itself
	textfield = GNOME_CANVAS_GROUP (
				gnome_canvas_item_new (textf_outline,
					gnome_canvas_group_get_type(),
					"x", 0.0 + TEXTF_PADDING,
					"y", 0.0 + TEXTF_PADDING_VERT,
					NULL));
	
	if (TEXTF_INNER_SHOW)
	{				
		// create points
		GnomeCanvasPoints *points;
		points = gnome_canvas_points_new (5);
		// fill in points
		points->coords[0] = 0.0;
		points->coords[1] = 0.0;
		points->coords[2] = tf_width;
		points->coords[3] = 0.0;
		points->coords[4] = tf_width;
		points->coords[5] = tf_height;
		points->coords[6] = 0.0;
		points->coords[7] = tf_height;
		points->coords[8] = 0.0;
		points->coords[9] = 0.0;
		// create item
		gnome_canvas_item_new (textfield,
							gnome_canvas_line_get_type(),
							"points", points,
							"fill-color", TEXTF_BORDER_COLOR,
							"width-units", TEXTF_BORDER,
							NULL);
		gnome_canvas_points_free (points);
	}
	
	// insert NEXT/PREV buttons to handle pages (page buttons)
	if (BUTTON_SHOW_IMAGE_BUTTONS)
	{
		// Image buttons
		// corners
//		button_next = ButtonImageNew (corner_right_turned, corner_right, tf_width-50.0, 0.0);
//		g_signal_connect (G_OBJECT(button_next->root_grp), "event", G_CALLBACK(OnNextPageMouseUp), this);
//		canvas_button_disable (button_next);
//		button_prev = ButtonImageNew (corner_left_turned, corner_left, 0.0, 0.0);
//		g_signal_connect (G_OBJECT(button_prev->root_grp), "event", G_CALLBACK(OnPrevPageMouseUp), this);
//		canvas_button_disable (button_prev);
		// forward/backward buttons
		button_next = ButtonImageNew (btn_next, NULL, btn_next_dis, tf_width-50.0, 0.0);
		g_signal_connect (G_OBJECT(button_next->root_grp), "event", G_CALLBACK(OnNextPageMouseUp), this);
		canvas_button_disable (button_next);
		button_prev = ButtonImageNew (btn_prev, NULL, btn_prev_dis, 0.0, 0.0);
		g_signal_connect (G_OBJECT(button_prev->root_grp), "event", G_CALLBACK(OnPrevPageMouseUp), this);
		canvas_button_disable (button_prev);
	}
	else
	{
		// Label buttons
		button_next = ButtonStandardNew ("Next", tf_width-50.0, 0.0);
		g_signal_connect (G_OBJECT(button_next->root_grp), "event", G_CALLBACK(OnNextPageMouseUp), this);
		canvas_button_disable (button_next);
		button_prev = ButtonStandardNew ("Prev", 0.0, 0.0);
		g_signal_connect (G_OBJECT(button_prev->root_grp), "event", G_CALLBACK(OnPrevPageMouseUp), this);
		canvas_button_disable (button_prev);
	}
	
	// word list initialization (empty word is inserted as last_word)
	last_word = gnome_canvas_item_new (textfield,
								gnome_canvas_text_get_type(),
								"x", 0.0,
								"y", 0.0,
								"anchor", GTK_ANCHOR_NORTH_WEST,
								"fill-color", FONT_COLOR,
								"size", TEXT_SIZE,
								"text", "",
								NULL);
	gnome_canvas_item_lower_to_bottom (last_word);
	
}
void Canvas::DrawAuthorInfo ()
{
	// figure out coords for docking
	if (INFO_DOCK)
	{
		if (INFO_DOCK_LEFT >= 0)
		{
			INFO_X = INFO_DOCK_LEFT;
		}
		else
		{
			// do nothing
		}
		
		if (INFO_DOCK_RIGHT >= 0)
		{
			INFO_WIDTH = TEXTF_X - INFO_DOCK_RIGHT - INFO_X;
		}
		else
		{
			// do nothing
		}
		
		if (INFO_DOCK_BOTTOM >= 0)
		{
			//INFO_Y = CANVAS_HEIGHT - INFO_DOCK_BOTTOM - INFO_HEIGHT;
			INFO_HEIGHT = CANVAS_HEIGHT - INFO_Y - INFO_DOCK_BOTTOM;
		}
		else
		{
			// do nothing
		}
	}
	
	// draw outline
	info_outline = GNOME_CANVAS_GROUP (
					gnome_canvas_item_new (
							board,
							gnome_canvas_group_get_type (),
							"x", INFO_X,
							"y", INFO_Y,
							NULL));
	
	if (INFO_BORDER_SHOW)
	{
		// create points
		GnomeCanvasPoints *points;
		points = gnome_canvas_points_new (5);
		// fill in points
		points->coords[0] = 0.0;
		points->coords[1] = 0.0;
		points->coords[2] = INFO_WIDTH;
		points->coords[3] = 0.0;
		points->coords[4] = INFO_WIDTH;
		points->coords[5] = INFO_HEIGHT;
		points->coords[6] = 0.0;
		points->coords[7] = INFO_HEIGHT;
		points->coords[8] = 0.0;
		points->coords[9] = 0.0;
		// create item
		gnome_canvas_item_new (info_outline,
							gnome_canvas_line_get_type(),
							"points", points,
							"fill-color", INFO_BORDER_COLOR,
							"width-units", INFO_BORDER,
							NULL);
		gnome_canvas_points_free (points);
	}
	
	// draw background image
	// get center of the outline
	double w,h;
	w = INFO_WIDTH/2 + INFO_X;
	h = INFO_HEIGHT/2 + INFO_Y;
	//g_print ("info x,y %4.1f, %4.1f; info width, height %4.1f, %4.1f, w/h, %4.1f/%4.1f\n", INFO_X, INFO_Y, INFO_WIDTH, INFO_HEIGHT, w, h);
	
	// draw image
//	if (info_background_image != NULL)
//		gnome_canvas_item_new (info_outline,
//					gnome_canvas_pixbuf_get_type (),
//					"pixbuf", info_background_image,
//					"x", 0.0, "y", 0.0,
//					"anchor", GTK_ANCHOR_NORTH_WEST,
////					"width", 200, "width-set", true, // 300x451 for image pencilholder.jpg
////					"height", 266, "height-set", true,
//					NULL);
	
	
	// create group item for text
	author_info = 	GNOME_CANVAS_GROUP (
					gnome_canvas_item_new (
						info_outline,
						gnome_canvas_group_get_type (),
						"x", 0.0 + INFO_PADDING,
						"y", 0.0 + INFO_PADDING,
						NULL));

	// perform docking
	
	// set info width/height variables
	info_width = INFO_WIDTH - 2* INFO_PADDING;
	info_height = INFO_HEIGHT - 2* INFO_PADDING;
}
void Canvas::DrawSingleLine(GnomeCanvasGroup *group,
	    int x1, int y1, int x2, int y2, char *color, double thickness)
{
	// --------------------------------------------------------------------
	// Draws a single line
	// --------------------------------------------------------------------
	//GdkColor colorr = {0, 65000, 0, 0};
	GnomeCanvasPoints *points;
	points = gnome_canvas_points_new (2);
	
	
	points->coords[0] = x1;
	points->coords[1] = y1;
	points->coords[2] = x2;
	points->coords[3] = y2;

	gnome_canvas_item_new(group,
			      gnome_canvas_line_get_type(),
			      "points", points,
			      "fill_color", color,
			      "width_units", thickness,
			      NULL);
			
	gnome_canvas_points_free(points);
}
void Canvas::LoadText ()
{
	// --------------------------------------------------------------------
	// Iterates through sStory struct and inserts words into textfield
	// --------------------------------------------------------------------
	int w, p; // word/paragraph
	
	LoadHeadline ();
	
	// text
	for (p = 0; p < (int) Story.Paragraphs.size(); p++)
	{
		for (w = 0; w < (int) Story.Paragraphs[p].Words.size(); w++)
		{
			InsertWord (Story.Paragraphs[p].Words[w]);
		}
		// insert enter
		InsertWordEndl (TEXT_STYLE_NORMAL);
	}
	
	canvas_button_disable (button_next);
	canvas_button_disable (button_prev);
	// also hide page information
	gnome_canvas_item_hide (info_page_number);
}
void Canvas::LoadAuthorInfo (char* dir_path)
{
	GnomeCanvasItem *item;
	char* txt;
	double x1, y1, x2, y2;
	char* picture_file;
	
	// draw white background
	gnome_canvas_item_new (
					author_info,
					gnome_canvas_rect_get_type (),
					"x1", 0.0, "y1", 0.0,
					"x2", INFO_WIDTH - 2*INFO_PADDING, "y2", INFO_WIDTH - 2*INFO_PADDING,
					"fill-color", WHITE,
					NULL);
	
	txt = g_strdup_printf ("Meno: %s %s", person_data->name, person_data->surname);
	item = gnome_canvas_item_new (
			author_info,
			gnome_canvas_text_get_type (),
			"x", 0.0, "y", 0.0,
			"text", txt,
			"fill-color", D_INFO_TEXT_COLOR,
			"size", D_INFO_TEXT_SIZE,
			"anchor", GTK_ANCHOR_NORTH_WEST,
			NULL);
	g_free (txt);
	
	gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
	char* gend;
	if (person_data->gender == 1) { gend = "chlapec"; } else if (person_data->gender == 2) { gend = "dievča"; } else { gend = "pohlavie neisté"; }
	txt = g_strdup_printf ("Vek: %d rokov, %s", person_data->age, gend );
	item = gnome_canvas_item_new (
			author_info,
			gnome_canvas_text_get_type (),
			"x", 0.0, "y", y2,
			"text", txt,
			"fill-color", D_INFO_TEXT_COLOR,
			"size", D_INFO_TEXT_SIZE,
			"anchor", GTK_ANCHOR_NORTH_WEST,
			NULL);
	g_free (txt);
	
	// show picture
	gnome_canvas_item_get_bounds (item, &x1, &y1, &x2, &y2);
	picture_file = g_strdup_printf ("%s/%s", dir_path, PICTURE_FILE);
	GdkPixbuf *obr = gdk_pixbuf_new_from_file (picture_file, NULL); // add error handling
	g_free (picture_file);
	GnomeCanvasItem *picture;
	picture = gnome_canvas_item_new (
							author_info,
							gnome_canvas_pixbuf_get_type (),
							"x", 0.0, "y", y2 + 5.0,
							"pixbuf", obr,
							"anchor", GTK_ANCHOR_NORTH_WEST,
							"width", D_INFO_PICTURE_WIDTH, "width-set", true,
							"height", D_INFO_PICTURE_HEIGHT, "height-set", true,
							NULL);
	g_object_unref (obr);

	txt = g_strdup_printf ("Strana");
	// insert page number on the bottom
	info_page_number = gnome_canvas_item_new (
			author_info,
			gnome_canvas_text_get_type (),
			"x", 0.0, "y", 0.0,
			"text", txt,
			"fill-color", D_INFO_TEXT_COLOR,
			"size", D_INFO_TEXT_SIZE,
			"anchor", GTK_ANCHOR_NORTH_WEST,
			NULL);
	g_free (txt);
	gnome_canvas_item_get_bounds (info_page_number, &x1, &y1, &x2, &y2);
	gnome_canvas_item_move (info_page_number, 0.0, info_height - y2);
	
}
ErrorLoadPage Canvas::LoadPage (int page_number)
{
	// --------------------------------------------------------------------
	// Load single page into textfield (page_number is zero-based).
	// --------------------------------------------------------------------
	int w;	// general loop variable
	
	// check for existence of this page
	if (page_number > (int) Story.Paragraphs.size() - 1)
	{
		return eLoadPageNotExist;
	}
	
	//clear textfield and re-initialize it
	ClearTextfield ();
	DrawTextfield ();
	
	// Load complete text on negative page number
	if (page_number < 0)
	{
		LoadText (); // next/prev buttons disabled by default, [hide info_page_number field] (inside)
	}
	else
	{
		LoadHeadline ();
		
		// insert text of this page
		for (w = 0; w < (int) Story.Paragraphs[page_number].Words.size(); w++)
		{
			InsertWord (Story.Paragraphs[page_number].Words[w]);
		}
		
		// handle next/prev buttons - these are disabled by default as they are created in DrawTextfield ()
		if (page_number > 0) canvas_button_enable (button_prev);
		if (page_number < (int) Story.Paragraphs.size() - 1) canvas_button_enable (button_next);
		// update information about current page in info field
		char* page_nr = g_strdup_printf ("Strana %d z %d", page_number+1, (int) Story.Paragraphs.size());
		g_object_set (info_page_number, "text", page_nr, NULL);
		gnome_canvas_item_show (info_page_number);
	}
	
	page = page_number;
	MarksReset ();
	return eLoadPageOk;
}
ErrorLoadPage Canvas::LoadHeadline ()
{
	// --------------------------------------------------------------------
	// Loads headline into textfield.
	// This method is to be included from other Load* methods..
	// 
	// Method:
	// Insert words as usually and count their width. Center will occur if
	// and only if TEXT_HEADLINE_CENTER variable is true AND if the whole
	// headline fits into a single line. Otherwise, don't even bother.
	// Figure out width of the space left, divide by two and move all words
	// in tf_words by this much to the right (i.e. by positive amount).
	// Note that since this is headline, it's sufficent to move all words
	// from tf_words. We neglect whitespace, though.
	// --------------------------------------------------------------------
	int w;
	double headline_width = 0.0;
	
	// insert text
	for (w = 0; w < (int) Story.Headline.Words.size(); w++)
	{
		headline_width += InsertWord (Story.Headline.Words[w]);
	}
	
	// center text (if applicable && headline_width < line_width)
	if ((TEXT_HEADLINE_CENTER) && (headline_width < tf_width))
	{
		double df = (tf_width - headline_width) / 2;
		
		for (w = 0; w < (int) tf_words.size (); w++)
		{
			gnome_canvas_item_move (tf_words[w], df, 0.0);
		}
	}
	else
	{
		// do nothing
	}
	
	// insert 2x endline marker
	InsertWordEndl (TEXT_STYLE_NORMAL);
	InsertWordEndl (TEXT_STYLE_NORMAL);
	
	return eLoadPageOk;
}
void Canvas::ClearTextfield ()
{
	// --------------------------------------------------------------------
	// Clears and re-initializes textfield.
	// --------------------------------------------------------------------
	// clear data
	// destroy components
	if (button_next != NULL)
	{
		canvas_button_destroy (button_next);
		button_next = NULL;
	}
	if (button_prev != NULL)
	{
		canvas_button_destroy (button_prev);
		button_prev = NULL;
	}
	
	gtk_object_destroy(GTK_OBJECT(textfield));
	tf_words.clear ();	
	tf_words.resize (0);
	tf_swords.clear ();
	tf_swords.resize (0);
	tf_marks.clear ();
	tf_marks.resize (0);
	selected_word = -1;
//	canvas_button_disable (btnPlay);
//	canvas_button_disable (btnPush);
	page = -1;
//	story_id = -1;
	
	// re-initialize textfield (turned off, textfield is initialized separately)
	// DrawTextfield ();
}
void Canvas::ClearAuthorInfo ()
{
	// --------------------------------------------------------------------
	// Clears and re-initializes info window.
	// --------------------------------------------------------------------
	gtk_object_destroy (GTK_OBJECT(info_outline));
}
double Canvas::InsertWord (sWord word)
{
	// --------------------------------------------------------------------
	// This is to insert a single word into textfield. Note that width of
	// of the whole word is supposed to be less then the overall width of
	// the textfield. TODO: this needs to be fixed later somehow (low
	// priority).
	//
	// RETURN VALUE: double - width of the word + width of the whitespace
	// 
	// Issues:
	// When a new_word hits end of the line and is moved to a new line, then
	// the callback function doesn't react, even though is connected. All
	// other words work properly.
	// Therefore, new method was introduced in version v0.2.3, that in case
	// end of line is encountered, new_word item is destroyed and new one is
	// created on the correct coordinates.
	// Solution: 
	// New_word must not be created outside of textfield. This may happen,
	// when the creation process hits end of the line. Simple solution
	// to this problem is to create all items within textfield (on coords 0,0)
	// and then move them to their proper location.
	// --------------------------------------------------------------------
	if (SHOW_DEBUG) g_debug ("InsertWord(): ==== Running with input: %s", word.Text.c_str());
	
	// --------------------------------------------------------------------
	// Initializations, local variable declarations
	// --------------------------------------------------------------------
	GnomeCanvasItem *new_word;
	double last_x, last_y, last_xx, last_yy, new_x, new_y, new_xx, new_yy, new_w, new_h;
	TextStyle text_style = word.isHeadline ? TEXT_STYLE_HEADLINE : TEXT_STYLE_NORMAL;
	
	//char* text = (char*) word.Text.c_str();
	GError *utf8_error = NULL;
	char* text = g_locale_to_utf8 (word.Text.c_str(), -1, NULL, NULL, &utf8_error);
	if (utf8_error != NULL)
		if (SHOW_WARNINGS) g_warning ("g_locale_to_utf8(): Error #%d: `%s`!\n", utf8_error->code, utf8_error->message);
	
	// figure out where to place new word - after the last word
	gnome_canvas_item_get_bounds (last_word, &last_x, &last_y, &last_xx, &last_yy);
	if (SHOW_DEBUG) g_debug (" = last word, x1, y1, x2, y2: %4.1f, %4.1f | %4.1f, %4.1f", last_x, last_y, last_xx, last_yy);
	
	// --------------------------------------------------------------------
	// New Word creation
	// --------------------------------------------------------------------
	// Now we need to create this new word at coords 0,0
	// It is essential to create it on [0,0] or somewhere inside the textfield,
	// otherwise callback event functions will not react properly.
	new_word = InsertWordCreate (text, 0.0, 0.0, text_style);
	// connect handler to the newly created text_item
	g_signal_connect (G_OBJECT(new_word), "event", G_CALLBACK(OnItemEvent), this);
	// remember this item
	tf_words.resize(tf_words.size() + 1);
	tf_words[tf_words.size() - 1] = new_word;
	tf_swords.resize (tf_swords.size() + 1);
	tf_swords[tf_swords.size() - 1].Text = word.Text;
	tf_swords[tf_swords.size() - 1].ind = word.ind;
	tf_swords[tf_swords.size() - 1].Index = word.Index;
	tf_swords[tf_swords.size() - 1].isHeadline = word.isHeadline;
	tf_marks.resize (tf_marks.size() + 1);
	tf_marks[tf_marks.size() -1] = MARK_MODE_NONE;
	
	// --------------------------------------------------------------------
	// Place text item
	// --------------------------------------------------------------------
	// figure out new_word's width and height to see, if it fits into textfield
	gnome_canvas_item_get_bounds (new_word, &new_x, &new_y, &new_xx, &new_yy);
	new_w = new_xx - new_x; new_h = new_yy - new_y;
	if (SHOW_DEBUG) g_debug (" = textfield w,h: %4.1f, %4.1f || new item's w,h: %4.1f, %4.1f", tf_width, tf_height, new_w, new_h);
	
	// Now we need to figure out where to move this new word
	// check the current line first
	if (SHOW_DEBUG) g_debug (" ==condition: new_w > (tf_width - last_xx), %4.1f > %4.1f (%4.1f - %4.1f)", new_w, tf_width - last_xx, tf_width, last_xx);
	if ( new_w > (tf_width - last_xx) ) // word won't fit on this line
	{
		// try if the word fits on an empty line (if its size ain't greater then the size of a line)
		if (new_w > tf_width)
		{
			// Houston, we've got a problem!
			// Let's not panic, we just output it through the outline of the
			// the textfield. This needs to be fixed later (TODO).
			// >> do nothing here <<
			if (SHOW_WARNINGS) g_warning ("Canvas::InsertWord: `%s` doesn't seem to fit a line.", word.Text.c_str());
		}
		// check there is free line available
		if ( (last_yy+new_h) > tf_height )
		{
			// There is no more free lines available. As with the previous
			// problem (single word doesn't fit a line), we'll just ignore
			// this in early versions (TODO) and print the words out of scope.
			// >> do nothing here <<
			if (SHOW_WARNINGS) g_warning ("Canvas::InsertWord: Text hit bottom of textfield.");
		}
		// figure out shift vector for the word - now we have the word at [0,0]
		new_x = 0.0; 		// beginning of new line
		new_y = last_yy + TEXT_SPACING_VERT; 	// move it on next line
		gnome_canvas_item_move (new_word, new_x, new_y);
	}
	else // in this case word will fit into the current line
	{
		new_x = last_xx + TEXT_SPACING_HORIZ; // put it just behind the last word
		new_y = last_y;  // keep it on the same line
		gnome_canvas_item_move (new_word, new_x, new_y);
	}
	
	// --------------------------------------------------------------------
	// Insert whitespace
	// --------------------------------------------------------------------
	// One last thing to do it to put space just behind this new word and this
	// is going to be new `last_word`. We can ignore textfield bounds for this
	// one. We will also need new coords of new_word after it was moved.
	gnome_canvas_item_get_bounds (new_word, &new_x, &new_y, &new_xx, &new_yy);
	
	// create text and put it down as white space is not what we are interested in
	last_word = InsertWordCreate (WORD_DELIMITER, new_xx, new_y, text_style);
	gnome_canvas_item_lower_to_bottom (last_word);
	
	// Last but not least, return widths (new_w + last_word.width)
	// Only applies for Headline and if headline should be centered, otherwise, don't even bother..
	double width = 0.0;
	if ((word.isHeadline) && (TEXT_HEADLINE_CENTER))
	{
		gnome_canvas_item_get_bounds (last_word, &last_x, &last_y, &last_xx, &last_yy);
		width = new_w + (last_xx - last_x);
	}
	
	return width;
}
gint Canvas::InsertWordEndl (TextStyle style)
{
	// --------------------------------------------------------------------
	// Inserts end of line into textfield
	// This is done by just moving `last_item` into new position and setting
	// font properties if necessary.
	// --------------------------------------------------------------------
	// get current position
	double x1, y1, x2, y2;
	gnome_canvas_item_get_bounds (last_word, &x1, &y1, &x2, &y2);
	
	// manage font sizes
	int text_size;
	switch (style)
	{
		case TEXT_STYLE_HEADLINE:
		{
			text_size = TEXT_SIZE_HEADING;
			break;
		}
		default:
		{
			text_size = TEXT_SIZE;
		}
	}
	
	// move item and set properties
	gnome_canvas_item_set (last_word,
							"x", 0.0, // start new line
							"y", y2,
							"size", text_size,
							"text", TEXT_PARAGRAPH_INDEND,
							NULL);
	
	
	return 0;
}
GnomeCanvasItem* Canvas::InsertWordCreate (char* text, double x, double y, TextStyle style)
{
	// --------------------------------------------------------------------
	// Creates GnomeCanvasText item with proper font
	// --------------------------------------------------------------------
	// Colors
	char* fill_color_attr;	// fill-color or fill-collor-rgba
	char* fill_color_gdk;		// color code
	int   fill_color_aa;	// aa color code
	void* fill_color; // this pointer is cast into either int or char* as needed
	
	if (CANVAS_ANTI_ALIASED)
	{
		fill_color_attr = "fill-color-rgba";
		fill_color_aa = FONT_COLOR_AA;
		fill_color = (int*) fill_color_aa;
	}
	else
	{
		fill_color_attr = "fill-color";
		fill_color_gdk = FONT_COLOR;
		fill_color = (char**) fill_color_gdk;
	}
	
	// Fonts
	PangoFontDescription *desc;
	desc = PangoFontDescGet (style);
	
	// now create and return item
	GnomeCanvasItem* new_word = gnome_canvas_item_new (textfield,
						gnome_canvas_text_get_type (),
						"x", x,
						"y", y,
						"anchor", GTK_ANCHOR_NORTH_WEST,
						fill_color_attr, fill_color,
						"font-desc", desc,
						"text", text,
						NULL);
						
	pango_font_description_free (desc);
	
	return new_word;
}

PangoAttribute* Canvas::PangoAttrForegroundNew (sColor color, glong endindex)
{
	// --------------------------------------------------------------------
	// Creates PangoAttribute for foreground using custom color
	// --------------------------------------------------------------------
	PangoAttribute* at;
	at = pango_attr_foreground_new (color.red, color.green, color.blue);
	at->start_index = 0;
	at->end_index = endindex;
	return at;
}
PangoAttribute* Canvas::PangoAttrForegroundNew (char *spec, glong endindex)
{
	// --------------------------------------------------------------------
	// Creates PangoAttribute for foreground using custom color code
	// --------------------------------------------------------------------
//	if (SHOW_DEBUG) g_debug ("Running Foreground new with string spec: `%s`", spec);
	GdkColor *gdk_color;
	sColor *color;
	
	gdk_color = (GdkColor*) g_malloc (sizeof(GdkColor));
	color = (sColor*) g_malloc (sizeof (sColor));
		
	if (!gdk_color_parse (spec, gdk_color))
	{
		// parsing error
		if (SHOW_WARNINGS) g_warning ("PangoAttrForegroundNew: Error parsing color spec: %s", spec);
		color->red = 0; color->green = 0; color->blue = 0; // use black as default
	}
	else
	{
		color->red = gdk_color->red;
		color->green = gdk_color->green;
		color->blue = gdk_color->blue;
	}
	
	PangoAttribute* at;
	at = pango_attr_foreground_new (color->red, color->green, color->blue);
	at->start_index = 0;
	at->end_index = endindex;
	
	g_free (color);
	g_free (gdk_color);
	return at;
}
PangoAttribute* Canvas::PangoAttrBackgroundNew (sColor color, glong endindex)
{
	// --------------------------------------------------------------------
	// Creates PangoAttribute for background using custom color
	// --------------------------------------------------------------------
	PangoAttribute* at;
	at = pango_attr_background_new (color.red, color.green, color.blue);
	at->start_index = 0;
	at->end_index = endindex;
	return at;
}
PangoAttribute* Canvas::PangoAttrBackgroundNew (char *spec, glong endindex)
{
	// --------------------------------------------------------------------
	// Creates PangoAttribute for background using custom color code
	// --------------------------------------------------------------------
//	if (SHOW_DEBUG) g_debug ("Running Background new with string spec: `%s`", spec);
	GdkColor *gdk_color;
	sColor *color;
	
	gdk_color = (GdkColor*) g_malloc (sizeof(GdkColor));
	color = (sColor*) g_malloc (sizeof (sColor));
		
	if (!gdk_color_parse (spec, gdk_color))
	{
		// parsing error
//		if (SHOW_WARNINGS) g_warning ("PangoAttrBackgroundNew: Error parsing color spec: %s", spec);
		color->red = 0; color->green = 0; color->blue = 0; // use black as default
		//color = FONT_COLOR;
	}
	else
	{
		color->red = gdk_color->red;
		color->green = gdk_color->green;
		color->blue = gdk_color->blue;
	}
//	if (SHOW_DEBUG) g_debug (" - resulting color: RGB: %d, %d, %d", color->red, color->green, color->blue);
	
	PangoAttribute* at;
	at = pango_attr_background_new (color->red, color->green, color->blue);
	at->start_index = 0;
	at->end_index = endindex;
	
	g_free (color);
	g_free (gdk_color);
	return at;
}
void Canvas::ChangeTextColorMode (GnomeCanvasItem *item, TextColorMode mode, int text_length)
{
	// --------------------------------------------------------------------
	// Creates PangoAttribute for background using custom color code
	// --------------------------------------------------------------------
	PangoAttrList *attrs;
	PangoAttribute *attr_fore;//, *attr_back;
	
	g_object_get (item, "attributes", &attrs, NULL);
	if (attrs == NULL) attrs = pango_attr_list_new ();
	
	switch (mode)
	{
		case TEXT_COLOR_MODE_HLT:
		{
			attr_fore = PangoAttrForegroundNew (FONT_HLT_COLOR, text_length);
			//attr_back = PangoAttrBackgroundNew (FONT_HLT_BCOLOR, text_length);
			break;
		}
		case TEXT_COLOR_MODE_SELECT:
		{
			attr_fore = PangoAttrForegroundNew (FONT_SELECT_COLOR, text_length);
			//attr_back = PangoAttrBackgroundNew (FONT_SELECT_BCOLOR, text_length);
			break;
		}
		case TEXT_COLOR_MODE_SELECT_HLT:
		{
			attr_fore = PangoAttrForegroundNew (FONT_SELECT_HLT_COLOR, text_length);
			//attr_back = PangoAttrBackgroundNew (FONT_SELECT_HLT_BCOLOR, text_length);
			break;
		}
		
		default:
		{
			attr_fore = PangoAttrForegroundNew (FONT_COLOR, text_length);
			//attr_back = PangoAttrBackgroundNew (FONT_BCOLOR, text_length);
		}
	}
	
	pango_attr_list_change (attrs, attr_fore);
	//pango_attr_list_change (attrs, attr_back);
	g_object_set (item, "attributes", attrs, NULL);
}
PangoFontDescription* Canvas::PangoFontDescGet (TextStyle style)
{
	// --------------------------------------------------------------------
	// Creates PangoFontDescription for InsertWordCreate ()
	// --------------------------------------------------------------------
	PangoFontDescription *desc = pango_font_description_new ();
	
	// font family
	pango_font_description_set_family (desc, TEXT_FAMILY);
	
	// font size
	switch (style)
	{
		case TEXT_STYLE_HEADLINE:
		{
			pango_font_description_set_size (desc, TEXT_SIZE_HEADING);
			break;
		}
		default:
		{
			pango_font_description_set_size (desc, TEXT_SIZE);
		}
	}
	
	// Configuration of following attributes is not supported by ConfigHandler yet.
	
	// font style
	pango_font_description_set_style (desc, PANGO_STYLE_NORMAL); // PANGO_STYLE_NORMAL, PANGO_STYLE_ITALIC, PANGO_STYLE_OBLIQUE
	
	// font variant
	pango_font_description_set_variant (desc, PANGO_VARIANT_NORMAL); // PANGO_STYLE_NORMAL, PANGO_STYLE_SMALL_CAPS
	
	// font weight
	pango_font_description_set_weight (desc, PANGO_WEIGHT_NORMAL); // _ULTRALIGHT=200, _LIGHT=300, _NORMAL=400, _SEMIBOLD=600, _BOLD=700, _ULTRABOLD=800, _HEAVY=900
	
	// font stretch
	pango_font_description_set_stretch (desc, PANGO_STRETCH_NORMAL); // TODO
	
	return desc;
}
void Canvas::ChangeTextMarkMode (GnomeCanvasItem *item, MarkMode mode)
{
	// --------------------------------------------------------------------
	// Changes marking mode for given GnomeCanvasItem.
	// --------------------------------------------------------------------
	gchar *txt;
	PangoAttrList *attrs;
	PangoAttribute *attr_fore;
	
	g_object_get (item, "text", &txt, NULL);
	
	g_object_get (item, "attributes", &attrs, NULL);
	if (attrs == NULL) attrs = pango_attr_list_new ();
	
	attr_fore = PangoAttrForegroundNew (GetMarkModeColor (mode), (int) strlen (txt));
	
	g_free (txt);
	
	pango_attr_list_change (attrs, attr_fore);
	g_object_set (item, "attributes", attrs, NULL);
}
void Canvas::MarksReset ()
{
	// --------------------------------------------------------------------
	// Resets all marks. 
	// --------------------------------------------------------------------
	int i;
	
	if ((tf_marks.size() > 0) && (tf_words.size() > 0))
	{
		for (i = 0; i < (int) tf_marks.size(); i++)
		{
			tf_marks[i] = MARK_MODE_NONE;
			ChangeTextMarkMode (tf_words[i], MARK_MODE_NONE);
		}
	}
	
	marked_word = -1;
}

void Canvas::HandleResult ()
{
	// --------------------------------------------------------------------
	// Purpose of this method is to retrieve result of speech recognition
	// from sound module, evaluate the result, mark the word with color
	// based on how successfull the recognition was and manage selection
	// of the next word.
	// Here is the brief outline of what this method does:
	// 1) Retrieve result of speech recognition from sound module, using
	//    its GetLastResult property.
	// 2) Get the original word, based on selected_item index and then trim
	//    this word (trim characters, lower case, etc..).
	// 3) Compare these words and evaluate the result.
	// 4) De-select currently selected word (there is no need to change text
	//    color of current word, as we will mark this word in the next step).
	// 5) Mark current word according to the result of speech recognition.
	// 6) Select the next word in the list, if there is one available. Other-
	//    wise, do nothing in this part.
	// --------------------------------------------------------------------
	
	// 1) Retrieve result of speech recognition
	string result = sound->GetLastResult ();
	if (result == "")
	{
		if (SHOW_WARNINGS) g_warning ("Canvas::HandleResult: Empty result returned!");
		return;
	}
	
	
	// 2) Get original word
	gchar *cword;
	g_object_get (tf_words[selected_word], "text", &cword, NULL);
	string word = trim_word (cword);
	
	
	// 3) Evaluate result
	double perc = EvaluateResult (result, word);
	
	
	cout << "LAST RESULT '" << result << "' | ";
	cout << "SELECTED WORD '" << word << "' | ";
	cout << "EVALUATE: " << perc << "%" << endl;
	
	// 4) De-select word
	// no need to perform this step (skipped)
	
	
	// 5) Mark word
	if (selected_word == -1) if (SHOW_WARNINGS) g_warning ("Canvas::HandleResult: No word selected!");
	MarkResult (tf_words[selected_word], perc, result);
	tf_marks[selected_word] = get_mark_mode (perc);
	
	// 6) Select next word
	if (selected_word < (int) tf_words.size() - 1) // if there are more words on this page
	{
		selected_word++;
		
		// get word
		char *next_word;
		g_object_get (tf_words[selected_word], "text", &next_word, NULL);
		
		ChangeTextColorMode (tf_words[selected_word], TEXT_COLOR_MODE_SELECT, strlen (next_word));
		sound->GeneratePhoneticDict (next_word);
		sound->PhonetToGram();
		sound->GramToLat ();
	}
	else
	{
		// do nothing - no features implemented in this case
	}
	
}

double Canvas::EvaluateResult (string result, string word)
{
	// --------------------------------------------------------------------
	// Evaluate input strings and figure out how much they resemble each
	// other by computing their common distance. Return real number in 
	// range 0-100.
	// --------------------------------------------------------------------
	int dist = sdist (word, result);
	int size = (int) g_utf8_strlen (word.c_str(), -1);
	double perc = 100.0 - min (100.0, ((double) dist / (double) size * 100.0));
	
	return perc;
}
double Canvas::EvaluateResult_old (string result, string word)
{
	// --------------------------------------------------------------------
	// Evaluate input strings and figure out how much they resemble each
	// other. Return real number in range 0-100.
	// --------------------------------------------------------------------

	// check for substring
	gboolean is_substr = true;
	string::size_type loc = word.find (result, 0);
	if (loc == string::npos) is_substr = false;
	
	// get length ratio
	int size_word = word.size ();
	int size_result = result.size ();
	if (size_word == 0) return 0; // handle division by zero
	double perc = (double) size_result / (double) size_word;
	perc *= 100.0;
	
	if (!is_substr) perc *= 0.3;
	
	// finish evaluation
	// TODO fine tune result...
	// if the word is very short, result would be very bad even for correctly pronounced words
	#define USE_FINE_TUNE false
	
	if (USE_FINE_TUNE)
	{
		switch (size_word)
		{
			case 1:
			{
				perc *= 3.0;
				break;
			}
			case 2:
			{
				perc *= 2.0;
				break;
			}
			case 3:
			{
				perc *= 1.75;
				break;
			}
			case 4:
			{
				perc *= 1.5;
				break;
			}
			case 5:
			{
				perc *= 1.4;
				break;
			}
			case 6:
			{
				perc *= 1.3;
				break;
			}
			case 7:
			{
				perc *= 1.25;
				break;
			}
			case 8:
			{
				perc *= 1.18;
				break;
			}
			default:
			{
			}
		}
	}
	
	if (perc > 100.0) perc = 100.0;
	
	return perc;
}
void Canvas::MarkResult (GnomeCanvasItem *item, double perc, string result)
{
	gchar *txt;
	gchar* mark_color = get_mark_color (perc);
	g_object_get (item, "text", &txt, NULL);
	string word = g_strdup (txt);
	
	// check for substring
	gboolean is_substr = true;
	string::size_type loc = word.find (result, 0);
	if (loc == string::npos) is_substr = false;
	
	// create attribute(s)
	PangoAttrList *attrs;
	PangoAttribute *attr_fore;
	g_object_get (item, "attributes", &attrs, NULL);
	if (attrs == NULL) attrs = pango_attr_list_new ();
	// fill attribute
	attr_fore = PangoAttrForegroundNew (mark_color, strlen (txt));
	// apply attribute
	pango_attr_list_change (attrs, attr_fore);
	g_object_set (item, "attributes", attrs, NULL);
	
	// finalize
	g_free (txt);
}

// Tooltips
void Canvas::TipNew (char* label, double x, double y, double item_height)
{
	// --------------------------------------------------------------------
	// Creates new CanvasTip and places it on textfield
	// 
	// Parameters:
	// - label: text of the tooltip
	// - x, y : coords of the mouse event on the parent
	// - item_height : height of the text item, where the event took place
	// --------------------------------------------------------------------
	CanvasTipDesc *tip_desc;
	
	// create and initialize CanvasTipDesc struct
	tip_desc = g_new (CanvasTipDesc, 1);
	tip_desc->label = g_strdup (label);
	tip_desc->font_color = g_strdup (TIP_FONT_COLOR);
	tip_desc->font_size = TIP_FONT_SIZE;
	tip_desc->fill_color = g_strdup (TIP_COLOR);
	tip_desc->border_color = g_strdup (TIP_BORDER_COLOR);
	tip_desc->border_color_alt = g_strdup (TIP_BORDER_COLOR_ALT);
	tip_desc->border_thick = TIP_BORDER_THICK;
	tip_desc->border_plastic = TIP_BORDER_PLASTIC;
	tip_desc->padding_width = TIP_PADDING_WIDTH;
	tip_desc->padding_height = TIP_PADDING_HEIGHT;
	
	tip_desc->img = NULL; // no picture
	
	tooltip = canvas_tip_new (textfield, tip_desc);
	
	free (tip_desc->label);
	free (tip_desc->font_color);
	free (tip_desc->fill_color);
	free (tip_desc->border_color);
	free (tip_desc->border_color_alt);
	g_free (tip_desc);
	
	// move tip on its position
	double tx, ty;
	
	// get coords of root_grp (this gets coords relative to textfield canvas group - these default to 0,0)
	g_object_get (tooltip->root_grp, "x", &tx, "y", &ty, NULL);
	// convert coords relative to textfield --> to world coords (because out event x,y are in world coords)
	gnome_canvas_item_i2w (tooltip->root_grp, &tx, &ty);
	// tx, ty are current coords of the tooltip, x-tx == x if tx==0 (this is where the tips
	// are created by default. This moves tooltip on the mouse and +item_height moves it down
	// by height of the text item.
//	g_print ("[x,y]:[%4.1f, %4.1f]; [tx,ty]:[%4.1f, %4.1f], item_height: %4.1f\n", x, y, tx, ty, item_height);
	
	// this moves tooltip to the event x,y and down by item_height
//	gnome_canvas_item_move (tooltip->root_grp, x - tx, y - ty + item_height);
	
	// this positions tooltip on event x,y and on the bottom of the clicked item
	gnome_canvas_item_move (tooltip->root_grp, x - tx, item_height + 2);
	
}
void Canvas::TipNewImg (GdkPixbuf *img, double x, double y, double item_height)
{
	// --------------------------------------------------------------------
	// Creates new CanvasTip and places it on textfield
	// 
	// Parameters:
	// - label: text of the tooltip
	// - x, y : coords of the mouse event on the parent
	// - item_height : height of the text item, where the event took place
	// --------------------------------------------------------------------
	CanvasTipDesc *tip_desc;
	
	// create and initialize CanvasTipDesc struct
	tip_desc = g_new (CanvasTipDesc, 1);
	tip_desc->label = NULL;
	tip_desc->font_color = g_strdup_printf ("%s", TIP_FONT_COLOR);
	tip_desc->font_size = TIP_FONT_SIZE;
	tip_desc->fill_color = g_strdup_printf ("%s", TIP_COLOR);
	tip_desc->border_color = g_strdup_printf ("%s", TIP_BORDER_COLOR);
	tip_desc->border_color_alt = g_strdup_printf ("%s", TIP_BORDER_COLOR_ALT);
	tip_desc->border_thick = TIP_BORDER_THICK;
	tip_desc->border_plastic = TIP_BORDER_PLASTIC;
	tip_desc->padding_width = TIP_PADDING_WIDTH;
	tip_desc->padding_height = TIP_PADDING_HEIGHT;
	
	tip_desc->img = img; // no picture
	
	tooltip = canvas_tip_new (textfield, tip_desc);
	
	free (tip_desc->label);
	free (tip_desc->font_color);
	free (tip_desc->fill_color);
	free (tip_desc->border_color);
	free (tip_desc->border_color_alt);
	g_free (tip_desc);
	
	// move tip on its position
	double tx, ty;
	
	// get coords of root_grp (this gets coords relative to textfield canvas group - these default to 0,0)
	g_object_get (tooltip->root_grp, "x", &tx, "y", &ty, NULL);
	// convert coords relative to textfield --> to world coords (because out event x,y are in world coords)
	gnome_canvas_item_i2w (tooltip->root_grp, &tx, &ty);
	// tx, ty are current coords of the tooltip, x-tx == x if tx==0 (this is where the tips
	// are created by default. This moves tooltip on the mouse and +item_height moves it down
	// by height of the text item.
//	g_print ("[x,y]:[%4.1f, %4.1f]; [tx,ty]:[%4.1f, %4.1f], item_height: %4.1f\n", x, y, tx, ty, item_height);
	
	// this moves tooltip to the event x,y and down by item_height
//	gnome_canvas_item_move (tooltip->root_grp, x - tx, y - ty + item_height);
	
	// this positions tooltip on event x,y and on the bottom of the clicked item
	gnome_canvas_item_move (tooltip->root_grp, x - tx, item_height + 2);
	
}
void Canvas::TipDestroy ()
{
	canvas_tip_destroy (tooltip);
	tooltip = NULL;
}

// Canvas Buttons
CanvasButton* Canvas::ButtonStandardNew (char* label_text, double x, double y)
{
	// --------------------------------------------------------------------
	// Creates standard canvas button with mostly default attributes.
	// Note: it is for textfield only.
	// --------------------------------------------------------------------
	CanvasButtonDesc *desc;
	CanvasButton* button = NULL;
	
	desc = g_new (CanvasButtonDesc, 1);
	
	desc->type = ButtonTypeLabel;
	desc->label_text = NULL;
	desc->label_text_size = BUTTON_TEXT_SIZE;
	desc->label_color = BUTTON_TEXT_COLOR;
	desc->label_color_alt = BUTTON_TEXT_COLOR_ALT;
	desc->background_color = BUTTON_BACKGROUND_COLOR;
	desc->border_width = BUTTON_BORDER_WIDTH;
	desc->border_color = BUTTON_BORDER_COLOR;
	desc->shadow_color = BUTTON_SHADOW_COLOR;
	desc->border_show = true;
	desc->toggle_button = false;
	desc->plastic_on_motion = true;
	desc->width = BUTTON_WIDTH;
	desc->height = BUTTON_HEIGHT;
	
	button = canvas_button_new (textfield, desc);
	canvas_button_set_text (button, label_text);
	canvas_button_move_at (button, x, y);
	
	return button;
}
CanvasButton* Canvas::ButtonImageNew (GdkPixbuf *pic, GdkPixbuf *pic_clicked, GdkPixbuf *pic_disabled, double x, double y)
{
	// --------------------------------------------------------------------
	// Creates image canvas button with mostly default attributes.
	// --------------------------------------------------------------------
	CanvasButtonDesc *desc;
	CanvasButton* button;
	
	desc = g_new (CanvasButtonDesc, 1);
	
	desc->type = ButtonTypeImage;
	desc->label_text = NULL;
	desc->label_text_size = BUTTON_TEXT_SIZE;
	desc->label_color = BUTTON_TEXT_COLOR;
	desc->label_color_alt = BUTTON_TEXT_COLOR_ALT;
	desc->background_color = BUTTON_BACKGROUND_COLOR;
	desc->border_width = BUTTON_BORDER_WIDTH;
	desc->border_color = BUTTON_BORDER_COLOR;
	desc->shadow_color = BUTTON_SHADOW_COLOR;
	desc->border_show = false;
	desc->toggle_button = false;
	desc->plastic_on_motion = false;;
	desc->width = gdk_pixbuf_get_width (pic);
	desc->height = gdk_pixbuf_get_height (pic);
	
	button = canvas_button_new (textfield, desc);
	canvas_button_set_image (button, pic, pic_clicked, pic_disabled);
	canvas_button_move_at (button, x, y);
	
	return button;
}

// Listbox
CanvasListbox* Canvas::ListboxSelectNew (double x, double y)
{
	// --------------------------------------------------------------------
	// Creates listbox for listing available articles.
	// --------------------------------------------------------------------
	CanvasListboxDesc *desc;
	CanvasListbox *lb;
	
	// --------------------------------------------------------------------
	// Create listbox description.
	// --------------------------------------------------------------------
	desc = g_new (CanvasListboxDesc, 1);
	
	desc->width = LB_WIDTH;
	desc->height = LB_HEIGHT;
	desc->text_size = LB_TEXT_SIZE;
	desc->border_width = LB_BORDER_WIDTH;
	desc->padding = LB_PADDING;
	
	desc->text_color = LB_TEXT_COLOR;
	desc->border_color = LB_BORDER_COLOR;
	
	// --------------------------------------------------------------------
	// Create new listbox.
	// --------------------------------------------------------------------
	lb = canvas_listbox_new (board, desc);
	
	
	// --------------------------------------------------------------------
	// Configure listbox.
	// --------------------------------------------------------------------
	canvas_listbox_move_at (lb, x, y);
	
	return lb;
}
CanvasButton* Canvas::ButtonNewOnBoard (char* label_text, double x, double y)
{
	// --------------------------------------------------------------------
	// Creates standard canvas button with mostly default attributes.
	// Note: it puts it onto the board.
	// --------------------------------------------------------------------
	CanvasButtonDesc *desc;
	CanvasButton* button = NULL;
	
	desc = g_new (CanvasButtonDesc, 1);
	
	desc->type = ButtonTypeLabel;
	desc->label_text = NULL;
	desc->label_text_size = BUTTON_TEXT_SIZE;
	desc->label_color = BUTTON_TEXT_COLOR;
	desc->label_color_alt = BUTTON_TEXT_COLOR_ALT;
	desc->background_color = BUTTON_BACKGROUND_COLOR;
	desc->border_width = BUTTON_BORDER_WIDTH;
	desc->border_color = BUTTON_BORDER_COLOR;
	desc->shadow_color = BUTTON_SHADOW_COLOR;
	desc->border_show = true;
	desc->toggle_button = false;
	desc->plastic_on_motion = false;
	desc->width = BUTTON_WIDTH;
	desc->height = BUTTON_HEIGHT;
	
	button = canvas_button_new (board, desc);
	canvas_button_set_text (button, label_text);
	canvas_button_move_at (button, x, y);
	
	return button;
}
CanvasButton* Canvas::ButtonImageIconNew (char* label, GdkPixbuf *image, GdkPixbuf *image_clicked, GdkPixbuf *icon, GdkPixbuf *icon_disabled, double x, double y)
{
	// --------------------------------------------------------------------
	// Creates image canvas button with an icon image.
	// Note: it puts it onto the board.
	// --------------------------------------------------------------------
	CanvasButtonDesc *desc;
	CanvasButton* button = NULL;
	
	desc = g_new (CanvasButtonDesc, 1);
	
	desc->type = ButtonTypeImage;
	desc->label_text = NULL;
	desc->label_text_size = BUTTON_TEXT_SIZE;
	desc->label_color = BUTTON_TEXT_COLOR;
	desc->label_color_alt = BUTTON_TEXT_COLOR_ALT;
	desc->background_color = BUTTON_BACKGROUND_COLOR;
	desc->border_width = BUTTON_BORDER_WIDTH;
	desc->border_color = BUTTON_BORDER_COLOR;
	desc->border_show = false;
	desc->shadow_color = BUTTON_SHADOW_COLOR;
	desc->toggle_button = false;
	desc->plastic_on_motion = false;
	desc->width = BUTTON_WIDTH_ICON;
	desc->height = BUTTON_HEIGHT_ICON;
	
	button = canvas_button_new (board, desc);
	canvas_button_set_image (button, image, image_clicked, button_dis);
	canvas_button_set_icon (button, icon, icon_disabled);
	if (BUTTON_SHOW_IMG_LABELS) canvas_button_set_text (button, label);
	canvas_button_move_at (button, x, y);
	
	// connect tooltip
	CanvasTipDesc *tip_desc;
	tip_desc = canvas_tip_desc_new ();
	tip_desc->label = g_strdup_printf ("%s", label);
	tip_desc->font_color = g_strdup_printf ("%s", TIP_FONT_COLOR);
	tip_desc->font_size = TIP_FONT_SIZE;
	tip_desc->fill_color = g_strdup_printf ("%s", TIP_COLOR);
	tip_desc->border_color = g_strdup_printf ("%s", TIP_BORDER_COLOR);
	tip_desc->border_color_alt = g_strdup_printf ("%s", TIP_BORDER_COLOR_ALT);
	tip_desc->border_thick = TIP_BORDER_THICK;
	tip_desc->border_plastic = TIP_BORDER_PLASTIC;
	tip_desc->padding_width = TIP_PADDING_WIDTH;
	tip_desc->padding_height = TIP_PADDING_HEIGHT;
	tip_desc->img = NULL;
	
	canvas_button_set_tooltip (button, tip_desc);
	
	return button;
}

// Switch methods
void Canvas::SwitchButtons_LoadedStory ()
{
	// --------------------------------------------------------------------
	// Occurs when the 'open button' is clicked.
	// --------------------------------------------------------------------
	canvas_listbox_hide (lb_select);
	canvas_button_hide (btnLoadCancel);
	canvas_button_show (btnLoadStory);
	
	canvas_button_show (btnPlay);
	canvas_button_hide (btnPlayCancel);
	canvas_button_show (btnPush);
	canvas_button_show (btnRead);
	canvas_button_show (btnDictate);
	
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnPush);
	canvas_button_enable (btnRead);
	canvas_button_enable (btnDictate);
}
void Canvas::SwitchButtons_Load ()
{
	// --------------------------------------------------------------------
	// Occurs when we load a story in listbox.
	// --------------------------------------------------------------------
	canvas_listbox_show (lb_select);
	canvas_button_show (btnLoadCancel);
	canvas_button_hide (btnLoadStory);
	
	canvas_button_hide (btnPlay);
	canvas_button_hide (btnPlayCancel);
	canvas_button_hide (btnPush);
	canvas_button_hide (btnRead);
	canvas_button_hide (btnDictate);
}
void Canvas::SwitchButtons_LoadCancel ()
{
	// --------------------------------------------------------------------
	// Occurs when we click 'cancel button' in listbox.
	// --------------------------------------------------------------------
	canvas_listbox_hide (lb_select);
	canvas_button_hide (btnLoadCancel);
	canvas_button_show (btnLoadStory);
	
	canvas_button_show (btnPlay);
	canvas_button_hide (btnPlayCancel);
	canvas_button_show (btnPush);
	canvas_button_show (btnRead);
	canvas_button_show (btnDictate);
}
void Canvas::SwitchButtons_Play ()
{
	// --------------------------------------------------------------------
	// Occurs when we click the 'play button'.
	// --------------------------------------------------------------------
	// disable buttons (playback, recording and loading)
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnLoadStory);
	canvas_button_disable (btnPush);
	canvas_button_disable (btnRead);
	canvas_button_disable (btnDictate);
	
	#if GCOMPRIS_API
		// just disable play button
		
	#else
		// hide play button and show play_cancel button
		canvas_button_hide (btnPlay);
		canvas_button_show (btnPlayCancel);
	#endif
	
	tf_words_selectable = false;
}
void Canvas::SwitchButtons_PlayCancel ()
{
	// --------------------------------------------------------------------
	// Occurs when we cancel playback in progress.
	// --------------------------------------------------------------------
	if (selected_word == -1)
	{
		canvas_button_disable (btnPlay);
		canvas_button_disable (btnPush);
	}
	else
	{
		canvas_button_enable (btnPlay);
		canvas_button_enable (btnPush);
	}
	canvas_button_enable (btnLoadStory);
	canvas_button_enable (btnRead);
	canvas_button_enable (btnDictate);
	canvas_button_show (btnPlay);
	canvas_button_hide (btnPlayCancel);
	
	tf_words_selectable = true;
}
void Canvas::SwitchButtons_SelectWord ()
{
	// --------------------------------------------------------------------
	// Occurs when we select a word.
	// --------------------------------------------------------------------
	if (sound != NULL) sound->PlayStop (); // just making sure there in no playback in progress (feature: selecting a word during playback stops playback)
	canvas_button_enable (btnPlay);
	canvas_button_hide (btnPlayCancel);
	canvas_button_enable (btnPush);
}
void Canvas::SwitchButtons_DeselectWord ()
{
	// --------------------------------------------------------------------
	// Occurs when we de-select a word.
	// --------------------------------------------------------------------
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnPush);
}
void Canvas::SwitchButtons_StoryClosed ()
{
	// --------------------------------------------------------------------
	// Occurs when the story is closed (right mouse click).
	// --------------------------------------------------------------------
	// this actually resets button layout
	canvas_button_show (btnLoadStory);
	if (story_list.size () > 0)
		canvas_button_enable (btnLoadStory);
	else
		canvas_button_disable (btnLoadStory);
	canvas_listbox_hide (lb_select);
	
	canvas_button_hide (btnLoadCancel);
	
	canvas_button_disable (btnPlay);
	canvas_button_hide (btnPlayCancel);
	canvas_button_disable (btnPush);
	canvas_button_disable (btnRead);
	canvas_button_hide (btnReadCancel);
	canvas_button_disable (btnDictate);
	canvas_button_hide (btnDictateCancel);
}
void Canvas::SwitchButtons_Read ()
{
	// --------------------------------------------------------------------
	// When Read button is clicked.
	// --------------------------------------------------------------------
	canvas_button_disable (button_next);
	canvas_button_disable (button_prev);
	
	
	canvas_button_disable (btnLoadStory);
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnPush);
	
	#if GCOMPRIS_API
		canvas_button_disable (btnRead);
	#else
		canvas_button_hide (btnRead);
		canvas_button_show (btnReadCancel);
	#endif
	
	canvas_button_disable (btnDictate);
}
void Canvas::SwitchButtons_ReadCancel ()
{
	// --------------------------------------------------------------------
	// When ReadCancel button is clicked or when Reading finishes.
	// --------------------------------------------------------------------
	// enable next/prev buttons (now should be disabled)
	if (page > 0) canvas_button_enable (button_prev);
	else canvas_button_disable (button_prev);
	if (page < (int) Story.Paragraphs.size() - 1)
		canvas_button_enable (button_next);
	else canvas_button_disable (button_next);
	
	canvas_button_enable (btnLoadStory);
	canvas_button_enable (btnRead);
	canvas_button_enable (btnDictate);
	canvas_button_show (btnRead);
	canvas_button_hide (btnReadCancel);
	
	tf_words_selectable = true;
}
void Canvas::SwitchButtons_Dictate ()
{
	// --------------------------------------------------------------------
	// When Read button is clicked.
	// --------------------------------------------------------------------
	canvas_button_disable (button_next);
	canvas_button_disable (button_prev);
	
	
	canvas_button_disable (btnLoadStory);
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnPush);
	canvas_button_disable (btnRead);
	
//	#if GCOMPRIS_API
//		canvas_button_disable (btnDictate);
//	#else
		canvas_button_hide (btnDictate);
		canvas_button_show (btnDictateCancel);
//	#endif
	
}
void Canvas::SwitchButtons_DictateCancel ()
{
	// --------------------------------------------------------------------
	// When ReadCancel button is clicked or when Reading finishes.
	// --------------------------------------------------------------------
	// enable next/prev buttons (now should be disabled)
	if (page > 0) canvas_button_enable (button_prev);
	else canvas_button_disable (button_prev);
	if (page < (int) Story.Paragraphs.size() - 1)
		canvas_button_enable (button_next);
	else canvas_button_disable (button_next);
	
	canvas_button_enable (btnLoadStory);
	canvas_button_enable (btnRead);
	canvas_button_show (btnRead);
	canvas_button_hide (btnReadCancel);
	canvas_button_enable (btnDictate);
	canvas_button_show (btnDictate);
	canvas_button_hide (btnDictateCancel);
	
	tf_words_selectable = true;
}
void Canvas::SwitchButtons_DisableAll ()
{
	// --------------------------------------------------------------------
	// Disables all buttons.
	// --------------------------------------------------------------------
	canvas_button_disable (button_next);
	canvas_button_disable (button_prev);
	
	canvas_button_disable (btnLoadStory);
	canvas_button_disable (btnPlay);
	canvas_button_disable (btnPush);
	canvas_button_disable (btnRead);
	canvas_button_disable (btnDictate);
	canvas_button_show (btnLoadStory);
	canvas_button_show (btnPlay);
	canvas_button_show (btnPush);
	canvas_button_show (btnRead);
	canvas_button_show (btnDictate);
	
	canvas_button_hide (btnLoadCancel);
	canvas_button_hide (btnPlayCancel);
	
	tf_words_selectable = false;
}
void Canvas::SwitchButtons_EnableAll ()
{
	// --------------------------------------------------------------------
	// Enables all buttons.
	// --------------------------------------------------------------------
	if (page > 0) canvas_button_enable (button_prev);
	else canvas_button_disable (button_prev);
	if (page < (int) Story.Paragraphs.size() - 1)
		canvas_button_enable (button_next);
	else canvas_button_disable (button_next);
	
	canvas_button_enable (btnLoadStory);
	canvas_button_enable (btnPlay);
	canvas_button_enable (btnPush);
	canvas_button_enable (btnRead);
	canvas_button_enable (btnDictate);
	canvas_button_show (btnLoadStory);
	canvas_button_show (btnPlay);
	canvas_button_show (btnPush);
	canvas_button_show (btnRead);
	canvas_button_show (btnDictate);
	
	canvas_button_hide (btnLoadCancel);
	canvas_button_hide (btnPlayCancel);
	
	tf_words_selectable = true;
}


// EVENTS
gint Canvas::OnItemEvent (GnomeCanvasItem* item, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// This event handles selecting words, highlighting them as mouse moves
	// over them and tooltips.
	// --------------------------------------------------------------------
	double event_x, event_y;	// world coords where the event occured
	double bx1, by1, bx2, by2;	// bounding box of item (parent's relative coords)
	gchar *txt;					// current item's text property
	int i;						// general for-cycle variable
	int selected_word;			// index of selected item (or -1 if none is selected)
	gboolean thisSelected;		// is the current item selected? (for enter/leave events)
	int thisItemIndex;			// index of this clicked word in obj->tf_words (this can be different then selected_word) (not used!)
	TextColorMode text_clr_mode;// Selected/Hightlighted

	// --------------------------------------------------------------------
	// GLOBAL INITIALIZATION.
	// --------------------------------------------------------------------
	// get this item's bounds
	gnome_canvas_item_get_bounds (item, &bx1, &by1, &bx2, &by2);
	
	// get index of the selected word (before event)
	selected_word = obj->selected_word; //							==> selected_word
	
	// is current word `item` selected? (for enter/leave events)	==> thisSelected
	// what is the index of this 'item'?							==> thisItemIndex
	thisSelected = false;
	i = 0;
	thisItemIndex = -1; // this means word was not found in tf_words (after the following cycle)
	while (i < (int) obj->tf_words.size())
	{
		if (item == obj->tf_words[i])
		{
			if (i == obj->selected_word) thisSelected = true;
			thisItemIndex = i;
			break;
		}
		i++;
	}

	
	switch (event->type)
	{
		// --------------------------------------------------------------------
		// GDK_BUTTON_PRESS
		// --------------------------------------------------------------------
		case GDK_BUTTON_PRESS:
		{
			g_object_get (item, "text", &txt, NULL);
			if (obj->SHOW_DEBUG) g_debug ("GDK_BUTTON_PRESS: Clicked text '%s', thisItemIndex = %i, selected_word = %i", txt, thisItemIndex, obj->selected_word);
			
			/*
			 * To select/deselect items, one must determine what word was selected
			 * up to now (if any) and if it is other then current word 'item'.
			 * 
			 * 1) No word is selected <--> (selected_word < 0)
			 * ==> select current word 'item'
			 * 
			 * 2) There is some word selected <--> (selected_word >= 0)
			 * 
			 * 		a) This word is selected <--> (thisSelected == true)
			 * 		==> deselect this word (*)
			 * 
			 * 		b) Other word is selected <--> (thisSelected == false)
			 * 		==> deselect other word
			 * 		==> select this word
			 * 
			 * (*) - optional, if we wouldn't want to toggle word selection when
			 * clicking on it multiple times, we would set local variable
			 * toggle_selection to false.
			 */
			 gboolean select_new = false;
			 gboolean deselect_old = false;
			 #define toggle_selection true // this determines whether clicking on the same word multiple times toggle selection on and off (true) or only reselects the same word (false)
			 
			 if (selected_word < 0)	// 1)
			 {
			 	select_new = true;
			 }
			 else					// 2)
			 {
			 	if (thisSelected)		// a)
			 	{
			 		// do nothing here unless toggle_selection is turned on
			 		
			 		if (toggle_selection) deselect_old = true;
			 		else select_new = true;
			 	}
			 	else					// b)
			 	{
			 		select_new = true;
			 		deselect_old = true;
			 	}
			 }
			 
			 // --------------------------------------------------------------------
			 // DO SELECTION STUFF ONLY IN SELECTABLE MODE
			 // --------------------------------------------------------------------
			if (obj->tf_words_selectable)
			{
				// DESELECT old and re-hightlight it
				// - set text_clr_mode based on whether thisSelected
				if (deselect_old)
				{
					char* old_txt;
					g_object_get (obj->tf_words[selected_word], "text", &old_txt, NULL);
					if (thisSelected) text_clr_mode = TEXT_COLOR_MODE_HLT; else text_clr_mode = TEXT_COLOR_MODE_NORMAL;
					obj->ChangeTextColorMode (obj->tf_words[selected_word], text_clr_mode, strlen (old_txt));
					obj->ChangeTextMarkMode (obj->tf_words[selected_word], obj->tf_marks[obj->selected_word]);
					obj->selected_word = -1;
					obj->SwitchButtons_DeselectWord ();
					
					//if (obj->SHOW_MESSAGES) g_print ("GDK_BUTTON_PRESS: Word deselected '%s'\n", old_txt);
					g_free (old_txt);
				}
				
				// SELECT new and re-highlight it
				if (select_new)
				{
					obj->ChangeTextColorMode (item, TEXT_COLOR_MODE_SELECT_HLT, strlen (txt));
					obj->selected_word = thisItemIndex;
					obj->SwitchButtons_SelectWord ();
					obj->sound->GeneratePhoneticDict (txt);
					obj->sound->PhonetToGram();
					obj->sound->GramToLat ();
					//if (obj->SHOW_MESSAGES) g_print ("GDK_BUTTON_PRESS: Word selected '%s'\n", txt);
				}
			}
			
			g_free (txt);
			//g_print ("Koniec: selected_word = %d\n", obj->selected_word);
			return true; // do not propagate this event further as we have handled it
		}
		
		// --------------------------------------------------------------------
		// GDK_ENTER_NOTIFY
		// --------------------------------------------------------------------
		case GDK_ENTER_NOTIFY:
		{
			event_x = event->crossing.x;
			event_y = event->crossing.y;
			
			g_object_get (item, "text", &txt, NULL);
			if (obj->SHOW_DEBUG) g_debug ("GDK_ENTER_NOTIFY: with text: %s", txt);
			
			if (obj->CANVAS_ANTI_ALIASED)
			{
				// --------------------------------------------------------------------
				// Version for Anti-aliased GnomeCanvas
				// --------------------------------------------------------------------
				// background color
				// foreground color change
				g_object_set (item, "fill-color-rgba", obj->FONT_HLT_COLOR_AA, NULL);
			}
			else
			{
				// --------------------------------------------------------------------
				// Version for GDK GnomeCanvas - do selection stuff only if textfield
				// is selectable.
				// --------------------------------------------------------------------
				if (obj->tf_words_selectable)
				{
					if (thisSelected)
					{
						if (obj->SHOW_DEBUG) g_debug ("GDK_ENTER_NOTIFY: this text item is selected.");
						obj->ChangeTextColorMode (item, TEXT_COLOR_MODE_SELECT_HLT, strlen (txt));
					}
					else
					{
						if (obj->SHOW_DEBUG) g_debug ("GDK_ENTER_NOTIFY: this text item is not selected.");
						obj->ChangeTextColorMode (item, TEXT_COLOR_MODE_HLT, strlen (txt));
					}
				}
			}
			
			// show tooltip
			// Notes on last parameter:
			// by2: bottom of the clicked text item
			// by2 - by1: height of the clicked item
//			obj->TipNew (txt, event_x, event_y, by2);
			
			// show tooltip with smiley
			if (thisItemIndex >= 0)
			{
				GdkPixbuf* smiley = obj->GetMarkedPixbuf (obj->tf_marks[thisItemIndex]);
				if (smiley != NULL)
					obj->TipNewImg (smiley, event_x, event_y, by2);
			}
			
			g_free (txt);
			return true; // handled
		}
		
		// --------------------------------------------------------------------
		// GDK_LEAVE_NOTIFY
		// --------------------------------------------------------------------
		case GDK_LEAVE_NOTIFY:
		{
			// if (obj->SHOW_DEBUG) g_debug ("GDK_LEAVE_NOTIFY");
			event_x = event->crossing.x;
			event_y = event->crossing.y;
			g_object_get (item, "text", &txt, NULL);
			//if (obj->SHOW_DEBUG) g_debug ("GDK_LEAVE_NOTIFY: from text: %s", txt);
			
			if (obj->CANVAS_ANTI_ALIASED)
			{
				// --------------------------------------------------------------------
				// Version for Anti-aliased GnomeCanvas
				// --------------------------------------------------------------------
				// background color
				// foreground color change
				g_object_set (item, "fill-color-rgba", obj->FONT_COLOR_AA, NULL);
			}
			else
			{
				// --------------------------------------------------------------------
				// Version for GDK GnomeCanvas - DO SELECTION STUFF ONLY IN SELECTABLE
				// MODE!
				// --------------------------------------------------------------------
				if (obj->tf_words_selectable)
				{
					if (thisSelected)
					{
						obj->ChangeTextColorMode (item, TEXT_COLOR_MODE_SELECT, strlen (txt));
					}
					else
					{
						obj->ChangeTextColorMode (item, TEXT_COLOR_MODE_NORMAL, strlen (txt));
						obj->ChangeTextMarkMode (item, obj->tf_marks[thisItemIndex]);
					}
				}
			}
			
			// hide tooltip
			obj->TipDestroy ();
			
			g_free (txt);
			return true; // handled
		}
		
//		case GDK_MOTION_NOTIFY:
//		{
//			event_x = event->motion.x;
//			event_y = event->motion.y;
//			if (obj->SHOW_DEBUG) g_debug ("x,y: %4.1f, %4.1f", event_x, event_y);
//		
//			return false;	
//		}
		
		default:
			return false; // not handled
		
		
	}
	
	return false; // not handled
}
gint Canvas::OnCanvasEvent (GnomeCanvas *g_canvas, GdkEvent *event, Canvas* obj)
{
	double event_x, event_y;
	
	switch (event->type)
	{
		case GDK_BUTTON_PRESS:
		{
			event_x = event->button.x;
			event_y = event->button.y;
			
			if (event->button.button == 1) // LEFT MOUSE - deselect word...
			{
				//if (obj->SHOW_DEBUG) g_debug ("Left Mouse pressed - event start..");
				/*
				 * General idea:
				 * It is quite difficult and unreliable to find out at this moment, if
				 * there is text item or any other item bellow this point. We want to
				 * avoid handling items here, so we need to figure out whether there is
				 * an item on this coords. So we just cycle through all items and check
				 * their coords.
				 * Feature: Clicking on buttons page_next and page_prev thus deselects
				 * selected word.
				 */
				 double x1,y1, x2,y2;
				 gnome_canvas_item_w2i (GNOME_CANVAS_ITEM(obj->textfield), &event_x, &event_y);
				 gboolean item_found = false;
				 int i = 0;
				 while ((!item_found) && (i < (int) obj->tf_words.size()))
				 {
				 	gnome_canvas_item_get_bounds (obj->tf_words[i], &x1, &y1, &x2, &y2);
				 	// is event_x, event_y inside rect (x1, y1, x2, y2)?
				 	if ((event_x>=x1)&&
				 		(event_x<=x2)&&
				 		(event_y>=y1)&&
				 		(event_y<=y2))
				 	{
				 		item_found = true;
				 		if (obj->SHOW_DEBUG) g_debug ("Found text item: event[x,y]: %4.1f, %4.1f; Box: [%4.1f,%4.1f] [%4.1f,%4.1f]", event_x, event_y, x1, y1, x2, y2);
				 	}
				 	i++;
				 }
				 
				if ((obj->selected_word >= 0)&&(!item_found)) // ... if there is something selected and we didn't click an item
				{
					// turned off as it de-selects words, when a button is clicked
//					char* txt;
//					g_object_get (obj->tf_words[obj->selected_word], "text", &txt, NULL);
//					obj->ChangeTextColorMode (obj->tf_words[obj->selected_word], TEXT_COLOR_MODE_NORMAL, strlen (txt));
//					obj->selected_word = -1;
//					g_free (txt);
				}
				
				//if (obj->SHOW_DEBUG) g_debug ("  - event end.");
				
				return false; // propagate this event further
			}
			if (event->button.button == 3) // RIGHT MOUSE - clear textfield
			{
//				if (obj->SHOW_DEBUG) g_debug ("Right Mouse pressed - clearing textfield");
//				obj->ClearTextfield ();
//				obj->ClearAuthorInfo ();
//				obj->DrawTextfield ();
//				obj->DrawAuthorInfo ();
//				obj->story_id = -1;
//				obj->SwitchButtons_StoryClosed ();
				return true;
			}
			if (event->button.button == 2) // MIDDLE MOUSE - cycle pages
			{
				// check if some story is loaded
				if (obj->story_id == -1)
				{
					//if (obj->SHOW_DEBUG)
					 g_debug ("Cycle pages: no story loaded");
					return false;
				}
				
				if (obj->SHOW_DEBUG) g_debug ("Middle Mouse pressed - cycling pages");
				int err = obj->LoadPage (obj->page + 1);
				if (err == eLoadPageNotExist)
				{
					obj->LoadPage (0);
				}
				return true;
			}
			break;
		}
		
		// !Disabled!
//		case GDK_2BUTTON_PRESS:
//		{
//			if (event->button.button == 3) // RIGHT MOUSE
//			{
//				obj->LoadText ();
//				return true;
//			}
//			break;
//		}
		
		default:
			return false;
	}
	
	return false;
}
gint Canvas::OnButtonEvent (GnomeCanvasItem* item, GdkEvent* event, Canvas* obj)
{
	// tmp test event, doesn't do anything
	
	switch (event->type)
	{
		case GDK_BUTTON_PRESS:
		{
//			g_print ("Button clicked!\n");
			return true;
		}
		default:
			return false;
	}
}

gint Canvas::OnNextPageMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// This callback is to handle click on the next page.
	// --------------------------------------------------------------------
	if (!obj->button_next->enabled) return false;
	
	if (event->type != GDK_BUTTON_RELEASE)
	{
		return false;
	}
	
	int err = obj->LoadPage (obj->page + 1);
	if (err == eLoadPageNotExist)
	{
		// do nothing -- we are at the last page
		if (obj->SHOW_DEBUG) g_debug ("NextPageClick: Page %d does not exist", obj->page + 1);
	}
	return true;
}
gint Canvas::OnPrevPageMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// This callback is to handle click on the prev page.
	// --------------------------------------------------------------------
	if (!obj->button_prev->enabled) return false;
	
	if (event->type != GDK_BUTTON_RELEASE)
	{
		return false;
	}
	
	if (obj->page > 0)
	{
		obj->LoadPage (obj->page - 1);
	}
	return true;
}

gboolean gc_sound_play_finished;
gint Canvas::OnLstBoxMouseUp (GnomeCanvasItem* lst_box, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Click on listbox will load that article.
	// --------------------------------------------------------------------
	if (!obj->lb_select->enabled) return false;			// ignore disabled components
	// turned off - single click is enough to load // if (event->type != GDK_2BUTTON_PRESS) return false;	// react only on double clicks
	if (event->type != GDK_BUTTON_RELEASE) return false; // react only on mouse button releases
	if (event->button.button != 1) return false;		// LEFT MOUSE BUTTON only
	
	// get world coords and convert then into coords relative to lstbox->root_grp
	double event_x = event->button.x;
	double event_y = event->button.y;
	gnome_canvas_item_w2i (obj->lb_select->root_grp, &event_x, &event_y);
	
	// now, get clicked item
	CanvasListboxItem *item;
	item = canvas_listbox_get_item_at (obj->lb_select, event_x, event_y);
	if (item == NULL) return false;
	
	// tmp check
	if (obj->SHOW_DEBUG) g_debug ("Listbox item clicked: '%s', data: '%s'\n", item->text, item->data);
	
	// load file
	obj->ClearTextfield ();
	obj->ClearAuthorInfo ();
	obj->DrawAuthorInfo ();
	obj->DrawTextfield ();
	int err = obj->LoadFromFile (item->data);
	if ((err == -1) && (obj->SHOW_MESSAGES))
	{
		g_message ("Failed to load article '%s' from file '%s' (generic error)", item->text, item->data);
		obj->story_id = -1;
	}
	else
		obj->story_id = item->id;
	
	// create Sound class
	char *resx_path = g_strdup_printf ("%s", obj->config->PathResx());
	char *story_dir = g_strdup_printf ("%s", obj->story_list[obj->story_id]->dir_name);
	
	if (obj->sound != NULL) obj->sound->Stop ();
	obj->sound = new Sound (resx_path, story_dir);
	
	// now play story header
	char *header_wav = g_strconcat (obj->config->PathResx(), G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, D_PATH_MP3, G_DIR_SEPARATOR_S, D_HEADER_WAV_FILE, NULL);
	
	#if GCOMPRIS_API
		// gc_sound_play_ogg (header_wav); // spoils sounds in GCompris - reason: unknown
		
		// copied from PlayClick event - GCompris version
		gc_sound_play_finished = false;
		gc_sound_play_ogg_cb (header_wav, gc_play_cb); // verzia s callbackom
		g_timeout_add (100, (GSourceFunc)wait_gc_play, obj);
	#else
		obj->sound->PlayStart (header_wav);
	#endif
	
	g_free (resx_path);
	g_free (story_dir);
	g_free (header_wav);
	
	obj->SwitchButtons_LoadedStory ();
	
	return true;
}
gint Canvas::OnLoadStoryClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Called when clicked on 'load article' button.
	// --------------------------------------------------------------------
	if (!obj->btnLoadStory->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	obj->SwitchButtons_Load ();
	
	return true;
}
gint Canvas::OnLoadCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Cancel loading article.
	// --------------------------------------------------------------------
	if (!obj->btnLoadStory->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	obj->SwitchButtons_LoadCancel ();
	
	return true;
}

gint Canvas::OnPlayClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Play selected word.
	//
	// - check if some word is selected
	// - get .mp3 file for this word
	// - create thread to play this word + disable play button (or change it
	//   to cancel button
	// - enable play button again
	// 
	//  Note that this event behaves differently in GCompris and in vanilla.
	// 
	// Note the global variable gc_sound_play_finished, declared above.
	// There is no way how to avoid using global variable here, as the
	// GCompris callback function for gc_sound_play_ogg_cb is declared as
	// typedef void (*GcomprisSoundCallback) (gchar *);
	// Has one mandatory-typed argument of type gchar*. There is no way how to pass
	// arbitrary pointer (eg. (Canvas*)) to the function, thus there is but one
	// way how to inform Canvas class that the playback has finished.
	// --------------------------------------------------------------------
	
	// Common stuff
	if (!obj->btnPlay->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	// no story loaded
	if (obj->story_id == -1)
	{
		if (obj->SHOW_DEBUG) g_debug ("Playback: no story loaded.");
		canvas_button_disable (obj->btnPlay);
		return false;
	}
	
	// no word selected
	if (obj->selected_word == -1)
	{
		if (obj->SHOW_DEBUG) g_debug ("Playback: no word selected.");
		canvas_button_disable (obj->btnPlay);
		return false;
	}
	
	char *txt;
	g_object_get (obj->tf_words[obj->selected_word], "text", &txt, NULL);
	
	sWord ww = obj->tf_swords[obj->selected_word];
	
	// get word's path
	char *story_dir = g_strdup_printf ("%s", obj->story_list[obj->story_id]->dir_name);
	char *mp3_file = g_strconcat (obj->config->PathResx(), G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, D_PATH_MP3, G_DIR_SEPARATOR_S, "word", ww.ind.c_str(), ".wav", NULL);
	
	
	// GCompris API
	#if GCOMPRIS_API
		obj->SwitchButtons_Play ();		
		
		gc_sound_play_finished = false;
		gc_sound_play_ogg_cb (mp3_file, gc_play_cb); // verzia s callbackom
		g_timeout_add (100, (GSourceFunc)wait_gc_play, obj);
	#else
		if (obj->sound == NULL)
		{
			if (obj->SHOW_WARNINGS) g_warning ("Sound class not initialized");
			return false;
		}
		obj->SwitchButtons_Play ();
		obj->sound->PlayStart (mp3_file);
		g_timeout_add (100, (GSourceFunc)wait_a_sec, obj);
	#endif
	
	// free allocated variabled
	g_free (txt);
	g_free (story_dir);
	g_free (mp3_file);
	
	return true;
}
gint Canvas::OnPlayCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Cancels playback on the fly. Works for Read button as well.
	// --------------------------------------------------------------------
	if (!obj->btnPlayCancel->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	#if GCOMPRIS_API
		// TODO fix this!
		gc_sound_pause ();
		gc_sound_play_finished = true;
	#else
		if (obj->sound->Playback_in_progress())
		{
			g_print ("Stopping playback now!\n");
			obj->sound->PlayStop();
		}
	#endif
	
	return true;
}
gint Canvas::OnPushMouseDown (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// TODO initial checkings
	if (!obj->btnPush->enabled) return false;
	if (event->type != GDK_BUTTON_PRESS) return false;
	if (event->button.button != 1) return false;
	
	// no story loaded
	if (obj->story_id == -1)
	{
		if (obj->SHOW_DEBUG) g_debug ("Record: no story loaded.");
		canvas_button_disable (obj->btnPush);
		return false;
	}
	
	// no word selected
	if (obj->selected_word == -1)
	{
		if (obj->SHOW_DEBUG) g_debug ("Record: no word selected.");
		canvas_button_disable (obj->btnPush);
		return false;
	}
	
	// --------------------------------------------------------------------
	// Start recording.
	// --------------------------------------------------------------------
	if (obj->sound == NULL)
	{
		if (obj->SHOW_WARNINGS) g_warning ("Sound class not initialized");
		return false;
	}
	
	#if GCOMPRIS_API
		if (obj->sound != NULL) obj->sound->Stop();
		obj->sound->RecStart ();
	
	#else
		
		if (obj->sound != NULL) obj->sound->Stop();
		obj->sound->RecStart ();
	#endif
	
	
	return true;
}
gint Canvas::OnPushMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Stop recording.
	// --------------------------------------------------------------------
	if (!obj->btnPush->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
//	#if GCOMPRIS_API
	
	
//	#else
		if (obj->sound != NULL)
		{
			obj->sound->Stop ();
		}
		g_timeout_add (50, (GSourceFunc)wait_for_result, obj);
//	#endif
	
	
	return true;
}
gint Canvas::OnReadMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Play the whole loaded story.
	// --------------------------------------------------------------------
	if (!obj->btnRead->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	// check story is loaded
	if (obj->story_id == -1)
	{
		if (obj->SHOW_WARNINGS) g_warning ("Read button event: no story loaded.");
		// TODO this is not sufficient, we need to make sure at least Open button is enabled..
		canvas_button_disable (obj->btnRead);
		canvas_button_enable (obj->btnLoadStory);
		return false;
	}
	
	// check relevant page is loaded
	if (obj->page < 0)
	{
		if (obj->SHOW_WARNINGS) g_warning ("Read button event: incorrect page number (no page loaded?).");
		canvas_button_disable (obj->btnRead);
		canvas_button_enable (obj->btnLoadStory);
		return false;
	}
	
	// prepare for playback
	obj->MarksReset ();
	obj->selected_word = -1;
	obj->SwitchButtons_Read ();
	if (obj->tf_words_selectable) obj->tf_words_selectable = false;
	
	
	// convert page number to string
	string page_nr = "";
	stringstream ss;
	ss << (obj->page+1);
	
	if (obj->page+1 < 100) page_nr.append ("0");
	if (obj->page+1 <  10) page_nr.append ("0");
	page_nr.append (ss.str ());
	
	// get file name to play
	char *story_dir = g_strdup_printf ("%s", obj->story_list[obj->story_id]->dir_name);
	char *file_name = g_strconcat ( obj->config->PathResx (), G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, D_PATH_MP3, G_DIR_SEPARATOR_S, "page", page_nr.c_str(), ".wav", NULL);
//	g_print ("trying to open a file: '%s'\n", file_name);
	
	// perform playback
	#if GCOMPRIS_API
		gc_sound_play_finished = false;
		gc_sound_play_ogg_cb (file_name, gc_play_cb); // verzia s callbackom
		g_timeout_add (100, (GSourceFunc)wait_gc_read, obj);
	#else
		if (obj->sound == NULL)
		{
			if (obj->SHOW_WARNINGS) g_warning ("Sound class not initialized");
			return false;
		}
		obj->sound->PlayStart (file_name);
		g_timeout_add (100, (GSourceFunc)wait_a_sec_read, obj);
	#endif
	
	
	
	g_free (story_dir);
	g_free (file_name);
	
	
	////////////////////////////////
	// tmp - select words one by one
	////////////////////////////////
	/* COMMENTED OUT! This code may be useful when playing labeled wavs. So far (v0.9.4), this is not implemented.
	// make sure selectability is turned off + init marked_word
	if (obj->tf_words_selectable) obj->tf_words_selectable = false;
//	if (obj->marked_word == -1) obj->marked_word++; // put marker to the first word // this marked_word points to the CURRENT element
	// TODO Checks on array existence and existence of at least one element
	
	// move the marker to the next word
	obj->marked_word++; // in case this was -1, we are now on the first element
	
	// first, find last selected word
	//g_print ("marked_word: %d\n", obj->marked_word);
	int marked_word = obj->marked_word;
	
	// check we are within the array bounds
	if (marked_word > (int) obj->tf_words.size())
	{
		//g_print ("MARK is past the last element, reseting and turning page...\n");
		// do not forget to undo any changes, i.e. decrement marked_word
		obj->marked_word--;
		// now reset
		obj->MarksReset ();
		return false;
	}
	
	if (marked_word == (int) obj->tf_words.size ()) // if we are past the last element of tf_words
	{	// we mark the last as passed
		//g_print ("Last word reached and marked as passed!\n");
		obj->ChangeTextMarkMode (obj->tf_words[marked_word - 1], MARK_MODE_PASSED);
		return false;
	}
	
	// if we had just begon, mark the first word as current
	if (marked_word == 0) // we point on the first element in array -> mark it as current
	{
		//g_print ("(FIRST MARK): Size of tf_words: %d, size of tf_swords: %d and of tf_marks: %d\n", obj->tf_words.size(), obj->tf_swords.size(), obj->tf_marks.size());
		obj->ChangeTextMarkMode (obj->tf_words[marked_word], MARK_MODE_CURRENT);
	}
	// else mark the previous word as passed and mark current as CURRENT
	else
	{
		//g_print ("(CONSECUTIVE MARK): ...\n");
		obj->ChangeTextMarkMode (obj->tf_words[marked_word - 1], MARK_MODE_PASSED);
		obj->ChangeTextMarkMode (obj->tf_words[marked_word], MARK_MODE_CURRENT);
	}
	*/
	
	return false;
}
gint Canvas::OnDictateMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Dictate the page starting on selected word.
	// --------------------------------------------------------------------
	if (!obj->btnDictate->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	// check story is loaded
	if (obj->story_id == -1)
	{
		if (obj->SHOW_WARNINGS) g_warning ("Read button event: no story loaded.");
		// TODO this is not sufficient, we need to make sure at least Open button is enabled..
		canvas_button_disable (obj->btnDictate);
		canvas_button_enable (obj->btnLoadStory);
		return false;
	}
	
	// check relevant page is loaded
	if (obj->page < 0)
	{
		if (obj->SHOW_WARNINGS) g_warning ("Read button event: incorrect page number (no page loaded?).");
		canvas_button_disable (obj->btnDictate);
		canvas_button_enable (obj->btnLoadStory);
		return false;
	}
	
	// determine word, where the dictation begins and handle selecting/marking
	obj->MarksReset ();
	if (obj->selected_word == -1)
	{
		obj->marked_word = 0;
	}
	else
	{
		obj->marked_word = obj->selected_word;
	}
	// dictate heading only on first page
	if (obj->page > 0) // we are past the first page
	{
		while (obj->tf_swords[obj->marked_word].isHeadline && obj->marked_word < (int) obj->tf_swords.size())
			obj->marked_word++;
	}
	
	// prepare for action
	obj->selected_word = -1;
	obj->dictate_in_progress = true;
	obj->tf_words_selectable = false;
	obj->SwitchButtons_Dictate ();
	obj->ChangeTextMarkMode (obj->tf_words[obj->marked_word], MARK_MODE_CURRENT);
	
	
	gchar *ind = g_strdup (obj->tf_swords[obj->marked_word].ind.c_str());
	gchar *story_dir = g_strdup (obj->story_list[obj->story_id]->dir_name);
	gchar *file_name = g_strconcat (obj->config->PathResx (), G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, D_PATH_MP3, G_DIR_SEPARATOR_S, "word", ind, ".wav", NULL);
	g_print ("Dictating file: '%s'\n", file_name);
	
	#if GCOMPRIS_API
		obj->dictate_paused = true;
		
		gc_sound_play_finished = false;
		gc_sound_play_ogg_cb (file_name, gc_play_cb);
		g_timeout_add (100, (GSourceFunc)wait_for_dictate, obj);
		g_timeout_add (D_DICTATE_PAUSE, (GSourceFunc)wait, obj);
	#else
		obj->sound->PlayStart (file_name);
		g_timeout_add (100, (GSourceFunc)wait_for_dictate, obj);
		obj->dictate_paused = true;
		g_timeout_add (D_DICTATE_PAUSE, (GSourceFunc)wait, obj);
	#endif
	
	g_free (ind);
	g_free (story_dir);
	g_free (file_name);
	
	return false;
}
gint Canvas::OnDictateCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj)
{
	// --------------------------------------------------------------------
	// Cancel during dictating..
	// --------------------------------------------------------------------
	if (!obj->btnDictateCancel->enabled) return false;
	if (event->type != GDK_BUTTON_RELEASE) return false;
	if (event->button.button != 1) return false;
	
	obj->dictate_in_progress = false;
	if (obj->sound->Playback_in_progress())
	{
		g_print ("Stopping playback now!\n");
		obj->sound->PlayStop();
	}
	return true;
}

// timer functions
gboolean wait (Canvas &cvs)
{
	cvs.dictate_paused = false;
	return false;
}
gboolean wait_a_sec (Canvas &cvs)
{
	// timeout function for Play
	if (cvs.sound == NULL)
	{
		cvs.SwitchButtons_PlayCancel ();
		return false;
	}
	
	if (cvs.sound->Playback_in_progress())
	{
		return true;
	}
	else
	{
		cvs.SwitchButtons_PlayCancel ();
		return false; // end this timeout
	}
}
gboolean wait_a_sec_read (Canvas &cvs)
{
	// timeout function for Read
	if (cvs.sound == NULL)
	{
		cvs.SwitchButtons_ReadCancel ();
		return false;
	}
	
	if (cvs.sound->Playback_in_progress())
	{
		return true;
	}
	else
	{
		cvs.SwitchButtons_ReadCancel ();
		return false; // end this timeout
	}
}
gboolean wait_for_result (Canvas &cvs)
{
	if (cvs.sound == NULL)
	{
		g_warning ("wait_for_result: Canvas.sound not initialized! Cancelling...");
		return false;
	}
	
	if (cvs.sound->CheckLastResult())
	{
		cvs.SwitchButtons_EnableAll ();
		cvs.HandleResult ();
		return false;
	}
	else
	{
		return true;
	}
}
gboolean wait_for_dictate (Canvas &cvs)
{
	// timeout for Dictate button
	#if GCOMPRIS_API
	
	
	#else
		if (cvs.sound == NULL)
		{
			cvs.SwitchButtons_DictateCancel ();
			return false; // end this timeout
		}
	#endif
		
		if (cvs.dictate_in_progress) // keep dictating
		{
		
		// check playback is still in progress
		#if GCOMPRIS_API
			if (!gc_sound_play_finished)
		#else
			if (cvs.sound->Playback_in_progress())
		#endif
		
			{
				return true;
			}
			else // playback in finished
			{
				if (cvs.dictate_paused)
				{
					// waiting for pause to end
					return true;
				}
				else
				{
					// mark current word as passed
					cvs.ChangeTextMarkMode (cvs.tf_words[cvs.marked_word], MARK_MODE_PASSED);
					// move cursor to the next word
					cvs.marked_word++;
					// check we are in the array bounds
					if (cvs.marked_word == (int) cvs.tf_words.size())
					{
						// cancel dictating
						cvs.SwitchButtons_DictateCancel ();
						cvs.MarksReset ();
						cvs.tf_words_selectable = true;
						cvs.dictate_in_progress = false;
						
						return false; // end this timeout
					}
					// mark next word as current
					cvs.ChangeTextMarkMode (cvs.tf_words[cvs.marked_word], MARK_MODE_CURRENT);
					
					// get file name of next word
					gchar *ind = g_strdup (cvs.tf_swords[cvs.marked_word].ind.c_str());
					gchar *story_dir = g_strdup (cvs.story_list[cvs.story_id]->dir_name);
					gchar *file_name = g_strconcat (cvs.config->PathResx (), G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, D_PATH_MP3, G_DIR_SEPARATOR_S, "word", ind, ".wav", NULL);
					g_print ("Dictating file: '%s'\n", file_name);
					
					// play word
					#if GCOMPRIS_API
						gc_sound_play_finished = false;
						gc_sound_play_ogg_cb (file_name, gc_play_cb);
					#else
						cvs.sound->PlayStart (file_name);
					#endif
					//set pause
					cvs.dictate_paused = true;
					g_timeout_add (D_DICTATE_PAUSE, (GSourceFunc)wait, &cvs);
					
					g_free (ind);
					g_free (story_dir);
					g_free (file_name);
					return true;
				}
			}
		}
		else
		{
			// stop dictating
			#if !GCOMPRIS_API
				cvs.sound->Stop ();
			#endif
			cvs.SwitchButtons_DictateCancel ();
			cvs.tf_words_selectable = true;
			cvs.MarksReset ();
			return false; // end this timeout
		}
}

MarkMode get_mark_mode (double perc)
{
	// --------------------------------------------------------------------
	// Returns marking mode based on evaluated recognition.
	// --------------------------------------------------------------------
	MarkMode m;
	
	if (perc < 20.0)					  m = MARK_MODE_0;
	else if (perc >= 20.0 && perc < 30.0) m = MARK_MODE_20;
	else if (perc >= 30.0 && perc < 40.0) m = MARK_MODE_30;
	else if (perc >= 40.0 && perc < 50.0) m = MARK_MODE_40;
	else if (perc >= 50.0 && perc < 60.0) m = MARK_MODE_50;
	else if (perc >= 60.0 && perc < 70.0) m = MARK_MODE_60;
	else if (perc >= 70.0 && perc < 80.0) m = MARK_MODE_70;
	else if (perc >= 80.0 && perc < 90.0) m = MARK_MODE_80;
	else if (perc >= 90.0 && perc < 95.0) m = MARK_MODE_90;
	else if (perc >= 95.0)				  m = MARK_MODE_95;
	
	return m;
}
char* get_mark_color (double perc)
{
	// --------------------------------------------------------------------
	// Returns marking color based on evaluated recognition.
	// --------------------------------------------------------------------
	char* mark_color;
	if (perc < 20.0)					  mark_color = MM_COLOR_0;
	else if (perc >= 20.0 && perc < 30.0) mark_color = MM_COLOR_20;
	else if (perc >= 30.0 && perc < 40.0) mark_color = MM_COLOR_30;
	else if (perc >= 40.0 && perc < 50.0) mark_color = MM_COLOR_40;
	else if (perc >= 50.0 && perc < 60.0) mark_color = MM_COLOR_50;
	else if (perc >= 60.0 && perc < 70.0) mark_color = MM_COLOR_60;
	else if (perc >= 70.0 && perc < 80.0) mark_color = MM_COLOR_70;
	else if (perc >= 80.0 && perc < 90.0) mark_color = MM_COLOR_80;
	else if (perc >= 90.0 && perc < 95.0) mark_color = MM_COLOR_90;
	else if (perc >= 95.0)				  mark_color = MM_COLOR_95;
	
	return mark_color;
}
char* Canvas::GetMarkModeColor (MarkMode mode)
{
	// --------------------------------------------------------------------
	// Returns marking color based on mark mode.
	// --------------------------------------------------------------------
	switch (mode)
	{
		case MARK_MODE_NONE:
		{
			return MARK_COLOR_NONE;
			break;
		}
		
		case MARK_MODE_CURRENT:
		{
			return MARK_COLOR_CURRENT;
			break;
		}
		
		case MARK_MODE_PASSED:
		{
			return MARK_COLOR_PASSED;
			break;
		}
		
		case MARK_MODE_0:
		{
			return MM_COLOR_0;
			break;
		}
		
		case MARK_MODE_20:
		{
			return MM_COLOR_20;
			break;
		}
		
		case MARK_MODE_30:
		{
			return MM_COLOR_30;
			break;
		}
		
		case MARK_MODE_40:
		{
			return MM_COLOR_40;
			break;
		}
		
		case MARK_MODE_50:
		{
			return MM_COLOR_50;
			break;
		}
		
		case MARK_MODE_60:
		{
			return MM_COLOR_60;
			break;
		}
		
		case MARK_MODE_70:
		{
			return MM_COLOR_70;
			break;
		}
		
		case MARK_MODE_80:
		{
			return MM_COLOR_80;
			break;
		}
		
		case MARK_MODE_90:
		{
			return MM_COLOR_90;
			break;
		}
		
		case MARK_MODE_95:
		{
			return MM_COLOR_95;
			break;
		}
		
		default:
		{
			return MARK_COLOR_NONE;
			break;
		}
	}
}
GdkPixbuf* Canvas::GetMarkedPixbuf (MarkMode mode)
{
	// --------------------------------------------------------------------
	// Returns pixbuf pointer based on current marked mode.
	// --------------------------------------------------------------------
	
	switch (mode)
	{
		case MARK_MODE_0:
		{
			return smile5;
			break;
		}
		
		case MARK_MODE_20:
		{
			return smile4;
			break;
		}
		
		case MARK_MODE_30:
		{
			return smile4;
			break;
		}
		
		case MARK_MODE_40:
		{
			return smile3;
			break;
		}
		
		case MARK_MODE_50:
		{
			return smile3;
			break;
		}
		
		case MARK_MODE_60:
		{
			return smile2;
			break;
		}
		
		case MARK_MODE_70:
		{
			return smile2;
			break;
		}
		
		case MARK_MODE_80:
		{
			return smile1;
			break;
		}
		
		case MARK_MODE_90:
		{
			return smile1;
			break;
		}
		
		case MARK_MODE_95:
		{
			return smile1;
			break;
		}
		
		
		default:
		{
			return NULL;
		}
	}
	
}


// GCompris API only function
#if GCOMPRIS_API
void gc_play_cb (gchar* file)
{
	// --------------------------------------------------------------------
	// This functions is called as callback by function gc_sound_play_ogg_cb
	// when it finishes its playback. Then, it sets global variable
	// gc_sound_play_finished to true. This indicates to Canvas, that the
	// playback is finished (see function wait_gc_play).
	// --------------------------------------------------------------------
	gc_sound_play_finished = true;
}
#endif

gboolean wait_gc_play (Canvas &cvs)
{
	// --------------------------------------------------------------------
	// This is a timeout function. It waits for function gc_play_cb to set
	// global variable gc_sound_play_finished to true. Then, it informs
	// Canvas class to finilize playback.
	// Note that this is used only if ReadTutor is compiled with GCOMPRIS
	// API support.
	// --------------------------------------------------------------------
	if (gc_sound_play_finished)
	{
		cvs.SwitchButtons_PlayCancel ();
		return false;
	}
	else
	{
		return true;
	}
}
gboolean wait_gc_read (Canvas &cvs)
{
	// --------------------------------------------------------------------
	// Same as 'wait_gc_play', but for Read button.
	// --------------------------------------------------------------------
	if (gc_sound_play_finished)
	{
		cvs.SwitchButtons_ReadCancel ();
		return false;
	}
	else
	{
		return true;
	}
}




