#include "sound.h"

static gboolean operator== (PhonetEntry a, PhonetEntry b);

Sound::Sound (char* resx_path, char* current_story_dir_name)
{
	
	end = true;
	playback_progress = false;
	playback = NULL;
	playback_file_name = NULL;
	
//	work_dir_name = "snd_rec";
	work_dir_name = D_PATH_HTK;
	path = g_strdup_printf ("%s", resx_path);
	work_dir_path = g_strconcat (resx_path, G_DIR_SEPARATOR_S, work_dir_name, NULL);
	story_dir = g_strdup_printf ("%s", current_story_dir_name);
	
//	GeneratePhoneticDict ("džungle");
//	GeneratePhoneticDict ("chodili");
	
	
}
Sound::~Sound ()
{
	// if (playback != NULL) {} // TODO playback terminate
	
	if (playback_file_name != NULL) g_free (playback_file_name);
	g_free (work_dir_name);
	g_free (work_dir_path);
	g_free (story_dir);
	g_free (path);
}

// PUBLIC
int Sound::PlayStart (char* file_name) // file_name - full path and filename of sound file to play
{
	playback_progress = true;
	
	// TODO handle errors	// params: GThreadFunc func, gpointer data, gboolean joinable (waits for special function to terminate), GError **error
	playback_file_name = g_strdup_printf ("%s", file_name);
//	playback = g_thread_create ((GThreadFunc)play_start_play, this, false, NULL); // using "play" shell command
	playback = g_thread_create ((GThreadFunc)play_start_new, this, false, NULL);
	if (playback == NULL) { g_print ("Error - thread not created\n"); return -1; } // TODO Finish error codes
	else { g_print ("Thread created successfully\n"); }
	
	return 0;
}
void Sound::PlayStop ()
{
	playback_progress = false;
}
int Sound::RecStart ()
{
	recording = g_thread_create ((GThreadFunc)rec_start, this, false, NULL);
	if (recording == NULL) { g_print ("RecStart(): Error - thread not created\n"); return -1; } // TODO Finish error codes
//	else { g_print ("RecStart(): Thread created successfully\n"); }
	
	return 0;
}
void Sound::Stop ()
{
	end = true;
}

// GCOMPRIS RECORDING METHOD
void Sound::RecStart_gc ()
{
	g_print (" - Starting RecStart_gc()\n");
	// --------------------------------------------------------------------
	// This method needs to be reworked somehow. It mirrors function rec_start().
	// The only difference (so far) is that the while cycle doesn't depend
	// on end variable, but it has fixed number of cycles.
	// --------------------------------------------------------------------
	int audio_fd;
	//int fd;
	
	// --------------------------------------------------------------------
	// Initialization.
	// --------------------------------------------------------------------
	g_print (" - Initialization\n");
	if ((audio_fd = open (AUDIO_DEV, O_RDONLY, 0)) < 0)
	{
      perror (AUDIO_DEV);
      g_thread_exit (NULL);
    }
    int f, g;
    f = g = AFMT_S16_LE;
    if (ioctl (audio_fd, SNDCTL_DSP_SETFMT, &f) < 0) printf ("InitAudio: error setting sample format\n");
    if (f != g) printf ("InitAudo: unable to set 16 bit sample format\n");
    f = g = 1;
    if (ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &f) < 0) printf ("InitAudio: error setting audio channel\n");
    if (f != g) printf ("InitAudo: unable to set %d chanels (actually = %d)\n", g, f);
    int channels = f;
    f = g = 16000;
    if (ioctl (audio_fd, SNDCTL_DSP_SPEED, &f) == -1) printf ("InitAudio: error setting sampling rate\n");
    if (f != g) printf ("InitAudo: unable to set %d sample rate (actually = %d)\n", g, f);
    int samplerate = f;
    
    // --------------------------------------------------------------------
	// Recording.
	// --------------------------------------------------------------------
	g_print (" - Recording\n");
    char *wavfile;
    wavfile = g_strconcat (work_dir_path, G_DIR_SEPARATOR_S, "tmp.wav", NULL);
    SF_INFO sfinfo = {0, samplerate, 1, SF_FORMAT_WAV | SF_FORMAT_PCM_16, 0, 0};
    SNDFILE *sf;
    sf = sf_open (wavfile, SFM_WRITE, &sfinfo);
    if (sf == NULL) printf ("wavfile %s hasn't been opened.\n", wavfile);
    
    short buf[channels];
    char *ptr;
    ptr = (char*)buf;
    
	int count = 0;
	while (count < D_GC_RECORD_TIME)
	{         
		if (read (audio_fd, ptr, 2 * channels) != 2 * channels)
			g_print ("Error reading from OSS.\n");
		if (sf_write_short (sf, buf, 1) != 1)
			g_print ("Error writing to file.\n");
		count++;
	}
	sf_close (sf);
	close (audio_fd);
	
	
	// --------------------------------------------------------------------
	// Recognition.
	// --------------------------------------------------------------------
	g_print (" - Recognition\n");
	gchar *stdout = NULL;
	gchar *stderr = NULL;
	gint exit_status;
	GError *cmd_err = NULL;
	gchar *working_directory = g_strdup_printf ("%s", work_dir_path);
	gchar *phonet_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, ".tmp", NULL);
	gchar *lat_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, LAT_FILE, NULL);
	gchar *hvite_cmd = g_strconcat ("./hvite.sh ", lat_file_name, " ", phonet_file_name, NULL); g_print ("hvite_cmd: '%s' spawned in '%s'\n", hvite_cmd, work_dir_path);
	gchar **hvite;
	
	g_shell_parse_argv (hvite_cmd, NULL, &hvite, NULL);
	
	// execute hvite command
	if (!g_spawn_sync (working_directory, hvite, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr, &exit_status, &cmd_err))
	{
		g_warning ("Error spawning shell command! (hvite.sh command)");
	}
	if (cmd_err) g_warning ("Error: %s\n", cmd_err->message);
//	g_print ("stdout: %s\n", stdout);
	if (stderr != NULL)
		g_print ("stderr: %s\n", stderr);
//	g_print ("exit_status: %d\n", exit_status);
	
	// get result (TODO error handling)
	gchar *result, *tmp_str;
	gchar *cmd_cat = g_strconcat ("cat ", working_directory, G_DIR_SEPARATOR_S, "tmp.rec", NULL);
	g_spawn_command_line_sync (cmd_cat, &tmp_str, NULL, NULL, NULL);
	result = g_strcompress (tmp_str);
	// parse result (get the first word)
	istringstream is (result);
	last_result.clear ();
	is >> last_result;
	g_print ("Result: %s\n", last_result.c_str());
    
	// --------------------------------------------------------------------
	// Finalization.
	// --------------------------------------------------------------------
	g_print (" - Finalization\n");
	g_print ("Finalizing recognition...\n");
	g_free (wavfile);
	
	g_free (stderr);
    g_free (stdout);
    if (cmd_err != NULL) g_error_free (cmd_err);
    g_strfreev (hvite);
    g_free (tmp_str);
    g_free (hvite_cmd);
    g_free (phonet_file_name);
    g_free (lat_file_name);
    g_free (result);
    g_print (" - end...\n");
}

gboolean Sound::PhonetToGram ()
{
	// --------------------------------------------------------------------
	// Purpose of this method is to take phonetical dictionary and generate
	// grammar .gram file.
	// Method: For every entry in phonetical dictionary generate one rule
	// of grammar. Add static stuff as needed.
	// - open the phonetical dict. file
	// - read the file's lines
	// - for each line, get the first word (word model id)
	// - - check this word is valid (doesn't start with #)
	// - - check this word is not included in array yet
	// - include this word into array
	// 
	// Generate .gram file
	// 
	// FORMAT OF PHONETIC file
	// - first column: identificator of the model
	// - second column: actual word (note that ids and words equal)
	// - third column: phonetical transcription of the actual word
	// 
	// FORMAT OF .gram FILE
	// $words =
	// <model id>|
	// <model id>|
	// ...
	// <model id>;
	// 
	// (#sil $words #sil)
	// EOF
	// 
	// Note that each model line is terminated with a pipe (|) but the last
	// line, which is terminated by a semi-colon (;).
	// --------------------------------------------------------------------
	
	string line, first;		// line of text from phonetical dict, first word of this line
	string::size_type loc, locc;
	vector<string> rules;	// names of gramatical rules (not formatted for output yet)
	vector<string>::iterator iter;
	rules.resize (0);
	gchar *phonet_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, ".tmp", NULL);
	gchar *gram_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, GRAM_FILE, NULL);
	gboolean add_to_rules = false; // add this current word into rules
	gboolean result = true;
	
	ifstream phonet (phonet_file_name);
	if (phonet.is_open())
	{
		while (getline (phonet, line))
		{
			// Now we need to process this line further.. actually, we only need 
			// the very first word of this line.
			// Get the first word in line and append it into vector 'rules'
			
			// find occurence of the first whitespace
			loc = line.find (" ", 0);
			if (loc == string::npos)
			{
				// no whitespace on this line - do nothing, thus ignoring this line
			}
			else
			{
				// get the first word on the line
				first = line.substr (0, loc);
				// filter strings starting with '#'
				locc = first.find ("#", 0, 1);
				if (locc != string::npos)
				{
					// do nothing
				}
				else
				{
					add_to_rules = true;
					// remove duplicates (disabled)
//					for (iter = rules.begin(); iter != rules.end(); iter++)
//					{
//						if (first.compare (*iter) == 0) add_to_rules = false;
//					}
					// add this word now
					if (add_to_rules)
						rules.push_back (first);
				}
			}
		}
		phonet.close ();
		
		// print contents of the vector 'rules'
//		for (iter = rules.begin(); iter != rules.end(); iter++)
//			cout << *iter;
		
		// Generate output file
		ofstream gram (gram_file_name);
		if (gram.is_open())
		{
			// start generating grammar
			gram << "$words =";
			for (iter = rules.begin(); iter != rules.end(); iter++)
			{
				gram << endl << *iter;
				if ((iter + 1) == rules.end()) // if the next iterator is at the end of vector
				{
//					cout << "Koniec pola detekovany na prvku '" << *iter << "'" << endl;
					gram << ";";
				}
				else
				{
//					cout << "Este su prvy v poli na prvku '" << *iter << "'" << endl;
					gram << "|";
				}
			}
			gram << endl << endl;
			gram << "(#sil $words #sil)" << endl;
		}
		else
		{
			cout << "Unable to open file " << phonet_file_name << endl;
			result = false;
		}
		
	}
	else
	{
		cout << "Unable to open file " << phonet_file_name << endl;
		result = false;
	}
	
	g_free (phonet_file_name);
	g_free (gram_file_name);
	if (result) return true; else return false;
}
gboolean Sound::GramToLat ()
{
	// --------------------------------------------------------------------
	// Generates .lat file out of .gram file.
	// --------------------------------------------------------------------
	GError *cmd_err = NULL;
	gchar *working_directory = g_strdup_printf ("%s", work_dir_path);
	gchar *gram_file = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, GRAM_FILE, NULL);
	gchar *lat_file = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, LAT_FILE, NULL);
	gchar *cmd = g_strconcat ("./HParse ", gram_file, " ", lat_file, NULL);
	//g_print ("parsovaci command: '%s'\n", cmd);
	gchar **hparse;
	gboolean result = true;
	
	g_shell_parse_argv (cmd, NULL, &hparse, NULL);
	if (!g_spawn_sync (working_directory, hparse, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, NULL, &cmd_err))
	{
		g_warning ("Error spawning shell command! (parse gram->lat)");
		result = false;
	}
	if (cmd_err) { g_warning ("Error: '%s'", cmd_err->message); g_error_free (cmd_err); }
	
	g_free (working_directory);
	g_free (gram_file);
	g_free (lat_file);
	g_free (cmd);
	g_strfreev (hparse);
	
	if (result) return true; else return false;
}
vector<PhonetEntry> Sound::GetPhonemes (char* c_word, vector<PhonetEntry>& words_all)
{
	// --------------------------------------------------------------------
	// Reads phonetical dictionary, finds given word and returns its
	// phonetical transcription.
	// Side effect: Loads _all_ transcriptions into words_all vector.
	// --------------------------------------------------------------------
	vector<PhonetEntry> entries;
	PhonetEntry tmp_entry;
	
	string tmp, name, word, line;
	gboolean ok;				// when reading line of text, while this is true, we can proceed parsing the line
	
	// string streams
	istringstream in_ss;
	
	string prototype;
	prototype = g_strdup_printf ("%s", c_word);
	ostringstream prot;			// prototype with brackets
	prot << "[" << prototype << "]";
	
	gchar *phonet_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, NULL);
	
	// open input file
	ifstream phonet (phonet_file_name);
	if (!phonet.is_open())
	{
		g_error ("Sound::GetPhonemes: failed to open phonetical dictionary file '%s'.", phonet_file_name);
	}
	else
	{
		// read input file
		// Format of this file:
		// 
		// <name of word>	<word>		<list_of_phonemes, space separated>
		// ...
		// ...
		// #<foo>
		// #<bar>
		// EOF
		
		// get line from file
		while (getline (phonet, line))
		{
			// discard line starting with #
			if (line.size() >= 1 && line[0] == '#')
			{
				// do nothing, ignore this line
			}
			else
			{
				// parse this line
				ok = false;
				entry_clear (&tmp_entry);
				name.clear(); word.clear (); tmp.clear (); in_ss.clear ();
				in_ss.str(line);
				
				// read words one by one, start with word_name and word itself
				if (in_ss >> name >> word)
				{
					ok = true;
				}
				
				// we have two words, now compare the second word with our prototype
				if (word != prot.str())
				{
					ok = false;
				}
				
				// iterate to the end of line and save all phonemes + prepare tmp_entry
				while (in_ss >> tmp)
				{
					tmp_entry.phonemes.push_back (tmp);
				}
				tmp_entry.name = trim_word (name.c_str());
				// remove brackets [] from the word
				tmp_entry.word = trim_word (word.substr (1, word.size() - 2).c_str());
				
				// now append this phonemes to the vector
				if (ok)
				{
					entries.push_back (tmp_entry);
				}
				
				// SIDE EFFECT FEATURE: save it into the output vector all_words as well
				words_all.push_back (tmp_entry);
			}
		}
		phonet.close ();
	}
	
	g_free (phonet_file_name);
	return entries;
}
string Sound::GetModelName (char* c_word)
{
	string name, word, line;
	gboolean found = false;
	
	istringstream in_ss;
	
	string prototype = g_strdup_printf ("%s", c_word);
	ostringstream prot;
	prot << "[" << prototype << "]";
	
	gchar *phonet_file_name = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, NULL);
	
	// open input file
	ifstream phonet (phonet_file_name);
	if (!phonet.is_open())
	{
		g_error ("Sound::GetModelName: failed to open phonetical dictionary file '%s'.", phonet_file_name);
	}
	else
	{
		while (getline (phonet, line) && !found)
		{
			in_ss.clear (); name.clear (); word.clear ();
			in_ss.str (line);
			if (in_ss >> name >> word)
			{
				if (word == prot.str()) found = true;
			}
		}
		
		phonet.close ();
	}
	
	g_free (phonet_file_name);
	
	if (found) return name;
	else return "";
}
vector<PhonetEntry> Sound::GeneratePrefixes (PhonetEntry ent)
{
	// --------------------------------------------------------------------
	// Given one entry from phonetical dict, this function attempts to
	// generate all prefixes of the word.
	// --------------------------------------------------------------------
	vector<PhonetEntry> e;
	PhonetEntry tmp_e;
	gchar *word = g_strdup (ent.word.c_str());
	gchar *copy;
	int pos;
	int i, j;
	
	
	//cout << "Word length: " << g_utf8_strlen (ent.word.c_str(), -1) << ", phoneme count: " << ent.phonemes.size() << endl;
	if (g_utf8_strlen (ent.word.c_str(), -1) < (glong) ent.phonemes.size())
		g_warning ("Sound::GeneratePrefixes: word length less then phoneme count");
	if (!g_utf8_validate (ent.word.c_str(), -1, NULL))
		g_warning ("Error, input is not valid utf8! '%s'", ent.word.c_str());
	
	// --------------------------------------------------------------------
	// Iterate through phonemes, for each next phoneme figure out, how many
	// characters to take from the 'word' (some phonemes like ch or dz).
	// - pos local variable means position in the 'word', it is initialized
	//   to 0, but is increased immediately to 1 or 2. "pos-many" characters
	//   (not bytes) will be eaten from 'word' at each iteration and these
	//   characters will form up next prefix of 'word'.
	// - g_utf8_strncpy expects its dest string to be allocated.
	// --------------------------------------------------------------------
	pos = 0;
	for (i = 0; i < (int) ent.phonemes.size(); i++)
	{
		entry_clear (&tmp_e);
		
		// handle phonemes, which include more characters, so we need to read more characters
		// these are: ch, dz, dz~
		// their phonet. transcriptions are: x, (??), dZ
		if ((ent.phonemes[i] == "x") || (ent.phonemes[i] == "dZ")) // these phonemes consist of two characters in 'word'
			pos += 2;
		else
			pos++;
		
		copy = (gchar*) g_malloc (strlen(word) + 1);
		copy = g_utf8_strncpy (copy, word, pos);
		copy[strlen(word)] = '\0';
		
		tmp_e.word = g_strdup (copy);
		for (j = 0; j <= i; j++) tmp_e.phonemes.push_back (ent.phonemes[j]);
		tmp_e.name = g_strdup (copy);
		e.push_back (tmp_e);
		
		g_free (copy);
	}
	
	entry_clear (&tmp_e);
	g_free (word);
	return e;
}
vector<PhonetEntry> Sound::GenerateSuffixes (PhonetEntry ent)
{
	// --------------------------------------------------------------------
	// Given one entry from phonetical dict, this function attempts to
	// generate all prefixes of the word.
	// --------------------------------------------------------------------
	vector<PhonetEntry> e;
	PhonetEntry tmp_e;
	gchar *word = g_strdup (ent.word.c_str());
	gchar *handle = word;
	int pos, len, byte_len, phon_len, i;
	
	
	//cout << "Word length: " << g_utf8_strlen (ent.word.c_str(), -1) << ", phoneme count: " << ent.phonemes.size() << endl;
	if (g_utf8_strlen (ent.word.c_str(), -1) < (glong) ent.phonemes.size())
		g_warning ("Sound::GeneratePrefixes: word length less then phoneme count");
	if (!g_utf8_validate (ent.word.c_str(), -1, NULL))
		g_warning ("Error, input is not valid utf8! '%s'", ent.word.c_str());
	
	len = g_utf8_strlen (word, -1);
	byte_len = strlen (word);
	phon_len = (int) ent.phonemes.size();
	pos = 1;	// position in phonemes
	
	// --------------------------------------------------------------------
	// 
	// --------------------------------------------------------------------
	while (phon_len - pos > 0)
	{
		// If previous phoneme was "x" or "dZ", skip one more character from input 'word'
		// Note: there is always previous phoneme, as 'pos' is initialized to 1, not 0.
		if (ent.phonemes[pos-1] == "x" || ent.phonemes[pos-1] == "dZ")
		{
			handle = g_utf8_next_char (handle);
		}
		handle = g_utf8_next_char (handle);
		
		// now copy rest of the string
		tmp_e.word = g_strdup (handle);
		tmp_e.name = g_strdup (handle);
		for (i = pos; i < (int) ent.phonemes.size(); i++) tmp_e.phonemes.push_back (ent.phonemes[i]);
		
		e.push_back (tmp_e);
		entry_clear (&tmp_e);
			
		pos++;
	}
	
	g_free (word);
	return e;
}
void Sound::GenerateSimilar (char* c_word, vector<PhonetEntry>& e, vector<PhonetEntry> e_all)
{
	// --------------------------------------------------------------------
	// Insert transcriptions of words whose distance from input string
	// c_word is somewhat small. Somewhat means to some % of its length.
	// --------------------------------------------------------------------
	/*
	 * Which strings are going to be included into final dictionary depends
	 * on their distance from the input word c_word. This defines percent,
	 * all words, whose distance from word_c is less then this percent (of
	 * their overall length) will be included.
	 */
	#define WORD_LENGTH_PERC 0.5
	
	string word = g_strdup (c_word);
	int word_size = g_utf8_strlen (c_word, -1);
	string target;
	int dist;
	double perc;
	
	for (int i = 0; i < (int) e_all.size(); i++)
	{
		target = e_all[i].word;
		dist = sdist (word, target);
		perc = (double) dist / (double) word_size;
		
		// debug output
//		int target_size = g_utf8_strlen (target.c_str(), -1);
//		cout << "Considering words: " << c_word << " and " << target << " | dist: " << dist;
//		cout << ", perc: " << perc << "size word,target: " << word_size << ", " << target_size << endl;
		
		// insert word
		if (perc < WORD_LENGTH_PERC)
			e.push_back (e_all[i]);
	}
}
void Sound::RemoveDuplicates (vector<PhonetEntry>& e)
{
	// --------------------------------------------------------------------
	// Removes duplicates from entry vector.
	// --------------------------------------------------------------------
	int i, j;
	vector<PhonetEntry>::iterator jj;	// shadows j, this iterator is needed by vector::erase() method
	i = 0;
	while (i < (int) e.size() - 1)		// do not consider last element in vector, this is going to be different from the other ones
	{
		j = i + 1;
		jj = e.begin();					// set this iterator to the begin() of the vector
		jj += j;						// and now move it j steps (it now points to the same element as j)
		while (j < (int) e.size())
		{
			if (e[i] == e[j])
				e.erase (jj);
			j++; jj++;
		}
		i++;
	}
}
gboolean Sound::GeneratePhoneticDict (char* c_word)
{
	// --------------------------------------------------------------------
	// Generates phonetical dictionary with given word's prefixes added.
	// TODO finish error messages
	// --------------------------------------------------------------------
	vector<PhonetEntry> e;			// vector of entries from phonet dict (of the give c_word)
	vector<PhonetEntry> e_all;		// vector of _all_ transcription entries
	vector<PhonetEntry> output_e;	// vector of entries for output
	vector<PhonetEntry> out_e_pref;	// vector of prefixes for output
	vector<PhonetEntry> out_e_suff;	// vector of suffixes for output
	
	PhonetEntry tmp_e;
	vector<string> tmp_str;
	
	string model_name, word;
	int i, j;						// generic cycle variables
	
	// --------------------------------------------------------------------
	// Initialization.
	// --------------------------------------------------------------------
	gchar* output_phonet_file = g_strconcat (path, G_DIR_SEPARATOR_S, story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, ".tmp", NULL);
	
	// strip c_word of unnecessary characters: space, !, ?, ,, ., ...
	string stripped = trim_word (c_word);
	
	char* cc_word = g_strdup (stripped.c_str());
	e = GetPhonemes (cc_word, e_all);
	
	// --------------------------------------------------------------------
	// Generate prefixes/suffixes.
	// --------------------------------------------------------------------
	for (i = 0; i < (int) e.size(); i++)
	{
		entry_clear (&tmp_e);
		
		tmp_e.name = e[i].name;
		tmp_e.word = e[i].word;
		for (j = 0; j < (int) e[i].phonemes.size(); j++) tmp_e.phonemes.push_back (e[i].phonemes[j]); // add phonemes one by one
		output_e.push_back (tmp_e); // this doesnt have to be added, as suffixes generate also the complete form - just to be sure the complete form is added
		
		out_e_pref = GeneratePrefixes (tmp_e);
		out_e_suff = GenerateSuffixes (tmp_e);
		// add these entries into output_e
		for (j = 0; j < (int) out_e_pref.size(); j++)
			output_e.push_back (out_e_pref[j]);
		for (j = 0; j < (int) out_e_suff.size(); j++)
			output_e.push_back (out_e_suff[j]);
		
		out_e_pref.clear ();
		out_e_suff.clear ();
		entry_clear (&tmp_e);
	}
	
	// --------------------------------------------------------------------
	// Insert transcriptions of all words.
	// --------------------------------------------------------------------
	// DISABLED for strange results
	//for (i = 0; i < (int) e_all.size(); i++) output_e.push_back (e_all[i]);
	
	
	// --------------------------------------------------------------------
	// Insert transcriptions of words whose distance in somewhat small.
	// --------------------------------------------------------------------
	GenerateSimilar (cc_word, output_e, e_all);
	
	// --------------------------------------------------------------------
	// Remove duplicates from the entries.
	// --------------------------------------------------------------------
	RemoveDuplicates (output_e);
	
	
	// --------------------------------------------------------------------
	// Generate output file.
	// --------------------------------------------------------------------
	ofstream output (output_phonet_file);
	if (!output.is_open())
	{
		g_warning ("Sound::GeneratePhDict: Failed to open file '%s'", output_phonet_file);
		return false;
	}
	else
	{
		for (i = 0; i < (int) output_e.size(); i++)
		{
			output << output_e[i].name << "   [";
			output << output_e[i].word << "]   ";
			for (j = 0; j < (int) output_e[i].phonemes.size(); j++) output << output_e[i].phonemes[j] << " ";
			output << endl;
		}
		
		// input static lines
		output << "#sil [#sil] sil" << endl;
		output << "#sp [#sp] sp" << endl;
		
		output.close ();
	}
	
	g_free (output_phonet_file);
	g_free (cc_word);
	return true;
}

// PROPERTIES
gboolean Sound::Playback_in_progress ()
{
	// --------------------------------------------------------------------
	// Returns if playback is in progress.
	// --------------------------------------------------------------------
	return playback_progress;
}
string Sound::GetLastResult ()
{
	string result = last_result;
	last_result.clear ();
	return result;
}
gboolean Sound::CheckLastResult ()
{
	if (last_result != "") return true;
	else return false;
}

// PRIVATE



// OTHER

void play_start_play (Sound &snd)
{
	gchar  *cmd;
	gchar **command;
	
//	if (snd.playback_progress) g_print ("playback_progress true!\n"); else g_print ("playback_progress false\n");
	
	cmd = g_strconcat ("play ", snd.playback_file_name, NULL);
	g_shell_parse_argv (cmd, NULL, &command, NULL);
	g_spawn_sync (NULL, command, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, NULL, NULL);
	g_print ("Finished!\n");
	snd.playback_progress = false;
	
	g_free (cmd);
	g_strfreev (command);
}
void rec_start (Sound &snd)
{
	// --------------------------------------------------------------------
	// Taken from example uloha02.
	// --------------------------------------------------------------------
	int audio_fd;
	//int fd;
	
	// --------------------------------------------------------------------
	// Initialization.
	// --------------------------------------------------------------------
//	g_print ("rec_start: Init..\n");
	if ((audio_fd = open (AUDIO_DEV, O_RDONLY, 0)) < 0)
	{
      perror (AUDIO_DEV);
      g_thread_exit (NULL);
    }
    int f, g;
    f = g = AFMT_S16_LE;
    if (ioctl (audio_fd, SNDCTL_DSP_SETFMT, &f) < 0) printf ("InitAudio: error setting sample format\n");
    if (f != g) printf ("InitAudo: unable to set 16 bit sample format\n");
    f = g = 1;
    if (ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &f) < 0) printf ("InitAudio: error setting audio channel\n");
    if (f != g) printf ("InitAudo: unable to set %d chanels (actually = %d)\n", g, f);
    int channels = f;
    f = g = 16000;
    if (ioctl (audio_fd, SNDCTL_DSP_SPEED, &f) == -1) printf ("InitAudio: error setting sampling rate\n");
    if (f != g) printf ("InitAudo: unable to set %d sample rate (actually = %d)\n", g, f);
    int samplerate = f;
    
    // --------------------------------------------------------------------
	// Recording.
	// --------------------------------------------------------------------
//	g_print ("rec_start: Record..\n");
    char *wavfile;
    wavfile = g_strconcat (snd.work_dir_path, G_DIR_SEPARATOR_S, "tmp.wav", NULL);
    SF_INFO sfinfo = {0, samplerate, 1, SF_FORMAT_WAV | SF_FORMAT_PCM_16, 0, 0};
    SNDFILE *sf;
    sf = sf_open (wavfile, SFM_WRITE, &sfinfo);
    if (sf == NULL) printf ("wavfile %s hasn't been opened.\n", wavfile);
    
    short buf[channels];
    char *ptr;
    ptr = (char*)buf;
    
	snd.end = false;
	while (!snd.end)
	{         
		if (read (audio_fd, ptr, 2 * channels) != 2 * channels)
			g_print ("Error reading from OSS.\n");
		if (sf_write_short (sf, buf, 1) != 1)
			g_print ("Error writing to file.\n");
	}
	sf_close (sf);
	close (audio_fd);
	
	
	// --------------------------------------------------------------------
	// Recognition.
	// --------------------------------------------------------------------
//	g_print ("rec_start: Recognition..\n");
	gchar *stdout = NULL;
	gchar *stderr = NULL;
	gint exit_status;
	GError *cmd_err = NULL;
	gchar *working_directory = g_strdup_printf ("%s", snd.work_dir_path);
	gchar *phonet_file_name = g_strconcat (snd.path, G_DIR_SEPARATOR_S, snd.story_dir, G_DIR_SEPARATOR_S, PHONET_FILE, ".tmp", NULL);
	gchar *lat_file_name = g_strconcat (snd.path, G_DIR_SEPARATOR_S, snd.story_dir, G_DIR_SEPARATOR_S, LAT_FILE, NULL);
	gchar *hvite_cmd = g_strconcat ("./hvite.sh ", lat_file_name, " ", phonet_file_name, NULL); //g_print ("hvite_cmd: '%s' spawned in '%s'\n", hvite_cmd, snd.work_dir_path);
	gchar **hvite;
	
	g_shell_parse_argv (hvite_cmd, NULL, &hvite, NULL);
	
	// execute hvite command
	if (!g_spawn_sync (working_directory, hvite, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr, &exit_status, &cmd_err))
	{
		g_warning ("Error spawning shell command! (hvite.sh command)");
	}
	if (cmd_err) g_warning ("Error: %s\n", cmd_err->message);
//	g_print ("stdout: %s\n", stdout);
//	if (stderr != NULL)
//		g_print ("stderr: %s\n", stderr);
//	g_print ("exit_status: %d\n", exit_status);
	
	// get result (TODO error handling)
	gchar *result, *tmp_str;
	gchar *cmd_cat = g_strconcat ("cat ", working_directory, G_DIR_SEPARATOR_S, "tmp.rec", NULL);
	g_spawn_command_line_sync (cmd_cat, &tmp_str, NULL, NULL, NULL);
	result = g_strcompress (tmp_str);
	// parse result (get the first word)
	istringstream is (result);
	snd.last_result.clear ();
	is >> snd.last_result;
//	g_print ("Result: %s\n", snd.last_result.c_str());
    
	// --------------------------------------------------------------------
	// Finalization.
	// --------------------------------------------------------------------
//	g_print ("Finalizing recognition...\n");
	g_free (wavfile);
	
	g_free (stderr);
    g_free (stdout);
    if (cmd_err != NULL) g_error_free (cmd_err);
    g_strfreev (hvite);
    g_free (tmp_str);
    g_free (hvite_cmd);
    g_free (phonet_file_name);
    g_free (lat_file_name);
    g_free (result);
}
void rec_start_old (Sound &snd)
{
	// --------------------------------------------------------------------
	// Taken from example uloha02.
	// --------------------------------------------------------------------
	int audio_fd;
	//int fd;
	
	// --------------------------------------------------------------------
	// Initialization.
	// --------------------------------------------------------------------
	if ((audio_fd = open (AUDIO_DEV, O_RDONLY, 0)) < 0)
	{
      perror (AUDIO_DEV);
      g_thread_exit (NULL);
    }
    int f, g;
    f = g = AFMT_S16_LE;
    if (ioctl (audio_fd, SNDCTL_DSP_SETFMT, &f) < 0) printf ("InitAudio: error setting sample format\n");
    if (f != g) printf ("InitAudo: unable to set 16 bit sample format\n");
    f = g = 1;
    if (ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &f) < 0) printf ("InitAudio: error setting audio channel\n");
    if (f != g) printf ("InitAudo: unable to set %d chanels (actually = %d)\n", g, f);
    int channels = f;
    f = g = 16000;
    if (ioctl (audio_fd, SNDCTL_DSP_SPEED, &f) == -1) printf ("InitAudio: error setting sampling rate\n");
    if (f != g) printf ("InitAudo: unable to set %d sample rate (actually = %d)\n", g, f);
    int samplerate = f;
    
    // --------------------------------------------------------------------
	// Recording.
	// --------------------------------------------------------------------
    char *wavfile;
    wavfile = g_strconcat (snd.work_dir_path, G_DIR_SEPARATOR_S, "tmp.wav", NULL);
    SF_INFO sfinfo = {0, samplerate, 1, SF_FORMAT_WAV | SF_FORMAT_PCM_16, 0, 0};
    SNDFILE *sf;
    sf = sf_open (wavfile, SFM_WRITE, &sfinfo);
    if (sf == NULL) printf ("wavfile %s hasn't been opened.\n", wavfile);
    
    short buf[channels];
    char *ptr;
    ptr = (char*)buf;
    //int nb, index = 0;
    //audio_buf_info audio_info;
    //double a = 0.0;
    
	snd.end = false;
	while (!snd.end)
	{         
		if (read (audio_fd, ptr, 2 * channels) != 2 * channels)
			g_print ("Error reading from OSS.\n");
		if (sf_write_short (sf, buf, 1) != 1)
			g_print ("Error writing to file.\n");
	}
	sf_close (sf);
	close (audio_fd);
	
	
	// --------------------------------------------------------------------
	// Recognition.
	// --------------------------------------------------------------------
	gchar *stdout;
	gchar *stderr;
	gint exit_status;
	GError *cmd_err = NULL;
	gchar *working_directory = g_strdup_printf ("%s", snd.work_dir_path);
	gchar **hvite;
	
	g_shell_parse_argv ("./hvite.sh", NULL, &hvite, NULL);
	
	// execute hvite command
	if (g_spawn_sync (working_directory, hvite, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &stdout, &stderr, &exit_status, &cmd_err))
	{
		g_print ("Success\n");
	}
	else
	{
		g_print ("Failure\n");
	}
	if (cmd_err)
	{
		g_print ("Error: %s\n", cmd_err->message);
	}
	else
	{
		g_print ("No error\n");
	}
	g_print ("stdout: %s\n", stdout);
	g_print ("stderr: %s\n", stderr);
	g_print ("exit_status: %d\n", exit_status);
	
	// get result (TODO error handling)
	gchar *result, *tmp_str;
	gchar *cmd_cat = g_strconcat ("cat ", working_directory, G_DIR_SEPARATOR_S, "tmp.rec", NULL);
	g_spawn_command_line_sync (cmd_cat, &tmp_str, NULL, NULL, NULL);
	result = g_strcompress (tmp_str);
	g_print ("Vysledok: %s\n", result);
	
    
	// --------------------------------------------------------------------
	// Finalization.
	// --------------------------------------------------------------------
	g_print ("Finalizing recognition...\n");
	g_free (wavfile);
	
	g_free (stderr);
    g_free (stdout);
    if (cmd_err != NULL) g_error_free (cmd_err);
    g_strfreev (hvite);
    g_free (tmp_str);
    g_free (result);
}
void play_start_new (Sound &snd)
{
	char* file_name = g_strdup_printf ("%s", snd.playback_file_name);
	g_print ("playing file '%s'\n", file_name);
	
    // --------------------------------------------------------------------
	// Audio device initialization
	// --------------------------------------------------------------------
//	g_print ("audio init\n");
	SF_INFO sfinfo;
    SNDFILE *sf;
    sfinfo.format = 0;
    sf = sf_open (file_name, SFM_READ, &sfinfo);
    if (sf == NULL) g_warning ("Error. Opening file %s.", file_name);
    if (!(sfinfo.format & SF_FORMAT_PCM_16)) g_warning ("It's not 16bit.");
//	g_print ("channels = %d, samplerate = %d\n", sfinfo.channels, sfinfo.samplerate);
    
    int audio_fd;
    if ((audio_fd = open (AUDIO_DEV, O_WRONLY, 0)) < 0) {
		perror (AUDIO_DEV);
//		exit (1);
		sf_close (sf);
		g_free (file_name);
		snd.playback_progress = false;
		return;
    }
    
    int f, g;
    f = g = AFMT_S16_LE;
    if (ioctl (audio_fd, SNDCTL_DSP_SETFMT, &f) < 0) g_warning ("InitAudo: error setting sample format");
    if (f != g) g_warning ("InitAudio: unable to set 16 bit sample format");
    f = g = sfinfo.channels;
    if (ioctl (audio_fd, SNDCTL_DSP_CHANNELS, &f) < 0) g_warning ("InitAudio: error setting audio channel");
    if (f != g) g_warning ("InitAudio: unable to set %d chanels (actually = %d)", g, f);
    int channels = f;
    f = g = sfinfo.samplerate;
    if (ioctl (audio_fd, SNDCTL_DSP_SPEED, &f) == -1) g_warning ("InitAudio: error setting sampling rate");
    if (f != g) g_warning ("InitAudio: unable to set %d sample rate (actually = %d)", g, f);
    
    short buf[channels > sfinfo.channels ? channels : sfinfo.channels];
    char *ptr;
    ptr = (char*)buf;
    int nb, index = 0; 
	audio_buf_info audio_info;
	gboolean end_while = false;
    
    // --------------------------------------------------------------------
	// Playback
	// --------------------------------------------------------------------
//	g_print ("audio playback\n");
	while (snd.playback_progress && !end_while)
	{
		if (ioctl (audio_fd, SNDCTL_DSP_GETOSPACE, &audio_info) < 0)
			g_warning ("InitAudio: error getting fragment size");
		nb = audio_info.bytes / 2 / channels;
		for (; nb > 0 && snd.playback_progress; nb--, index++)
		{
			if (sf_read_short (sf, buf, sfinfo.channels) < sfinfo.channels)
				end_while = true;
			
			if (channels > sfinfo.channels) 
				for (int i=sfinfo.channels; i < channels; i++)
					buf[i]=buf[0];
		       
			if (write (audio_fd, ptr, 2 * channels) != 2 * channels) g_warning ("Error writing to OSS.");
		}
	}
	
	// --------------------------------------------------------------------
	// Cleanup
	// --------------------------------------------------------------------
//	g_print ("audio cleanup\n");
    close (audio_fd);
    sf_close (sf);
    g_free (file_name);
    
    if (snd.playback_file_name != NULL)
    {
    	g_free (snd.playback_file_name);
    	snd.playback_file_name = NULL;
    }
    snd.playback_progress = false;
}

void entry_clear (PhonetEntry *e)
{
	e->name = "";
	e->word = "";
	e->phonemes.clear ();
}
string trim_word (const char* w)
{
	// --------------------------------------------------------------------
	// Prepares string 'w' for recognition process, which means that it
	// converts the string to lower case, trimms it for white space and
	// additional characters (like commas, dots, exclamation and question
	// marks etc..).
	// --------------------------------------------------------------------
	string s;
	gchar* copy;
	gchar *p = g_strdup (w);
	int size = g_utf8_strlen (p, -1);
	
	// validate string
	if (!g_utf8_validate (w, -1, NULL))
		g_warning ("TrimWord: Error, input is not valid utf8! '%s'", w);
	
	// find the last character
	for (int i = 0; i < size - 1; i++)
		p = g_utf8_next_char (p);
	
	// if it contains special characters...
	if (g_strcasecmp (p, " ") == 0 ||
		g_strcasecmp (p, ".") == 0 ||
		g_strcasecmp (p, ",") == 0 ||
		g_strcasecmp (p, "!") == 0 ||
		g_strcasecmp (p, "?") == 0 )
	{
		// ...remove the last character
		copy = (gchar*) g_malloc (strlen(w) + 1); // +1 for the trailing '\0'
		copy = g_utf8_strncpy (copy, w, strlen(w));
		copy[strlen(copy)-1] = '\0';
		s = g_strdup (copy);
		g_free (copy);
	}
	else
	{
		// .. otherwise just make a copy
		s = g_strdup (w);
	}
	
	// TODO trimm white space
	
	// convert it to lowercase
	copy = g_utf8_strdown (s.c_str(), -1);
	s = copy;
	g_free (copy);
	
	return s;
}
int sdist (string source, string target)
{
	// --------------------------------------------------------------------
	// This function computes string distance between strings s1, s2.
	// String distance is the minimum number of editing changes we need to
	// perform to transform source into target (and vice versa).
	// Source and target are considered utf-8 encoded trimmed strings (see
	// function trim_word for details on trimming).
	// 
	// Algorithm:
	// 1) sdist ("", "") = 0
	// 2) sdist (s, "") = sdist ("", s) = strlen (s)
	// 3) sdist (s1+ch1, s2+ch2) =						; ch1, ch2 - last character
	//		  = min( 	sdist(s1, s2) + if ch1=ch2 then 0 else 1
    // 					sdist(s1+ch1, s2) + 1,
    //					sdist(s1, s2+ch2) + 1 )
	// --------------------------------------------------------------------
	
	gchar* src = g_strdup (source.c_str());
	gchar* src_orig = src;
	gchar* dst = g_strdup (target.c_str());
	gchar* dst_orig = dst;
	
	// validate input
	if (!g_utf8_validate (src, -1, NULL)) return -1;
	if (!g_utf8_validate (dst, -1, NULL)) return -1;
	
	// declare sizes of both input strings
	const int n = g_utf8_strlen (src, -1);
	const int m = g_utf8_strlen (dst, -1);
	
	// return length if one of strings is empty
	if (n == 0) return m;
	if (m == 0) return n;
	
	typedef vector<vector<int> > tMatrix;
	// initialize matrix
	tMatrix matrix (n+1);
	for (int i = 0; i <= n; i++)
		matrix[i].resize (m+1);
	
	// init first row and column
	for (int i = 0; i <= n; i++)
		matrix[i][0] = i;
	for (int j = 0; j <= m; j++)
		matrix[0][j] = j;
	
	
	// start algorithm
	for (int i = 1; i <= n; i++)
	{
		gunichar src_i = g_utf8_get_char_validated (src, -1);
		if (src_i < 0) return -1; // error
		src = g_utf8_next_char (src);
		
		for (int j = 1; j <= m; j++)
		{
			gunichar dst_j = g_utf8_get_char_validated (dst, -1);
			if (dst_j < 0) return -1; // error
			dst = g_utf8_next_char (dst);
			
			int cost;
			if (src_i == dst_j)
				cost = 0;
			else
				cost = 1;
			
			const int above = matrix[i-1][j];
			const int left  = matrix[i][j-1];
			const int diag  = matrix[i-1][j-1];
			const int cell  = min (above + 1, min (left + 1, diag + cost));
			
			matrix[i][j] = cell;
		}
		dst = dst_orig;
	}
	
	// print matrix
//	for (int i = 0; i <= n; i++)
//	{
//		for (int j = 0; j <= m; j++)
//		{
//	 		cout << "[" << matrix[i][j] << "] ";
//		}
//		cout << endl;
//	}
	
	g_free (src_orig);
	g_free (dst_orig);
	return matrix[n][m];
}

static gboolean operator== (PhonetEntry a, PhonetEntry b)
{
	if (a.name != b.name) return false;
	if (a.word != b.word) return false;
	
	if (a.phonemes.size() != b.phonemes.size())
		return false;
	
	for (int i = 0; i < (int) a.phonemes.size(); i++)
		if (a.phonemes[i] != b.phonemes[i])
			return false;
	
	return true;
}





