#ifndef CANVAS_H_
#define CANVAS_H_

// If this is defined to true, GCompris API is assumed. Otherwise, project will be compiled with vanilla flavour.
//#define GCOMPRIS_API			true
#define GCOMPRIS_API			false

#if GCOMPRIS_API
extern "C" {
#include "libgcompris/gcompris.h"
}
#endif /*GCOMPRIS_API*/

#include "xml.h"
#include "config_handler.h"

#include "canvas_tips.h"
#include "canvas_button.h"
#include "canvas_listbox.h"

#include "sound.h"

/**
 * @author Martin Malar
 * Feb 23, 2007 10:22:30 PM
 * 
 * Handles a GnomeCanvas.
 * 
 */
class Canvas
{
	public:
		Canvas (GnomeCanvas* gcanvas);
	   ~Canvas ();
		// --------------------------------------------------------------------
		// METHODS 			- PUBLIC
		// --------------------------------------------------------------------
		gint LoadFromFile (char *filePath); // loads .xml file, parses it and formats to textfield - filePath is directory of the story
		void PrintVersion ();				// prints version information
		// --------------------------------------------------------------------
		// PROPERTIES 		- PUBLIC
		// --------------------------------------------------------------------
		gchar* GetVersion ();				// gets current version
		GnomeCanvas* GetCanvas ();			// returns gnomecanvas itself to the application
		GnomeCanvasItem* GetTextField (); 	// returns textfield group
		void ClearBoard ();					// destroys board group
		// --------------------------------------------------------------------
		// ATTRIBUTES 		- PUBLIC
		// --------------------------------------------------------------------
		
		//
		friend gboolean wait (Canvas &cvs);
		friend gboolean wait_a_sec (Canvas &cvs);
		friend gboolean wait_a_sec_read (Canvas &cvs);
		friend gboolean wait_for_result (Canvas &cvs);
		friend gboolean wait_gc_play (Canvas &cvs);
		friend gboolean wait_gc_read (Canvas &cvs);
		friend gboolean wait_for_dictate (Canvas &cvs);
		
	private:
		// --------------------------------------------------------------------
		// METHODS 			- PRIVATE
		// --------------------------------------------------------------------
		void Configure (char* configFileName); // reads configuration from .xml file
		ErrorLoadData LoadData ();			// scans resources dir for available articles
		void Init();						// initializes gnomecanvas
		void DrawBackground ();				// draws background items
		void DrawTextfield ();				// draws outline of textfield
		void DrawAuthorInfo ();				// draws info field onto board
		void DrawSingleLine (GnomeCanvasGroup *group,int x1, int y1, int x2, int y2, char *color, double thickness); // creates a single line item with given color
		void LoadText ();					// loads sStory structure into textfield (called from LoadFromFile)
		void LoadAuthorInfo (char* dir_path);// loads info about text into info field
		ErrorLoadPage LoadPage (int page_number);// loads page of sStory into textfield
		ErrorLoadPage LoadHeadline ();		// loads headline
		void ClearTextfield ();				// destroys textfield
		void ClearAuthorInfo ();			// destroys info
		double InsertWord (sWord word);		// inserts a single word (or text which then acts as a whole) into textfield. Note that width of the word cannot exceed textfield's width (tf_width)
		gint InsertWordEndl (TextStyle style);	// simulates end line inserted into text
		GnomeCanvasItem* InsertWordCreate (char* text, double x, double y, TextStyle style); // wrapper - creates new word with proper attributes
		
		// pango wrappers
		PangoAttribute* PangoAttrForegroundNew (sColor color, glong endindex);
		PangoAttribute* PangoAttrForegroundNew (char *spec, glong endindex);
		PangoAttribute* PangoAttrBackgroundNew (sColor color, glong endindex);
		PangoAttribute* PangoAttrBackgroundNew (char *spec, glong endindex);
		void ChangeTextColorMode (GnomeCanvasItem *item, TextColorMode mode, int text_length);
		PangoFontDescription* PangoFontDescGet (TextStyle style);
		void ChangeTextMarkMode (GnomeCanvasItem *item, MarkMode mode);	// changes font color of the given item according to the markmode provided. Note that this method is similar to the ChangeTextColorMode, that is used for highlighting/selecting text items
		void MarksReset ();
		
		void HandleResult ();				// gets reset from sound class and marks current word appropriately
		double EvaluateResult (string result, string word);	// evaluates result in comparison to word
		double EvaluateResult_old (string result, string word);	// evaluates result in comparison to word
		void MarkResult (GnomeCanvasItem *item, double perc, string result);	// marks GnomeCanvasItem with a mark
		
		char* GetMarkModeColor (MarkMode mode);
		GdkPixbuf* GetMarkedPixbuf (MarkMode mode);
		
		// --------------------------------------------------------------------
		// EVENTS 			- PRIVATE
		// --------------------------------------------------------------------
		static gint OnItemEvent (GnomeCanvasItem* item, GdkEvent* event, Canvas* obj);
		static gint OnCanvasEvent (GnomeCanvas *g_canvas, GdkEvent *event, Canvas* obj);
		static gint OnButtonEvent (GnomeCanvasItem* item, GdkEvent* event, Canvas* obj);
		static gint OnNextPageMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnPrevPageMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnLstBoxMouseUp (GnomeCanvasItem* lst_box, GdkEvent* event, Canvas* obj);
		static gint OnLoadStoryClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnLoadCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnPlayClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnPlayCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnPushMouseDown (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnPushMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnReadMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnDictateMouseUp (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		static gint OnDictateCancelClick (GnomeCanvasItem* button, GdkEvent* event, Canvas* obj);
		// --------------------------------------------------------------------
		// ATTRIBUTES 		- PRIVATE
		// --------------------------------------------------------------------
		gboolean use_api;					// true - gcompris api, false - tutor app
		GnomeCanvas *canvas;				// gnome canvas itself
		GnomeCanvasGroup *board;			// group for all content
		GnomeCanvasItem *board_border;		// rectangle whose purpose is to create border around board
		GnomeCanvasGroup *textf_outline;	// group for outline of textfield
		GnomeCanvasGroup *textfield;		// group for text items, specified by tf_width and tf_height
		GnomeCanvasGroup *info_outline;
		GnomeCanvasGroup *author_info;		// info about author of the story
		GnomeCanvasItem *info_page_number;	// item representing information about current page
		
		double tf_width, tf_height;			// width/height of textfield area (area where to put text, not TEXTF_WIDTH/TEXTF_HEIGHT - these represent overall size of textfield including outline.
		double info_width, info_height;		// width/height of info field, where to put actual content
		GnomeCanvasItem *last_word;			// last inserted word
		
		vector<GnomeCanvasItem*> tf_words;	// vector of words already in textfield
		vector<sWord> tf_swords;			// analog vector to tf_words, but with sWords as its elements
		vector<MarkMode> tf_marks;			// analog with tf_words, tf_swords, this saves marking information
		int selected_word;					// index of selected word (in tf_words) - no word is selected, if it equals -1
		int marked_word;					// index of the last marked words (or current word)
		gboolean tf_words_selectable;		// if true, words are highlighted as mouse moves over them and can be selected by clicking on them
		sStory Story;						// structure to hold parsed .xml file
		int page;							// currently loaded page (zero-based)
		int story_id;						// currently loaded article id, -1 for none (in story_list - see below)
		PersonData* person_data;			// information about current author
		vector<StoryInfo*> story_list;		// detected articles
		
		// IMAGES
		GdkPixbuf *btn_next, *btn_prev, *btn_next_dis, *btn_prev_dis;
		GdkPixbuf *background_image;
		GdkPixbuf *info_background_image;	// background image for author's info field
		GdkPixbuf *gc_image;				// gcompris-styled image
		GdkPixbuf *button_std, *button_alt;	// button images
		GdkPixbuf *button_dis;				// disabled button
		//icons
		GdkPixbuf *icon_cancel;				// icon for 'cancel button'
		GdkPixbuf *icon_cancel_dis;			// icon for 'cancel button'
		GdkPixbuf *icon_open;				// icon for 'open button'
		GdkPixbuf *icon_open_dis;			// icon for 'open button'
		GdkPixbuf *icon_play;				// icon for 'play button'
		GdkPixbuf *icon_play_dis;			// icon for 'play button'
		GdkPixbuf *icon_record;				// icon for 'record button'
		GdkPixbuf *icon_record_dis;			// icon for 'record button'
		GdkPixbuf *icon_read;
		GdkPixbuf *icon_read_dis;
		GdkPixbuf *icon_dictate;
		GdkPixbuf *icon_dictate_dis;
		GdkPixbuf *smile1, *smile2, *smile3, *smile4, *smile5;
		
		// ToolTips
		CanvasTip		*tooltip;
		
		void TipNew (char* label, double x, double y, double item_height); // label - text of tooltip, x,y - coords of event (world), by2 - bottom of textitem, signaling event (parent relative)
		void TipNewImg (GdkPixbuf *img, double x, double y, double item_height); // label - text of tooltip, x,y - coords of event (world), by2 - bottom of textitem, signaling event (parent relative)
		void TipDestroy ();
		
		// Textfield Buttons
		CanvasButton *button_next, *button_prev;
		CanvasButton* ButtonStandardNew (char* label_text, double x, double y);
		CanvasButton* ButtonImageNew (GdkPixbuf *pic, GdkPixbuf *pic_clicked, GdkPixbuf *pic_disabled, double x, double y);
		
		// Listbox - loading files
		CanvasButton *btnLoadStory;			// show dialog for loading articles
		CanvasButton *btnLoadCancel;		// cancel loading article
		CanvasListbox *lb_select;			// for selecting story for opening
		CanvasListbox* ListboxSelectNew (double x, double y);
		CanvasButton* ButtonNewOnBoard (char* label_text, double x, double y);
		CanvasButton* ButtonImageIconNew (char* label, GdkPixbuf *image, GdkPixbuf *image_clicked, GdkPixbuf *icon, GdkPixbuf *icon_disabled, double x, double y); // put image button with icon on coords x,y onto 'board'
		
		// Buttons - play words
		Sound *sound;
		CanvasButton* btnPlay;				// plays selected word
		CanvasButton* btnPlayCancel;		// cancels playback in progress
		CanvasButton* btnPush;				// push to talk button
		CanvasButton* btnRead;				// plays wav with whole text
		CanvasButton* btnReadCancel;
		CanvasButton* btnDictate;			// same as above, but reads text with pauses
		CanvasButton* btnDictateCancel;
		gboolean dictate_in_progress;
		gboolean dictate_paused;
		
		// Button switch methods 
		// These methods manage enable/disable visible/invisible properties of buttons
		// on certain event.
		void SwitchButtons_LoadedStory ();	// manages buttons on loading the story
		void SwitchButtons_Load ();			// manages buttons on Load story click (opening the open_story_dialog)
		void SwitchButtons_LoadCancel ();	// manages buttons upon cancelling loading
		void SwitchButtons_Play ();			// manages buttons when PLAY button is clicked
		void SwitchButtons_PlayCancel ();	// --//-- for PlayCancelClick
		void SwitchButtons_SelectWord ();	// when a word is selected
		void SwitchButtons_DeselectWord ();	// after deselecting a word
		void SwitchButtons_StoryClosed ();	// when the story is closed (right mouse click)
		void SwitchButtons_Read ();			// Read button was clicked
		void SwitchButtons_ReadCancel ();
		void SwitchButtons_Dictate ();			// Dictate button was clicked
		void SwitchButtons_DictateCancel ();
		void SwitchButtons_DisableAll ();	// disable all buttons
		void SwitchButtons_EnableAll ();	// enable all butons
		
		
		// --------------------------------------------------------------------
		// CONFIGURATION ATTRIBUTES
		// --------------------------------------------------------------------
		ConfigHandler* config;				// configuration class
		// system
		gboolean SHOW_DEBUG, SHOW_MESSAGES, SHOW_WARNINGS, SHOW_ERRORS;
		// paths, files
		gchar *PATH_RESX;
		char *GLADE_FILE, *PRG_ICON, *CONFIG_FILE, *PICTURE_FILE, *STORY_FILE;
		char *PIC_BTN_NEXT, *PIC_BTN_NEXT_DIS, *PIC_BTN_PREV, *PIC_BTN_PREV_DIS, *BACKGROUND_IMAGE_FILENAME;
		char *PIC_OPEN_BOOK;
		char *PIC_BUTTON_STD, *PIC_BUTTON_ALT, *PIC_BUTTON_DIS;
		
		// canvas
		gboolean CANVAS_ANTI_ALIASED, CANVAS_GRID_SHOW;
		double CANVAS_WIDTH, CANVAS_HEIGHT, CANVAS_BORDER, CANVAS_GRID_THICK;
		char *CANVAS_BORDER_COLOR, *CANVAS_BACKGROUND_COLOR, *CANVAS_GRID_COLOR;
		
		// textfield
		double TEXTF_WIDTH, TEXTF_HEIGHT, TEXTF_Y, TEXTF_X, TEXTF_PADDING, TEXTF_PADDING_VERT, TEXTF_BORDER;
		char *TEXTF_BORDER_COLOR;
		gboolean TEXTF_DOCK, TEXTF_BORDER_SHOW, TEXTF_INNER_SHOW;
		
		gboolean TEXTF_DOCK_TOP, TEXTF_DOCK_BOTTOM, TEXTF_DOCK_RIGHT;
		double TEXTF_DOCK_TOP_MARGIN, TEXTF_DOCK_BOTTOM_MARGIN, TEXTF_DOCK_RIGHT_MARGIN;
		
		// (author) info
		double INFO_X, INFO_Y, INFO_WIDTH, INFO_HEIGHT, INFO_PADDING;
		double INFO_DOCK_BOTTOM, INFO_DOCK_RIGHT, INFO_DOCK_LEFT, INFO_BORDER;
		gboolean INFO_DOCK, INFO_BORDER_SHOW; 
		char *INFO_BORDER_COLOR;
		
		// text
		int TEXT_SIZE, TEXT_SIZE_HEADING;
		double TEXT_SPACING_VERT, TEXT_SPACING_HORIZ;
		gboolean TEXT_HEADLINE_CENTER;
		char* TEXT_FAMILY;
		
		// font
		char *FONT_COLOR, *FONT_HLT_COLOR, *FONT_BCOLOR, *FONT_HLT_BCOLOR, *TEXT_PARAGRAPH_INDEND;
		char *FONT_SELECT_COLOR, *FONT_SELECT_BCOLOR, *FONT_SELECT_HLT_COLOR, *FONT_SELECT_HLT_BCOLOR;
		int FONT_COLOR_AA, FONT_HLT_COLOR_AA, FONT_BCOLOR_AA, FONT_HLT_BCOLOR_AA;
		
		// mark colors
		char *MARK_COLOR_NONE, *MARK_COLOR_CURRENT, *MARK_COLOR_PASSED;
		
		// tooltips
		char *TIP_COLOR, *TIP_BORDER_COLOR, *TIP_BORDER_COLOR_ALT, *TIP_FONT_COLOR;
		double TIP_BORDER_THICK, TIP_PADDING_WIDTH, TIP_PADDING_HEIGHT;
		int TIP_FONT_SIZE;
		gboolean TIP_BORDER_PLASTIC, TIP_SHOW_TOOLTIPS;
		
		// buttons
		char *BUTTON_BORDER_COLOR, *BUTTON_SHADOW_COLOR, *BUTTON_BACKGROUND_COLOR, *BUTTON_TEXT_COLOR, *BUTTON_TEXT_COLOR_ALT;
		double BUTTON_BORDER_WIDTH, BUTTON_WIDTH, BUTTON_HEIGHT, BUTTON_WIDTH_ICON, BUTTON_HEIGHT_ICON;
		gboolean BUTTON_PLASTIC_ON_MOTION;
		int BUTTON_TEXT_SIZE;
		gboolean BUTTON_SHOW_IMAGE_BUTTONS, BUTTON_SHOW_IMG_LABELS;
		
		// listbox
		double LB_X, LB_Y, LB_WIDTH, LB_HEIGHT, LB_BORDER_WIDTH, LB_PADDING;
		int LB_TEXT_SIZE;
		char *LB_TEXT_COLOR, *LB_BORDER_COLOR;
		double LB_BTN_CANCEL_X, LB_BTN_CANCEL_Y;
		
};

gboolean wait (Canvas &cvs);
gboolean wait_a_sec (Canvas &cvs);
gboolean wait_a_sec_read (Canvas &cvs);
gboolean wait_for_result (Canvas &cvs);
gboolean wait_for_dictate (Canvas &cvs);

MarkMode get_mark_mode (double perc);
char* get_mark_color (double perc);

#if GCOMPRIS_API
static void gc_play_cb (gchar* file);
#endif

gboolean wait_gc_play (Canvas &cvs);
gboolean wait_gc_read (Canvas &cvs);


#endif /*CANVAS_H_*/
