#ifndef CANVAS_BUTTON_H_
#define CANVAS_BUTTON_H_

// INCLUDES
#include "config_handler.h" // using gnome.h and some default configurations
#include "canvas_tips.h"	// for tooltips on buttons

// TYPES
typedef struct _CanvasButton CanvasButton;
typedef struct _CanvasButtonDesc CanvasButtonDesc;


// ENUMERATIONS
typedef enum
{
	ButtonTypeLabel,		// button with text label
	ButtonTypeEmpty,		// empty button
	ButtonTypeImage,		// button with image
	ButtonTypeCanvasItem,	// button with another canvas item
} ButtonType;


// STRUCTURES
struct _CanvasButton
{
	GnomeCanvasItem *root_grp;
	GnomeCanvasItem *rect;				// background of button
	GnomeCanvasItem *border_nw;
	GnomeCanvasItem *border_se;
	GnomeCanvasItem *label;				// text label
	GnomeCanvasItem *image;				// image, this is drawn onto rect
	GnomeCanvasItem *image_clicked;		// image, when the button is clicked down
	GnomeCanvasItem *image_disabled;	// image for disabled version
	GnomeCanvasItem *image_icon;		// button's icon
	GnomeCanvasItem *image_icon_clicked;	// TODO
	GnomeCanvasItem *image_icon_disabled;
	
	CanvasButtonDesc	*desc;
	
	CanvasTip			*tip;			// tooltip
	CanvasTipDesc		*tip_desc;		// tooltip desc
	
	// general attributes
	double			x,y;				// coords on the parent
	
	gboolean		hidden;				// hidden/visible
	gboolean		enabled;			// enabled/disabled
	gboolean		pressed;			// changes on mouse_down/mouse_up events (or clicks in case of toggle button)
	
	gboolean		has_mouse_hover;	// query whether the mouse is hovering over the button
	gboolean		border_shown;		// query on button borders visibility
	gboolean		has_icon;			// query whether the button has an icon
	gboolean		icon_visible;		// query whether the icon (if the button has one, is visible in general)
	gboolean		has_tooltip;		// query on tooltip existence
};

struct _CanvasButtonDesc
{
	ButtonType		type;				// type of the button - this determines how to create new button and what attributes to use and how
	char*			label_text;			// button label
	int				label_text_size;	// text size
	char*			label_color;		// std text color
	char*			label_color_alt;	// color of text on disabled button
	char*			background_color;	// color of the button background (if not image button)
	double			border_width;		// width of button border
	gboolean		border_show;		// true - show border, false - no border
	char*			border_color;		// color of border
	char*			shadow_color;		// color of alternative border to create plastic effect
	gboolean		toggle_button;		// whether the button is toggle button - this kind of button will be implemented later
	gboolean		plastic_on_motion;	// borders show only when mouse is hovering above the button
	
	double			width, height;		// button proportions
};

// FUNCTIONS
CanvasButton*		canvas_button_new (GnomeCanvasGroup *parent, CanvasButtonDesc* desc);
void				canvas_button_destroy (CanvasButton *button);

void				canvas_button_move_at (CanvasButton *button, double x, double y);
void				canvas_button_hide (CanvasButton *button);
void				canvas_button_show (CanvasButton *button);
void				canvas_button_set_text (CanvasButton *button, char *text);
void				canvas_button_set_image (CanvasButton *button, GdkPixbuf *pic, GdkPixbuf *pic_clicked, GdkPixbuf *pic_disabled);
void				canvas_button_set_icon (CanvasButton *button, GdkPixbuf *pic, GdkPixbuf *pic_disabled);
void				canvas_button_set_tooltip (CanvasButton *button, CanvasTipDesc* desc_tt);
void				canvas_button_up (CanvasButton *button);
void				canvas_button_down (CanvasButton *button);
void				canvas_button_enable (CanvasButton *button);
void				canvas_button_disable (CanvasButton *button);
void				canvas_button_border_show (CanvasButton *button);
void				canvas_button_border_hide (CanvasButton *button);

GnomeCanvasItem*	canvas_button_get_parent (CanvasButton *button);

// EVENTS
gint OnCanvasButtonSwitchEvent (GnomeCanvasItem* button, GdkEvent* event, CanvasButton* canvas_button);
gint OnCanvasButtonEnterLeaveEvent (GnomeCanvasItem* button, GdkEvent* event, CanvasButton* canvas_button);
gint OnCanvasButtonTooltipEvent (GnomeCanvasItem* button_item, GdkEvent* event, CanvasButton* button);

#endif /*CANVAS_BUTTON_H_*/
