#ifndef CANVAS_LISTBOX_H_
#define CANVAS_LISTBOX_H_

// INCLUDES
#include "config_handler.h"

// TYPES
typedef struct _CanvasListbox CanvasListbox;
typedef struct _CanvasListboxDesc CanvasListboxDesc;
typedef struct _CanvasListboxItem CanvasListboxItem;

// STRUCTURES
struct _CanvasListbox
{
	GnomeCanvasItem *outline;
	GnomeCanvasItem *root_grp;
	GnomeCanvasItem *rect;				// not implemented yet
	GnomeCanvasItem *border;
	
	CanvasListboxDesc* desc;
	GArray *items;
	
	
	// properties
	double		x, y;					// coords of root_grp
	
	gboolean	enabled;
	gboolean	hidden;
	gboolean	show_border;
	gboolean	empty;
};

struct _CanvasListboxDesc
{
	double	width;
	double	height;
	int		text_size;
	double	border_width;
	double	padding;
	
	char*	background_color;			// not used yet (background not implemented yet)
	char*	border_color;
	char*	text_color;
	
	gboolean	has_scrollbar_vert;		// TODO
	gboolean	has_scrollbar_horiz;	// not implemented, false by default
	
};

struct _CanvasListboxItem
{
	GnomeCanvasItem* item;
	char* text;
	char* data;							// filename to open
	int id;
};

// FUNCTIONS
CanvasListbox*	canvas_listbox_new (GnomeCanvasGroup *parent, CanvasListboxDesc *desc);
void			canvas_listbox_destroy (CanvasListbox *lbox);

// public functions
void			canvas_listbox_move_at (CanvasListbox *lbox, double x, double y);
void			canvas_listbox_hide (CanvasListbox *lbox);
void			canvas_listbox_show (CanvasListbox *lbox);
void			canvas_listbox_insert (CanvasListbox *lbox, char* text, char* data, int id);
CanvasListboxItem* canvas_listbox_get_item_at (CanvasListbox *lbox, double x, double y);

// properties
gboolean		canvas_listbox_is_empty (CanvasListbox *lbox);

// private functions
void			canvas_listbox_reorder (CanvasListbox *lbox);

// other functions
void			canvas_item_move_at (GnomeCanvasItem* item, double x, double y);

#endif /*CANVAS_LISTBOX_H_*/
