#include "config_handler.h"

ConfigHandler::ConfigHandler (char* configFileName)
{
	show_debug = false;
	config = XMLNode::openFileHelper (configFileName, "config");
}
ConfigHandler::~ConfigHandler ()
{
}

// PUBLIC
void ConfigHandler::WriteDefaultsToFile (const char* fileName)
{
	ofstream output;
	output.open (fileName);
	
	if (output.is_open())
	{
		output << "<?xml version=\"1.0\" encoding=\"utf-8\"?>" << endl;
		output << "<config package=\"" << PACKAGE << "\" version=\"" << VERSION << "\">" << endl << endl;
		
			// system
			output << "\t<system>" << endl;
				if (D_SHOW_DEBUG)
					output << "\t\t<show_debug>true</show_debug>" << endl;
				else
					output << "\t\t<show_debug>false</show_debug>" << endl;
				if (D_SHOW_MESSAGES)
					output << "\t\t<show_messages>true</show_messages>" << endl;
				else
					output << "\t\t<show_messages>false</show_messages>" << endl;
				if (D_SHOW_WARNINGS)
					output << "\t\t<show_warnings>true</show_warnings>" << endl;
				else
					output << "\t\t<show_warnings>false</show_warnings>" << endl;
				if (D_SHOW_ERRORS)
					output << "\t\t<show_errors>true</show_errors>" << endl;
				else
					output << "\t\t<show_errors>false</show_errors>" << endl;
			output << "\t</system>" << endl << endl;
			
			// path
			output << "\t<files>" << endl;
				output << "\t\t<path_resx>" << D_PATH_RESX << "</path_resx>" << endl;
				output << "\t\t<glade_file>" << D_GLADE_FILE << "</glade_file>" << endl;
				output << "\t\t<prg_icon>" << D_PRG_ICON << "</prg_icon>" << endl;
				output << "\t\t<config_file>" << D_CONFIG_FILE << "</config_file>" << endl;
				output << "\t\t<story_file>" << D_STORY_FILE << "</story_file>" << endl;
				output << "\t\t<picture_file>" << D_PICTURE_FILE << "</picture_file>" << endl;
				output << "\t\t<pic_background_image>" << D_PIC_BACKGROUND_IMAGE << "</pic_background_image>" << endl;
			output << "\t</files>" << endl << endl;
			
			// canvas
			output << "\t<canvas>" << endl;
				// Anti-aliased mode is not supported yet so it won't be possible to change it through configuration file
//				if (D_CANVAS_ANTI_ALIASED)
//					output << "\t\t<canvas_anti_aliased>true</canvas_anti_aliased>" << endl;
//				else
//					output << "\t\t<canvas_anti_aliased>false</canvas_anti_aliased>" << endl;
				output << "\t\t<canvas_width>" << D_CANVAS_WIDTH << "</canvas_width>" << endl;
				output << "\t\t<canvas_height>" << D_CANVAS_HEIGHT << "</canvas_height>" << endl;
				output << "\t\t<canvas_border>" << D_CANVAS_BORDER << "</canvas_border>" << endl;
				output << "\t\t<canvas_border_color>" << ColorToString (D_CANVAS_BORDER_COLOR) << "</canvas_border_color>" << endl;
				output << "\t\t<canvas_background_color>" << ColorToString (D_CANVAS_BACKGROUND_COLOR) << "</canvas_background_color>" << endl << endl;
				
				if (D_CANVAS_GRID_SHOW)
					output << "\t\t<canvas_grid_show>true</canvas_grid_show>" << endl;
				else
					output << "\t\t<canvas_grid_show>false</canvas_grid_show>" << endl;
				output << "\t\t<canvas_grid_color>" << ColorToString (D_CANVAS_GRID_COLOR) << "</canvas_grid_color>" << endl;
				output << "\t\t<canvas_grid_thick>" << D_CANVAS_GRID_THICK << "</canvas_grid_thick>" << endl;
			output << "\t</canvas>" << endl << endl;
				
			// textfield
			output << "\t<textfield>" << endl;
				if (D_TEXTF_DOCK)
					output << "\t\t<textf_dock>true</textf_dock>" << endl;
				else
					output << "\t\t<textf_dock>false</textf_dock>" << endl;
				if (D_TEXTF_DOCK_TOP)
					output << "\t\t<textf_dock_top>true</textf_dock_top>" << endl;
				else
					output << "\t\t<textf_dock_top>false</textf_dock_top>" << endl;
				if (D_TEXTF_DOCK_BOTTOM)
					output << "\t\t<textf_dock_bottom>true</textf_dock_bottom>" << endl;
				else
					output << "\t\t<textf_dock_bottom>false</textf_dock_bottom>" << endl;
				if (D_TEXTF_DOCK_RIGHT)
					output << "\t\t<textf_dock_right>true</textf_dock_right>" << endl;
				else
					output << "\t\t<textf_dock_right>false</textf_dock_right>" << endl;
				output << "\t\t<textf_dock_top_margin>" << D_TEXTF_DOCK_TOP_MARGIN << "</textf_dock_top_margin>" << endl;
				output << "\t\t<textf_dock_bottom_margin>" << D_TEXTF_DOCK_BOTTOM_MARGIN << "</textf_dock_bottom_margin>" << endl;
				output << "\t\t<textf_dock_right_margin>" << D_TEXTF_DOCK_RIGHT_MARGIN << "</textf_dock_right_margin>" << endl;
				
				output << "\t\t<textf_width>" << D_TEXTF_WIDTH << "</textf_width>" << endl;
				output << "\t\t<textf_height>" << D_TEXTF_HEIGHT << "</textf_height>" << endl;
				output << "\t\t<textf_y>" << D_TEXTF_Y << "</textf_y>" << endl;
				output << "\t\t<textf_x>" << D_TEXTF_X << "</textf_x>" << endl;
				output << "\t\t<textf_padding>" << D_TEXTF_PADDING << "</textf_padding>" << endl;
				output << "\t\t<textf_padding_vert>" << D_TEXTF_PADDING_VERT << "</textf_padding_vert>" << endl;
				output << "\t\t<textf_border>" << D_TEXTF_BORDER << "</textf_border>" << endl;
				output << "\t\t<textf_border_color>" << ColorToString (D_TEXTF_BORDER_COLOR) << "</textf_border_color>" << endl;
				if (D_TEXTF_BORDER_SHOW)
					output << "\t\t<textf_border_show>true</textf_border_show>" << endl;
				else
					output << "\t\t<textf_border_show>false</textf_border_show>" << endl;
				if (D_TEXTF_INNER_SHOW)
					output << "\t\t<textf_inner_show>true</textf_inner_show>" << endl;
				else
					output << "\t\t<textf_inner_show>false</textf_inner_show>" << endl;
			output << "\t</textfield>" << endl << endl;
			
			// (author) info
			output << "\t<info>" << endl;
				output << "\t\t<info_x>" << D_INFO_X << "</info_x>" << endl;
				output << "\t\t<info_y>" << D_INFO_Y << "</info_y>" << endl;
				output << "\t\t<info_width>" << D_INFO_WIDTH << "</info_width>" << endl;
				output << "\t\t<info_height>" << D_INFO_HEIGHT << "</info_height>" << endl;
				output << "\t\t<info_padding>" << D_INFO_PADDING << "</info_padding>" << endl;
				if (D_INFO_DOCK)
					output << "\t\t<info_dock>true</info_dock>" << endl;
				else
					output << "\t\t<info_dock>false</info_dock>" << endl;
				output << "\t\t<info_dock_bottom>" << D_INFO_DOCK_BOTTOM << "</info_dock_bottom>" << endl;
				output << "\t\t<info_dock_right>" << D_INFO_DOCK_RIGHT << "</info_dock_right>" << endl;
				output << "\t\t<info_dock_left>" << D_INFO_DOCK_LEFT<< "</info_dock_left>" << endl;
				output << "\t\t<info_border>" << D_INFO_BORDER << "</info_border>" << endl;
				if (D_INFO_BORDER_SHOW)
					output << "\t\t<info_border_show>true</info_border_show>" << endl;
				else
					output << "\t\t<info_border_show>false</info_border_show>" << endl;
				output << "\t\t<info_border_color>" << ColorToString (D_INFO_BORDER_COLOR) << "</info_border_color>" << endl;
			output << "\t</info>" << endl << endl;
			
			// text
			output << "\t<text>" << endl;
				output << "\t\t<text_family>" << D_TEXT_FAMILY << "</text_family>" << endl;
				output << "\t\t<text_size>" << D_TEXT_SIZE << "</text_size>" << endl;
				output << "\t\t<text_size_heading>" << D_TEXT_SIZE_HEADING << "</text_size_heading>" << endl;
				output << "\t\t<text_spacing_vert>" << D_TEXT_SPACING_VERT << "</text_spacing_vert>" << endl;
				output << "\t\t<text_spacing_horiz>" << D_TEXT_SPACING_HORIZ << "</text_spacing_horiz>" << endl;
				if (D_TEXT_HEADLINE_CENTER)
					output << "\t\t<text_headline_center>true</text_headline_center>" << endl;
				else
					output << "\t\t<text_headline_center>false</text_headline_center>" << endl;
				// this crashes program, if there are only spaces // output << "\t\t<text_paragraph_indend>" << D_TEXT_PARAGRAPH_INDEND << "</text_paragraph_indend>" << endl;
			output << "\t</text>" << endl << endl;
			
			// font
			output << "\t<font>" << endl;
				output << "\t\t<font_color>" << ColorToString (D_FONT_COLOR) << "</font_color>" << endl;
				output << "\t\t<font_bcolor>" << ColorToString (D_FONT_BCOLOR) << "</font_bcolor>" << endl;
				output << "\t\t<font_hlt_color>" << ColorToString (D_FONT_HLT_COLOR) << "</font_hlt_color>" << endl;
				output << "\t\t<font_hlt_bcolor>" << ColorToString (D_FONT_HLT_BCOLOR) << "</font_hlt_bcolor>" << endl;
				output << "\t\t<font_selected_color>" << ColorToString (D_FONT_SELECT_COLOR) << "</font_selected_color>" << endl;
				output << "\t\t<font_selected_bcolor>" << ColorToString (D_FONT_SELECT_BCOLOR) << "</font_selected_bcolor>" << endl;
				output << "\t\t<font_select_hlt_color>" << ColorToString (D_FONT_SELECT_HLT_COLOR) << "</font_select_hlt_color>" << endl;
				output << "\t\t<font_select_hlt_bcolor>" << ColorToString (D_FONT_SELECT_HLT_BCOLOR) << "</font_select_hlt_bcolor>" << endl << endl;
				
				output << "\t\t<font_color_aa>" << ColorToString_AA (D_FONT_COLOR_AA) << "</font_color_aa>" << endl;
				output << "\t\t<font_hlt_color_aa>" << ColorToString_AA (D_FONT_HLT_COLOR_AA) << "</font_hlt_color_aa>" << endl;
				output << "\t\t<font_bcolor_aa>" << ColorToString_AA (D_FONT_BCOLOR_AA) << "</font_bcolor_aa>" << endl;
				output << "\t\t<font_hlt_bcolor_aa>" << ColorToString_AA (D_FONT_HLT_BCOLOR_AA) << "</font_hlt_bcolor_aa>" << endl;
			output << "\t</font>" << endl << endl;
			
			// tooltips
			output << "\t<tooltips>" << endl;
				if (D_TIP_SHOW_TOOLTIPS)
					output << "\t\t<tip_show_tooltips>true</tip_show_tooltips>" << endl;
				else
					output << "\t\t<tip_show_tooltips>false</tip_show_tooltips>" << endl;
				output << "\t\t<tip_color>" << ColorToString (D_TIP_COLOR) << "</tip_color>" << endl;
				output << "\t\t<tip_border_color>" << ColorToString (D_TIP_BORDER_COLOR) << "</tip_border_color>" << endl;
				output << "\t\t<tip_border_color_alt>" << ColorToString (D_TIP_BORDER_COLOR_ALT) << "</tip_border_color_alt>" << endl;
				output << "\t\t<tip_border_thick>" << D_TIP_BORDER_THICK << "</tip_border_thick>" << endl;
				output << "\t\t<tip_font_color>" << ColorToString (D_TIP_FONT_COLOR) << "</tip_font_color>" << endl;
				output << "\t\t<tip_font_size>" << D_TIP_FONT_SIZE << "</tip_font_size>" << endl;
				output << "\t\t<tip_padding_height>" << D_TIP_PADDING_HEIGHT << "</tip_padding_height>" << endl;
				output << "\t\t<tip_padding_width>" << D_TIP_PADDING_WIDTH << "</tip_padding_width>" << endl;
				if (D_TIP_BORDER_PLASTIC)
					output << "\t\t<tip_border_plastic>true</tip_border_plastic>" << endl;
				else
					output << "\t\t<tip_border_plastic>false</tip_border_plastic>" << endl;
			output << "\t</tooltips>" << endl << endl;
			
			// buttons
			output << "\t<buttons>" << endl;
				output << "\t\t<button_border_width>" << D_BUTTON_BORDER_WIDTH << "</button_border_width>" << endl;
				output << "\t\t<button_border_color>" << ColorToString (D_BUTTON_BORDER_COLOR) << "</button_border_color>" << endl;
				output << "\t\t<button_background_color>" << ColorToString (D_BUTTON_BACKGROUND_COLOR) << "</button_background_color>" << endl;
				output << "\t\t<button_text_size>" << D_BUTTON_TEXT_SIZE << "</button_text_size>" << endl;
				output << "\t\t<button_text_color>" << ColorToString (D_BUTTON_TEXT_COLOR) << "</button_text_color>" << endl;
				output << "\t\t<button_text_color_alt>" << ColorToString (D_BUTTON_TEXT_COLOR_ALT) << "</button_text_color_alt>" << endl;
				output << "\t\t<button_width>" << D_BUTTON_WIDTH << "</button_width>" << endl;
				output << "\t\t<button_height>" << D_BUTTON_HEIGHT << "</button_height>" << endl;
				output << "\t\t<button_shadow_color>" << ColorToString (D_BUTTON_SHADOW_COLOR) << "</button_shadow_color>" << endl;
				if (D_BUTTON_SHOW_IMAGE_BUTTONS)
					output << "\t\t<button_show_image_buttons>true</button_show_image_buttons>" << endl;
				else
					output << "\t\t<button_show_image_buttons>false</button_show_image_buttons>" << endl;
				if (D_BUTTON_SHOW_IMG_LABELS)
					output << "\t\t<button_show_img_labels>true</button_show_img_labels>" << endl;
				else
					output << "\t\t<button_show_img_labels>false</button_show_img_labels>" << endl;
			output << "\t</buttons>" << endl << endl;
			
			// listbox
			output << "\t<listbox>" << endl;
				output << "\t\t<lb_x>" << D_LB_X << "</lb_x>" << endl;
				output << "\t\t<lb_y>" << D_LB_Y << "</lb_y>" << endl;
				output << "\t\t<lb_width>" << D_LB_WIDTH << "</lb_width>" << endl;
				output << "\t\t<lb_height>" << D_LB_HEIGHT << "</lb_height>" << endl;
				output << "\t\t<lb_border_width>" << D_LB_BORDER_WIDTH << "</lb_border_width>" << endl;
				output << "\t\t<lb_text_size>" << D_LB_TEXT_SIZE << "</lb_text_size>" << endl;
				output << "\t\t<lb_text_color>" << ColorToString (D_LB_TEXT_COLOR) << "</lb_text_color>" << endl;
				output << "\t\t<lb_border_color>" << ColorToString (D_LB_BORDER_COLOR) << "</lb_border_color>" << endl;
				output << "\t\t<lb_padding>" << D_LB_PADDING << "</lb_padding>" << endl;
				output << "\t\t<lb_btn_cancel_x>" << D_LB_BTN_CANCEL_X << "</lb_btn_cancel_x>" << endl;
				output << "\t\t<lb_btn_cancel_y>" << D_LB_BTN_CANCEL_Y << "</lb_btn_cancel_y>" << endl;
			
			output << "\t</listbox>" << endl << endl;
				
		output << "</config>" << endl;
		
		output.close ();
	}
	else
	{
		if (show_warnings) g_warning ("Unable to open file `%s` for input.", fileName);
	}
	
}

// system
gboolean ConfigHandler::ShowDebug ()
{
	if (config.nChildNode ("system") > 0)
	{
		XMLNode node = config.getChildNode ("system");
		if (node.nChildNode ("show_debug") > 0)
		{
			string value = getText (node.getChildNode ("show_debug"));
			if (value.compare ("true") == 0) { show_debug = true; return true; }
			else { show_debug = false; return false; }
		}
	}
	show_debug = D_SHOW_DEBUG;
	return D_SHOW_DEBUG;
}
gboolean ConfigHandler::ShowMessages ()
{
	if (show_debug) g_debug ("ConfigHandler::ShowMessages (): Running");
	if (config.nChildNode ("system") > 0)
	{
		XMLNode node = config.getChildNode ("system");
		if (node.nChildNode ("show_messages") > 0)
		{
			string value = getText (node.getChildNode ("show_messages"));
			if (value.compare ("true") == 0) { show_messages = true; return true; }
			else { show_messages = false; return false; }
		}
	}
	return D_SHOW_MESSAGES;
}
gboolean ConfigHandler::ShowWarnings ()
{
	if (show_debug) g_debug ("ConfigHandler::ShowWarnings (): Running");
	if (config.nChildNode ("system") > 0)
	{
		XMLNode node = config.getChildNode ("system");
		if (node.nChildNode ("show_warnings") > 0)
		{
			string value = getText (node.getChildNode ("show_warnings"));
			if (value.compare ("true") == 0) { show_warnings = true; return true; }
			else { show_warnings = false; return false; }
		}
	}
	return D_SHOW_WARNINGS;
}
gboolean ConfigHandler::ShowErrors ()
{
	if (show_debug) g_debug ("ConfigHandler::ShowErrors (): Running");
	if (config.nChildNode ("system") > 0)
	{
		XMLNode node = config.getChildNode ("system");
		if (node.nChildNode ("show_errors") > 0)
		{
			string value = getText (node.getChildNode ("show_errors"));
			if (value.compare ("true") == 0) { show_errors = true; return true; }
			else { show_errors = false; return false; }
		}
	}
	return D_SHOW_ERRORS;
}
// path
char* ConfigHandler::PathResx ()
{
	char* dir = g_strconcat (g_get_home_dir(), G_DIR_SEPARATOR_S, D_PATH_RESX, NULL);
	return dir;
}
// files
char* ConfigHandler::GladeFile ()
{
	if (show_debug) g_debug ("ConfigHandler::GladeFile (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("glade_file") > 0)
		{
			string value = getText (node.getChildNode ("glade_file"));
			return ParseColor (value);
		}
	}
	return D_GLADE_FILE;
}
char* ConfigHandler::PrgIcon ()
{
	if (show_debug) g_debug ("ConfigHandler::PrgIcon (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("prg_icon") > 0)
		{
			string value = getText (node.getChildNode ("prg_icon"));
			return ParseColor (value);
		}
	}
	return D_PRG_ICON;
}
char* ConfigHandler::ConfigFile ()
{
	if (show_debug) g_debug ("ConfigHandler::ConfigFile (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("config_file") > 0)
		{
			string value = getText (node.getChildNode ("config_file"));
			return ParseColor (value);
		}
	}
	return D_CONFIG_FILE;
}
char* ConfigHandler::StoryFile ()
{
	if (show_debug) g_debug ("ConfigHandler::StoryFile (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("story_file") > 0)
		{
			string value = getText (node.getChildNode ("story_file"));
			return ParseColor (value);
		}
	}
	return D_STORY_FILE;
}
char* ConfigHandler::PictureFile ()
{
	if (show_debug) g_debug ("ConfigHandler::PictureFile (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("picture_file") > 0)
		{
			string value = getText (node.getChildNode ("picture_file"));
			return ParseColor (value);
		}
	}
	return D_PICTURE_FILE;
}
char* ConfigHandler::PicBackgroundImage ()
{
	if (show_debug) g_debug ("ConfigHandler::PicBackgroundImage (): Running");
	if (config.nChildNode ("files") > 0)
	{
		XMLNode node = config.getChildNode ("files");
		if (node.nChildNode ("pic_background_image") > 0)
		{
			string value = getText (node.getChildNode ("pic_background_image"));
			return ParseColor (value);
		}
	}
	return D_PIC_BACKGROUND_IMAGE;
}

// canvas
gboolean ConfigHandler::CanvasAntiAliased ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasAntiAliased (): Running");
//	// antialiasing is not supported yet
//	if (config.nChildNode ("canvas") > 0)
//	{
//		XMLNode node = config.getChildNode ("canvas");
//		if (node.nChildNode ("canvas_anti_aliased") > 0)
//		{
//			string value = getText (node.getChildNode ("canvas_anti_aliased"));
//			if (value.compare ("true") == 0) return true;
//			else return false;
//		}
//	}
	return D_CANVAS_ANTI_ALIASED;
}
double ConfigHandler::CanvasWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasWidth (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_width") > 0)
		{
			string value = getText (node.getChildNode ("canvas_width"));
			return atof (value.c_str());
		}
	}
	return D_CANVAS_WIDTH;
}
double ConfigHandler::CanvasHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasHeight (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_height") > 0)
		{
			string value = getText (node.getChildNode ("canvas_height"));
			return atof (value.c_str());
		}
	}
	return D_CANVAS_HEIGHT;
}
double ConfigHandler::CanvasBorder ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasBorder (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_border") > 0)
		{
			string value = getText (node.getChildNode ("canvas_border"));
			return atof (value.c_str());
		}
	}
	return D_CANVAS_BORDER;
}
char* ConfigHandler::CanvasBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasBorderColor (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_border_color") > 0)
		{
			string value = getText (node.getChildNode ("canvas_border_color"));
			return ParseColor (value);
		}
	}
	return D_CANVAS_BORDER_COLOR;
}
char* ConfigHandler::CanvasBackgroundColor ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasBackgroundColor (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_background_color") > 0)
		{
			string value = getText (node.getChildNode ("canvas_background_color"));
			return ParseColor (value);
		}
	}
	return D_CANVAS_BACKGROUND_COLOR;
}

gboolean ConfigHandler::CanvasGridShow ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasGridShow (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_grid_show") > 0)
		{
			string value = getText (node.getChildNode ("canvas_grid_show"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_CANVAS_GRID_SHOW;
}
char* ConfigHandler::CanvasGridColor ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasGridColor (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_grid_color") > 0)
		{
			string value = getText (node.getChildNode ("canvas_grid_color"));
			return ParseColor (value);
		}
	}
	return D_CANVAS_GRID_COLOR;
}
double ConfigHandler::CanvasGridThick ()
{
	if (show_debug) g_debug ("ConfigHandler::CanvasGridThick (): Running");
	if (config.nChildNode ("canvas") > 0)
	{
		XMLNode node = config.getChildNode ("canvas");
		if (node.nChildNode ("canvas_grid_thick") > 0)
		{
			string value = getText (node.getChildNode ("canvas_grid_thick"));
			return atof (value.c_str());
		}
	}
	return D_CANVAS_GRID_THICK;
}

// textfield
gboolean ConfigHandler::TextfDock ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDock (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_DOCK;
}
gboolean ConfigHandler::TextfDockTop ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockTop (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_top") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_top"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_DOCK_TOP;
}
gboolean ConfigHandler::TextfDockBottom ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockBottom (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_bottom") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_bottom"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_DOCK_BOTTOM;
}
gboolean ConfigHandler::TextfDockRight ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockRight (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_right") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_right"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_DOCK_RIGHT;
}
double ConfigHandler::TextfDockTopMargin ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockTopMargin (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_top_margin") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_top_margin"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_DOCK_TOP_MARGIN;
}
double ConfigHandler::TextfDockBottomMargin ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockBottomMargin (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_bottom_margin") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_bottom_margin"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_DOCK_BOTTOM_MARGIN;
}
double ConfigHandler::TextfDockRightMargin ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfDockRightMargin (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_dock_right_margin") > 0)
		{
			string value = getText (node.getChildNode ("textf_dock_right_margin"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_DOCK_RIGHT_MARGIN;
}

double ConfigHandler::TextfWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfWidth (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_width") > 0)
		{
			string value = getText (node.getChildNode ("textf_width"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_WIDTH;
}
double ConfigHandler::TextfHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfHeight (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_height") > 0)
		{
			string value = getText (node.getChildNode ("textf_height"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_HEIGHT;
}
double ConfigHandler::TextfY ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfY (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_y") > 0)
		{
			string value = getText (node.getChildNode ("textf_y"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_Y;
}
double ConfigHandler::TextfX ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfX (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_x") > 0)
		{
			string value = getText (node.getChildNode ("textf_x"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_X;
}
double ConfigHandler::TextfPadding ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfPadding (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_padding") > 0)
		{
			string value = getText (node.getChildNode ("textf_padding"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_PADDING;
}
double ConfigHandler::TextfPaddingVert ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfPaddingVert (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_padding_vert") > 0)
		{
			string value = getText (node.getChildNode ("textf_padding_vert"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_PADDING_VERT;
}
double ConfigHandler::TextfBorder ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfBorder (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_border") > 0)
		{
			string value = getText (node.getChildNode ("textf_border"));
			return atof (value.c_str());
		}
	}
	return D_TEXTF_BORDER;
}
char* ConfigHandler::TextfBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfBorderColor (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_border_color") > 0)
		{
			string value = getText (node.getChildNode ("textf_border_color"));
			return ParseColor (value);
		}
	}
	return D_TEXTF_BORDER_COLOR;
}
gboolean ConfigHandler::TextfBorderShow ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfBorderShow (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_border_show") > 0)
		{
			string value = getText (node.getChildNode ("textf_border_show"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_BORDER_SHOW;
}
gboolean ConfigHandler::TextfInnerShow ()
{
	if (show_debug) g_debug ("ConfigHandler::TextfInnerShow (): Running");
	if (config.nChildNode ("textfield") > 0)
	{
		XMLNode node = config.getChildNode ("textfield");
		if (node.nChildNode ("textf_inner_show") > 0)
		{
			string value = getText (node.getChildNode ("textf_inner_show"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXTF_INNER_SHOW;
}

// (author) info
double ConfigHandler::InfoX ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoX (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_x") > 0)
		{
			string value = getText (node.getChildNode ("info_x"));
			return atof (value.c_str());
		}
	}
	return D_INFO_X;
}
double ConfigHandler::InfoY ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoY (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_y") > 0)
		{
			string value = getText (node.getChildNode ("info_y"));
			return atof (value.c_str());
		}
	}
	return D_INFO_Y;
}
double ConfigHandler::InfoWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoWidth (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_width") > 0)
		{
			string value = getText (node.getChildNode ("info_width"));
			return atof (value.c_str());
		}
	}
	return D_INFO_WIDTH;
}
double ConfigHandler::InfoHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoHeight (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_height") > 0)
		{
			string value = getText (node.getChildNode ("info_height"));
			return atof (value.c_str());
		}
	}
	return D_INFO_HEIGHT;
}
double ConfigHandler::InfoPadding ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoPadding (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_padding") > 0)
		{
			string value = getText (node.getChildNode ("info_padding"));
			return atof (value.c_str());
		}
	}
	return D_INFO_PADDING;
}
gboolean ConfigHandler::InfoDock ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoDock (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_dock") > 0)
		{
			string value = getText (node.getChildNode ("info_dock"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_INFO_DOCK;
}
double ConfigHandler::InfoDockBottom ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoDockBottom (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_dock_bottom") > 0)
		{
			string value = getText (node.getChildNode ("info_dock_bottom"));
			return atof (value.c_str());
		}
	}
	return D_INFO_DOCK_BOTTOM;
}
double ConfigHandler::InfoDockRight ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoDockRight (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_dock_right") > 0)
		{
			string value = getText (node.getChildNode ("info_dock_right"));
			return atof (value.c_str());
		}
	}
	return D_INFO_DOCK_RIGHT;
}
double ConfigHandler::InfoDockLeft ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoDockLeft (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_dock_left") > 0)
		{
			string value = getText (node.getChildNode ("info_dock_left"));
			return atof (value.c_str());
		}
	}
	return D_INFO_DOCK_LEFT;
}
double ConfigHandler::InfoBorder ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoBorder (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_border") > 0)
		{
			string value = getText (node.getChildNode ("info_border"));
			return atof (value.c_str());
		}
	}
	return D_INFO_BORDER;
}
gboolean ConfigHandler::InfoBorderShow ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoBorderShow (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_border_show") > 0)
		{
			string value = getText (node.getChildNode ("info_border_show"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_INFO_BORDER_SHOW;
}
char* ConfigHandler::InfoBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::InfoBorderColor (): Running");
	if (config.nChildNode ("info") > 0)
	{
		XMLNode node = config.getChildNode ("info");
		if (node.nChildNode ("info_border_color") > 0)
		{
			string value = getText (node.getChildNode ("info_border_color"));
			return ParseColor (value);
		}
	}
	return D_INFO_BORDER_COLOR;
}

// text
char* ConfigHandler::TextFamily ()
{
	if (show_debug) g_debug ("ConfigHandler::TextFamily (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_family") > 0)
		{
			string value = getText (node.getChildNode ("text_family"));
			return (char*) value.c_str();
		}
	}
	return D_TEXT_FAMILY;
}
int ConfigHandler::TextSize ()
{
	if (show_debug) g_debug ("ConfigHandler::TextSize (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_size") > 0)
		{
			string value = getText (node.getChildNode ("text_size"));
			return atoi (value.c_str()) * PANGO_SCALE;
		}
	}
	return D_TEXT_SIZE * PANGO_SCALE;
}
int ConfigHandler::TextSizeHeading ()
{
	if (show_debug) g_debug ("ConfigHandler::TextSizeHeading (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_size_heading") > 0)
		{
			string value = getText (node.getChildNode ("text_size_heading"));
			return atoi (value.c_str()) * PANGO_SCALE;
		}
	}
	return D_TEXT_SIZE_HEADING * PANGO_SCALE;
}
double ConfigHandler::TextSpacingVert ()
{
	if (show_debug) g_debug ("ConfigHandler::TextSpacingVert (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_spacing_vert") > 0)
		{
			string value = getText (node.getChildNode ("text_spacing_vert"));
			return atof (value.c_str());
		}
	}
	return D_TEXT_SPACING_VERT;
}
double ConfigHandler::TextSpacingHoriz ()
{
	if (show_debug) g_debug ("ConfigHandler::TextSpacingHoriz (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_spacing_horiz") > 0)
		{
			string value = getText (node.getChildNode ("text_spacing_horiz"));
			return atof (value.c_str());
		}
	}
	return D_TEXT_SPACING_HORIZ;
}
gboolean ConfigHandler::TextHeadlineCenter ()
{
	if (show_debug) g_debug ("ConfigHandler::TextHeadlineCenter (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_headline_center") > 0)
		{
			string value = getText (node.getChildNode ("text_headline_center"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TEXT_HEADLINE_CENTER;
}
char* ConfigHandler::TextParagraphIndend ()
{
	if (show_debug) g_debug ("ConfigHandler::TextParagraphIndend (): Running");
	if (config.nChildNode ("text") > 0)
	{
		XMLNode node = config.getChildNode ("text");
		if (node.nChildNode ("text_paragraph_indend") > 0)
		{
			string value = getText (node.getChildNode ("text_paragraph_indend"));
			return (char*) value.c_str();
		}
	}
	return D_TEXT_PARAGRAPH_INDEND;
}

// font
char* ConfigHandler::FontColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_color") > 0)
		{
			string value = getText (node.getChildNode ("font_color"));
			return ParseColor (value);
		}
	}
	return D_FONT_COLOR;
}
char* ConfigHandler::FontBColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontBColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_bcolor") > 0)
		{
			string value = getText (node.getChildNode ("font_bcolor"));
			return ParseColor (value);
		}
	}
	return D_FONT_BCOLOR;
}
char* ConfigHandler::FontHltColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontHltColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_hlt_color") > 0)
		{
			string value = getText (node.getChildNode ("font_hlt_color"));
			return ParseColor (value);
		}
	}
	return D_FONT_HLT_COLOR;
}
char* ConfigHandler::FontHltBColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontHltBColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_hlt_bcolor") > 0)
		{
			string value = getText (node.getChildNode ("font_hlt_bcolor"));
			return ParseColor (value);
		}
	}
	return D_FONT_HLT_BCOLOR;
}
char* ConfigHandler::FontSelectColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontSelectColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_select_color") > 0)
		{
			string value = getText (node.getChildNode ("font_select_color"));
			return ParseColor (value);
		}
	}
	return D_FONT_SELECT_COLOR;
}
char* ConfigHandler::FontSelectBColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontSelectBColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_select_bcolor") > 0)
		{
			string value = getText (node.getChildNode ("font_select_bcolor"));
			return ParseColor (value);
		}
	}
	return D_FONT_SELECT_BCOLOR;
}
char* ConfigHandler::FontSelectHltColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontSelectHltColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_select_hlt_color") > 0)
		{
			string value = getText (node.getChildNode ("font_select_hlt_color"));
			return ParseColor (value);
		}
	}
	return D_FONT_SELECT_HLT_COLOR;
}
char* ConfigHandler::FontSelectHltBColor ()
{
	if (show_debug) g_debug ("ConfigHandler::FontSelectHltBColor (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_select_hlt_bcolor") > 0)
		{
			string value = getText (node.getChildNode ("font_select_hlt_bcolor"));
			return ParseColor (value);
		}
	}
	return D_FONT_SELECT_HLT_BCOLOR;
}

int ConfigHandler::FontColorAA ()
{
	if (show_debug) g_debug ("ConfigHandler::FontColorAA (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_color_aa") > 0)
		{
			string value = getText (node.getChildNode ("font_color_aa"));
			return ParseColor_AA (value);
		}
	}
	return D_FONT_COLOR_AA;
}
int ConfigHandler::FontHltColorAA ()
{
	if (show_debug) g_debug ("ConfigHandler::FontHltColorAA (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_hlt_color_aa") > 0)
		{
			string value = getText (node.getChildNode ("font_hlt_color_aa"));
			return ParseColor_AA (value);
		}
	}
	return D_FONT_HLT_COLOR_AA;
}
int ConfigHandler::FontBColorAA ()
{
	if (show_debug) g_debug ("ConfigHandler::FontBColorAA (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_bcolor_aa") > 0)
		{
			string value = getText (node.getChildNode ("font_bcolor_aa"));
			return ParseColor_AA (value);
		}
	}
	return D_FONT_BCOLOR_AA;
}
int ConfigHandler::FontHltBColorAA ()
{
	if (show_debug) g_debug ("ConfigHandler::FontHltBColorAA (): Running");
	if (config.nChildNode ("font") > 0)
	{
		XMLNode node = config.getChildNode ("font");
		if (node.nChildNode ("font_hlt_bcolor_aa") > 0)
		{
			string value = getText (node.getChildNode ("font_hlt_bcolor_aa"));
			return ParseColor_AA (value);
		}
	}
	return D_FONT_HLT_BCOLOR_AA;
}

// tooltips
gboolean ConfigHandler::TipShowTooltips ()
{
	if (show_debug) g_debug ("ConfigHandler::TipShowTooltips (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_show_tooltips") > 0)
		{
			string value = getText (node.getChildNode ("tip_show_tooltips"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TIP_SHOW_TOOLTIPS;
}
char* ConfigHandler::TipColor ()
{
	if (show_debug) g_debug ("ConfigHandler::TipColor (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_color") > 0)
		{
			string value = getText (node.getChildNode ("tip_color"));
			return ParseColor (value);
		}
	}
	return D_TIP_COLOR;
}
char* ConfigHandler::TipBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::TipBorderColor (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_border_color") > 0)
		{
			string value = getText (node.getChildNode ("tip_border_color"));
			return ParseColor (value);
		}
	}
	return D_TIP_BORDER_COLOR;
}
char* ConfigHandler::TipBorderColorAlt ()
{
	if (show_debug) g_debug ("ConfigHandler::TipBorderColorAlt (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_border_color_alt") > 0)
		{
			string value = getText (node.getChildNode ("tip_border_color_alt"));
			return ParseColor (value);
		}
	}
	return D_TIP_BORDER_COLOR_ALT;
}
char* ConfigHandler::TipFontColor ()
{
	if (show_debug) g_debug ("ConfigHandler::TipFontColor (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_font_color") > 0)
		{
			string value = getText (node.getChildNode ("tip_font_color"));
			return ParseColor (value);
		}
	}
	return D_TIP_FONT_COLOR;
}
double ConfigHandler::TipBorderThick ()
{
	if (show_debug) g_debug ("ConfigHandler::TipBorderThick (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_border_thick") > 0)
		{
			string value = getText (node.getChildNode ("tip_border_thick"));
			return atof (value.c_str());
		}
	}
	return D_TIP_BORDER_THICK;
}
int ConfigHandler::TipFontSize ()
{
	if (show_debug) g_debug ("ConfigHandler::TipFontSize (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_font_size") > 0)
		{
			string value = getText (node.getChildNode ("tip_font_size"));
			return atoi (value.c_str()) * PANGO_SCALE;
		}
	}
	return D_TIP_FONT_SIZE * PANGO_SCALE;
}
double ConfigHandler::TipPaddingHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::TipPaddingHeight (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_padding_height") > 0)
		{
			string value = getText (node.getChildNode ("tip_padding_height"));
			return atof (value.c_str());
		}
	}
	return D_TIP_PADDING_HEIGHT;
}
double ConfigHandler::TipPaddingWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::TipPaddingWidth (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_padding_width") > 0)
		{
			string value = getText (node.getChildNode ("tip_padding_width"));
			return atof (value.c_str());
		}
	}
	return D_TIP_PADDING_WIDTH;
}
gboolean ConfigHandler::TipBorderPlastic ()
{
	if (show_debug) g_debug ("ConfigHandler::TipBorderPlastic (): Running");
	if (config.nChildNode ("tooltips") > 0)
	{
		XMLNode node = config.getChildNode ("tooltips");
		if (node.nChildNode ("tip_border_plastic") > 0)
		{
			string value = getText (node.getChildNode ("tip_border_plastic"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_TIP_BORDER_PLASTIC;
}

// buttons
double ConfigHandler::ButtonBorderWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonBorderWidth (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_border_width") > 0)
		{
			string value = getText (node.getChildNode ("button_border_width"));
			return atof (value.c_str());
		}
	}
	return D_BUTTON_BORDER_WIDTH;
}
char* ConfigHandler::ButtonBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonBorderColor (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_border_color") > 0)
		{
			string value = getText (node.getChildNode ("button_border_color"));
			return ParseColor (value);
		}
	}
	return D_BUTTON_BORDER_COLOR;
}
char* ConfigHandler::ButtonShadowColor ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonShadowColor (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_shadow_color") > 0)
		{
			string value = getText (node.getChildNode ("button_shadow_color"));
			return ParseColor (value);
		}
	}
	return D_BUTTON_SHADOW_COLOR;
}
char* ConfigHandler::ButtonBackgroundColor ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonBackgroundColor (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_background_color") > 0)
		{
			string value = getText (node.getChildNode ("button_background_color"));
			return ParseColor (value);
		}
	}
	return D_BUTTON_BACKGROUND_COLOR;
}
int ConfigHandler::ButtonTextSize ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonTextSize (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_text_size") > 0)
		{
			string value = getText (node.getChildNode ("button_text_size"));
			int result = atoi (value.c_str());
			return result * PANGO_SCALE;
		}
	}
	return D_BUTTON_TEXT_SIZE * PANGO_SCALE;
}
char* ConfigHandler::ButtonTextColor ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonTextColor (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_text_color") > 0)
		{
			string value = getText (node.getChildNode ("button_text_color"));
			return ParseColor (value);
		}
	}
	return D_BUTTON_TEXT_COLOR;
}
char* ConfigHandler::ButtonTextColorAlt ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonTextColorAlt (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_text_color_alt") > 0)
		{
			string value = getText (node.getChildNode ("button_text_color_alt"));
			return ParseColor (value);
		}
	}
	return D_BUTTON_TEXT_COLOR_ALT;
}
double ConfigHandler::ButtonWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonWidth (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_width") > 0)
		{
			string value = getText (node.getChildNode ("button_width"));
			return atof (value.c_str());
		}
	}
	return D_BUTTON_WIDTH;
}
double ConfigHandler::ButtonHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonHeight (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_height") > 0)
		{
			string value = getText (node.getChildNode ("button_height"));
			return atof (value.c_str());
		}
	}
	return D_BUTTON_HEIGHT;
}
gboolean ConfigHandler::ButtonShowImageButtons ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonShowImageButtons (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_show_image_buttons") > 0)
		{
			string value = getText (node.getChildNode ("button_show_image_buttons"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_BUTTON_SHOW_IMAGE_BUTTONS;
}
gboolean ConfigHandler::ButtonShowImgLabels ()
{
	if (show_debug) g_debug ("ConfigHandler::ButtonShowImgLabels (): Running");
	if (config.nChildNode ("buttons") > 0)
	{
		XMLNode node = config.getChildNode ("buttons");
		if (node.nChildNode ("button_show_img_labels") > 0)
		{
			string value = getText (node.getChildNode ("button_show_img_labels"));
			if (value.compare ("true") == 0) return true;
			else return false;
		}
	}
	return D_BUTTON_SHOW_IMG_LABELS;
}

//listbox
double ConfigHandler::LbX ()
{
	if (show_debug) g_debug ("ConfigHandler::LbX (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_x") > 0)
		{
			string value = getText (node.getChildNode ("lb_x"));
			return atof (value.c_str());
		}
	}
	return D_LB_X;
}
double ConfigHandler::LbY ()
{
	if (show_debug) g_debug ("ConfigHandler::LbY (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_y") > 0)
		{
			string value = getText (node.getChildNode ("lb_y"));
			return atof (value.c_str());
		}
	}
	return D_LB_Y;
}
double ConfigHandler::LbWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::LbWidth (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_width") > 0)
		{
			string value = getText (node.getChildNode ("lb_width"));
			return atof (value.c_str());
		}
	}
	return D_LB_WIDTH;
}
double ConfigHandler::LbHeight ()
{
	if (show_debug) g_debug ("ConfigHandler::LbHeight (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_height") > 0)
		{
			string value = getText (node.getChildNode ("lb_height"));
			return atof (value.c_str());
		}
	}
	return D_LB_HEIGHT;
}
double ConfigHandler::LbBorderWidth ()
{
	if (show_debug) g_debug ("ConfigHandler::LbBorderWidth (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_border_width") > 0)
		{
			string value = getText (node.getChildNode ("lb_border_width"));
			return atof (value.c_str());
		}
	}
	return D_LB_BORDER_WIDTH;
}
int ConfigHandler::LbTextSize ()
{
	if (show_debug) g_debug ("ConfigHandler::LbTextSize (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_text_size") > 0)
		{
			string value = getText (node.getChildNode ("lb_text_size"));
			return atoi (value.c_str()) * PANGO_SCALE;
		}
	}
	return D_LB_TEXT_SIZE * PANGO_SCALE;
}
char* ConfigHandler::LbTextColor ()
{
	if (show_debug) g_debug ("ConfigHandler::LbTextColor (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_text_color") > 0)
		{
			string value = getText (node.getChildNode ("lb_text_color"));
			return ParseColor (value);
		}
	}
	return D_LB_TEXT_COLOR;
}
char* ConfigHandler::LbBorderColor ()
{
	if (show_debug) g_debug ("ConfigHandler::LbBorderColor (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_border_color") > 0)
		{
			string value = getText (node.getChildNode ("lb_border_color"));
			return ParseColor (value);
		}
	}
	return D_LB_BORDER_COLOR;
}
double ConfigHandler::LbPadding ()
{
	if (show_debug) g_debug ("ConfigHandler::LbPadding (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_padding") > 0)
		{
			string value = getText (node.getChildNode ("lb_padding"));
			return atof (value.c_str());
		}
	}
	return D_LB_PADDING;
}
double ConfigHandler::LbBtnCancelX ()
{
	if (show_debug) g_debug ("ConfigHandler::LbBtnCancelX (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_btn_cancel_x") > 0)
		{
			string value = getText (node.getChildNode ("lb_btn_cancel_x"));
			return atof (value.c_str());
		}
	}
	return D_LB_BTN_CANCEL_X;
}
double ConfigHandler::LbBtnCancelY ()
{
	if (show_debug) g_debug ("ConfigHandler::LbBtnCancelY (): Running");
	if (config.nChildNode ("listbox") > 0)
	{
		XMLNode node = config.getChildNode ("listbox");
		if (node.nChildNode ("lb_btn_cancel_y") > 0)
		{
			string value = getText (node.getChildNode ("lb_btn_cancel_y"));
			return atof (value.c_str());
		}
	}
	return D_LB_BTN_CANCEL_Y;
}

// PRIVATE
string 	ConfigHandler::getValue (XMLNode node, char* attName)
{
	char* buff;
	string result;
	int reqMem, l_char, l_value, sprintf_buff;
	bool memAllocated;
	bool _DEBUG = false;
	
	if (_DEBUG) cout << "Getting value for attribute " << attName << endl;
	
	// if (_DEBUG) cout << " - Checking size of char ... ";
	l_char = sizeof (char);
	if (_DEBUG) cout << l_char << endl;
	if (_DEBUG) cout << " - Checking length of value ... ";
	l_value = strlen (node.getAttribute(attName));
	if (_DEBUG) cout << l_value << endl;
	
	reqMem = l_char * l_value + 1;
	if (_DEBUG) cout << " - Allocating " << reqMem << "B of memory ... ";
	buff = (char*) malloc (reqMem);
	if (buff == NULL) { memAllocated = false; if (_DEBUG) cout << "failed!" << endl; }
	else { memAllocated = true; if (_DEBUG) cout << "done" << endl; }
	
	if (!memAllocated)
	{
		if (_DEBUG) cout << " - Not enough memory available, aborting..." << endl;
		return NULL;
	}
	
	if (_DEBUG) cout << " - Copying string ... ";
	sprintf_buff = sprintf (buff, "%s", node.getAttribute(attName));
	if (sprintf_buff < 0)
	{
		if (_DEBUG) cout << " error #" << sprintf_buff << endl;
	}
	else
	{
		if (_DEBUG) cout << buff << endl;
	}
	result = buff;
	free (buff);
	return result;
}
string 	ConfigHandler::getText (XMLNode node)
{
	char* buff;
	string result;
	int reqMem, l_char, l_value, sprintf_buff;
	bool memAllocated;
	bool _DEBUG = false;
	
	if (_DEBUG) cout << "Getting text ..." << endl;
	
	// if (_DEBUG) cout << " - Checking size of char ... ";
	l_char = sizeof (char);
	// if (_DEBUG) cout << l_char << endl;
	if (_DEBUG) cout << " - Checking length of value ... ";
	l_value = strlen (node.getText());
	if (_DEBUG) cout << l_value << endl;
	
	reqMem = l_char * l_value + 1;
	if (_DEBUG) cout << " - Allocating " << reqMem << "B of memory ... ";
	buff = (char*) malloc (reqMem);
	if (buff == NULL) { memAllocated = false; if (_DEBUG) cout << "failed!" << endl; }
	else { memAllocated = true; if (_DEBUG) cout << " - Memory allocated successfully." << endl; }
	
	if (!memAllocated)
	{
		if (_DEBUG) cout << " - Not enough memory available, aborting..." << endl;
		return NULL;
	}
	
	if (_DEBUG) cout << " - Copying string ... ";
	sprintf_buff = sprintf (buff, "%s", node.getText());
	if (sprintf_buff < 0)
	{
		if (_DEBUG) cout << " - error #" << sprintf_buff << " while sprintf-ing" << endl;
	}
	else
	{
		if (_DEBUG) cout << " - Value in buff: " << buff << endl;
	}
	
	if (_DEBUG) cout << " - Assigning buff to result" << endl;
	result = buff;
	if (_DEBUG) cout << " - Trying to free buff..." << endl;
	free (buff);
	if (_DEBUG) cout << " - - Done freeing memory... returning result" << endl;
	return result;
}

char* ConfigHandler::ParseColor (string color)
{
	// --------------------------------------------------------------------
	// Uses color keyword (i.e. BLACK) and returns its GDK code (i.e. #000000).
	// --------------------------------------------------------------------
	if (color.compare ("BLACK") == 0) return BLACK;
	if (color.compare ("WHITE") == 0) return WHITE;
	if (color.compare ("BLUE") == 0) return BLUE;
	if (color.compare ("LIGHT_BLUE") == 0) return LIGHT_BLUE;
	if (color.compare ("DARK_BLUE") == 0) return DARK_BLUE;
	if (color.compare ("TIRQUISE") == 0) return TIRQUISE;
	if (color.compare ("GREEN") == 0) return GREEN;
	if (color.compare ("DARK_GREEN") == 0) return DARK_GREEN;
	if (color.compare ("LIGHT_GREEN") == 0) return LIGHT_GREEN;
	if (color.compare ("AQUA_GREEN") == 0) return AQUA_GREEN;
	if (color.compare ("YELLOW") == 0) return YELLOW;
	if (color.compare ("LIGHT_YELLOW") == 0) return LIGHT_YELLOW;
	if (color.compare ("GREY") == 0) return GREY;
	if (color.compare ("LIGHT_GREY") == 0) return LIGHT_GREY;
	if (color.compare ("RED") == 0) return RED;
	
	char* result = g_strdup_printf ("%s", color.c_str());
	return result; // if color is not recognized, return input value
}
int ConfigHandler::ParseColor_AA (string color)
{
	// --------------------------------------------------------------------
	// Uses color keyword (i.e. BLACK) and returns its RGBA code (i.e. 0x00000000).
	// --------------------------------------------------------------------
	if (color.compare ("BLACK") == 0) return BLACK_AA;
	if (color.compare ("WHITE") == 0) return WHITE_AA;
	if (color.compare ("BLUE") == 0) return BLUE_AA;
	if (color.compare ("LIGHT_BLUE") == 0) return LIGHT_BLUE_AA;
	if (color.compare ("DARK_BLUE") == 0) return DARK_BLUE_AA;
	if (color.compare ("TIRQUISE") == 0) return TIRQUISE_AA;
	if (color.compare ("GREEN") == 0) return GREEN_AA;
	if (color.compare ("LIGHT_GREEN") == 0) return LIGHT_GREEN_AA;
	if (color.compare ("AQUA_GREEN") == 0) return AQUA_GREEN_AA;
	if (color.compare ("YELLOW") == 0) return YELLOW_AA;
	if (color.compare ("LIGHT_YELLOW") == 0) return LIGHT_YELLOW_AA;
	if (color.compare ("GREY") == 0) return GREY_AA;
	if (color.compare ("LIGHT_GREY") == 0) return LIGHT_GREY_AA;
	
	return 0x00000000;
}
char* ConfigHandler::ColorToString (char* color_str)
{
	// --------------------------------------------------------------------
	// Converts color code in form #RRGGBB into  a color keyword.
	// --------------------------------------------------------------------
	string color = color_str;
	
	if (color.compare (BLACK) == 0) return "BLACK";
	if (color.compare (WHITE) == 0) return "WHITE";
	if (color.compare (BLUE) == 0) return "BLUE";
	if (color.compare (LIGHT_BLUE) == 0) return "LIGHT_BLUE";
	if (color.compare (DARK_BLUE) == 0) return "DARK_BLUE";
	if (color.compare (TIRQUISE) == 0) return "TIRQUISE";
	if (color.compare (GREEN) == 0) return "GREEN";
	if (color.compare (DARK_GREEN) == 0) return "DARK_GREEN";
	if (color.compare (LIGHT_GREEN) == 0) return "LIGHT_GREEN";
	if (color.compare (AQUA_GREEN) == 0) return "AQUA_GREEN";
	if (color.compare (YELLOW) == 0) return "YELLOW";
	if (color.compare (LIGHT_YELLOW) == 0) return "LIGHT_YELLOW";
	if (color.compare (GREY) == 0) return "GREY";
	if (color.compare (LIGHT_GREY) == 0) return "LIGHT_GREY";
	if (color.compare (RED) == 0) return "RED";
	
	return "BLACK";
}
char* ConfigHandler::ColorToString_AA (int color_code)
{
	// --------------------------------------------------------------------
	// Converts color code in form 0xRRGGBBOO into a color keyword.
	// --------------------------------------------------------------------
	
	if ((int) BLACK_AA == color_code) return "BLACK";
	if ((int) WHITE_AA == color_code) return "WHITE";
	if ((int) BLUE_AA == color_code) return "BLUE";
	if ((int) LIGHT_BLUE_AA == color_code) return "LIGHT_BLUE";
	if ((int) DARK_BLUE_AA == color_code) return "DARK_BLUE";
	if ((int) TIRQUISE_AA == color_code) return "TIRQUISE";
	if ((int) GREEN_AA == color_code) return "GREEN";
	if ((int) LIGHT_GREEN_AA == color_code) return "LIGHT_GREEN";
	if ((int) AQUA_GREEN_AA == color_code) return "AQUA_GREEN";
	if ((int) YELLOW_AA == color_code) return "YELLOW";
	if ((int) LIGHT_YELLOW_AA == color_code) return "LIGHT_YELLOW";
	if ((int) GREY_AA == color_code) return "GREY";
	if ((int) LIGHT_GREY_AA == color_code) return "LIGHT_GREY";
	
	return "BLACK";
}
