#ifndef SOUND_H_
#define SOUND_H_

#include "struct.h"
#include "config_handler.h"
#include <sndfile.h>

// includes for uloha02 example ( + sndfile)
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>


#define AUDIO_DEV "/dev/dsp"
#define BUF_SIZE 4096*8

#define PHONET_FILE		"phonet.txt"
#define GRAM_FILE		"words.gram"
#define LAT_FILE		"words.lat"


typedef struct _PhonetEntry PhonetEntry;
struct _PhonetEntry
{
	string name;
	string word;				// with brackets stripped
	vector<string> phonemes;
};


/**
 * @author Martin Malar
 * Apr 7, 2007 5:25:39 PM
 * 
 * Sound Record/Playback class.
 * 
 */
class Sound
{
	public:
		Sound (char* resx_path, char* current_story_dir_name); // current_story_dir_name = name of storydir within resx
	   ~Sound ();
		// --------------------------------------------------------------------
		// METHODS 			- PUBLIC
		// --------------------------------------------------------------------
		int PlayStart (char* file_name);
		void PlayStop ();
		int RecStart ();
		void Stop ();
		
		void RecStart_gc ();		// method for recording in GCompris
		
		gboolean PhonetToGram ();			// convert phonetical dict .txt file into grammar .gram file
		gboolean GramToLat ();				// generate .lat file from grammar
		vector<PhonetEntry> GetPhonemes (char* c_word, vector<PhonetEntry>& words_all);// load word's phonetical notation as array of strings
		string GetModelName (char* c_word);
		vector<PhonetEntry> GeneratePrefixes (PhonetEntry ent);
		vector<PhonetEntry> GenerateSuffixes (PhonetEntry ent);
		void GenerateSimilar (char* c_word, vector<PhonetEntry>& e, vector<PhonetEntry> e_all);
		void RemoveDuplicates (vector<PhonetEntry>& e);
		gboolean GeneratePhoneticDict (char* c_word); // generate new dictionary for recognizing specified word
		
		friend void play_start_play (Sound &snd);  // obsolete method using system play command
		friend void rec_start (Sound &snd);
		friend void rec_start_old (Sound &snd);
		friend void play_start_new (Sound &snd);
		
		// --------------------------------------------------------------------
		// PROPERTIES 		- PUBLIC
		// --------------------------------------------------------------------
		gboolean Playback_in_progress ();
		string GetLastResult ();
		gboolean CheckLastResult ();
		
		// --------------------------------------------------------------------
		// ATTRIBUTES 		- PUBLIC
		// --------------------------------------------------------------------
		
	
	private:
		// --------------------------------------------------------------------
		// METHODS 			- PRIVATE
		// --------------------------------------------------------------------
		
		// --------------------------------------------------------------------
		// EVENTS 			- PRIVATE
		// --------------------------------------------------------------------
		
		// --------------------------------------------------------------------
		// ATTRIBUTES 		- PRIVATE
		// --------------------------------------------------------------------
		gboolean end;			// indicates recording should be stopped at once
		gboolean playback_progress; // playback is in progress
		char* path;				// path to resx
		char* work_dir_name;	// name of directory with tools/data for speech recognition
		char* work_dir_path;	// path&name of working directory (i.e. path_resx / work_dir_name)
		char* story_dir;		// dir name (no path) of currently open story (full path is 'path / story_dir')
		
		GThread *playback;		// playback thread
		GThread *recording;		// thread for recording and voice recognition
		
		char* playback_file_name; // path&filename to play during play_start
		
		string last_result;
		
};


// other
void play_start_play (Sound &snd);
void rec_start (Sound &ss);
void rec_start_old (Sound &ss);
void play_start_new (Sound &snd);

// misc utility funtions
void entry_clear (PhonetEntry *e); // clears entry's contents
string trim_word (const char* w);
int sdist (string src, string dst); // computes distance of strings s1, s2 -- for details see header of the function


#endif /*SOUND_H_*/
