#ifndef STRUCT_H_
#define STRUCT_H_

// --------------------------------------------------------------------
// DEFINES
// --------------------------------------------------------------------
// general
#define PACKAGE					"ReadTutor"
#define _VERSION				"0.9"
#define _RELEASE				"4"
#define _BUILD					"9"
#define VERSION					_VERSION "." _RELEASE
#define VERSION_BUILD			_VERSION "." _RELEASE "-" _BUILD			// program.version.release-build
#define COPYRIGHT				"(c) 2007 FMFI UK Bratislava"
#define COMMENTS				"Projekt k diplomovej práci."
#define AUTHOR					"Martin Malár <martinmalar@gmail.com>"
#define WEBSITE					"http://elwin.php5.sk"


// --------------------------------------------------------------------
// INCLUDES
// --------------------------------------------------------------------
#include <iostream>
#include <string>
#include <vector>
#include <fstream>
#include <sstream>

#include <gnome.h>

#include "glib/gstdio.h" // required by g_stat(), g_lstat()

using namespace std;

// --------------------------------------------------------------------
// STRUCTURES
// --------------------------------------------------------------------
struct sWord
{
    string  	Text;			// word itself
    string  	ind;			// string index of word - extension of .mp3 file with spoken work
    int			Index;			// numerical index of word
    gboolean	isHeadline;		// is this word part of headline?
};
struct sParagraph
{
    vector<sWord>  Words;
};
struct sStory
{
	sParagraph			Headline;
	vector<sParagraph>	Paragraphs;
};

typedef struct _PersonData PersonData;
struct _PersonData
{
	char* name;
	char* surname;
	int age;
	int gender;
	
	gboolean has_picture;
};

typedef struct _StoryInfo StoryInfo;
struct _StoryInfo
{
	char* story_name;
	char* name;
	char* surname;
	char* file_name;
	char* dir_name;
	gboolean valid;
};

typedef struct _sColor sColor;
struct _sColor
{
	guint16	red;
	guint16	green;
	guint16	blue;
};

// text/fonts
typedef enum
{
	TEXT_STYLE_NORMAL,
	TEXT_STYLE_HEADLINE
} TextStyle;

typedef enum
{
	TEXT_COLOR_MODE_NORMAL,		// normal text
	TEXT_COLOR_MODE_HLT,		// highlighted text
	TEXT_COLOR_MODE_SELECT,		// selected text
	TEXT_COLOR_MODE_SELECT_HLT	// selected and highlighted text
} TextColorMode;

// markmode - style of marking color for a word.
typedef enum
{
	MARK_MODE_NONE,				// normal text
	MARK_MODE_CURRENT,			// current word in progress (in reading or dictate)
	MARK_MODE_PASSED,			// passed word (in reading or dictate)
	MARK_MODE_0,				// mark modes used to mark how successfull was speech recognition
	MARK_MODE_20,
	MARK_MODE_30,
	MARK_MODE_40,
	MARK_MODE_50,
	MARK_MODE_60,
	MARK_MODE_70,
	MARK_MODE_80,
	MARK_MODE_90,
	MARK_MODE_95
} MarkMode;

// components


// errors
typedef enum
{
	eLoadPageOk,				// page was loaded successfully
	eLoadPageNotExist,			// page with given page_number doesn't exist
	eLoadPageGeneralError
} ErrorLoadPage;

typedef enum
{
	eLoadDataOk,
	eLoadDataFailedOpenDir,
	
} ErrorLoadData;

// --------------------------------------------------------------------
// CONSTANTS
// --------------------------------------------------------------------
const sColor BLACK_S			= {     0,     0,     0 };	//   0,   0,   0
const sColor WHITE_S			= { 65535, 65535, 65535 };	// 255, 255, 255
const sColor BLUE_S				= {     0,     0, 49152 };	//   0,   0, 192
const sColor LIGHT_BLUE_S		= { 36352, 50944, 65535 };	// 142, 199, 255
const sColor LIGHT_GREEN_S		= { 36352, 65535, 50944 };	// 142, 255, 199
const sColor YELLOW_S			= { 65535, 55040,     0 };	// 255, 215,   0
const sColor GREY_S				= { 31232, 35584, 35584 };	// 122, 139, 139
const sColor TIRQUISE_S			= {     0, 50944, 65535 };	//   0, 199, 255


#endif /*STRUCT_H_*/

