#include  "xml.h"

Xml::Xml(int mode)
{
}
Xml::~Xml ()
{
}

// public
bool	Xml::ReadFile (string fileName)
{
	// check for file existence
	if (!g_file_test ((const gchar*) fileName.c_str (), G_FILE_TEST_EXISTS)) return false;
	
	// open the file
	root = XMLNode::openFileHelper (fileName.c_str(), "work");
	person = root.getChildNode ("person");
	headline = root.getChildNode ("headline");
	story = root.getChildNode ("story");
	
	// TODO: implement return value
	return true;
}
void	Xml::PrintInfo ()
{
	cout 	<< GetVersionInfo ()
			<< "Work id: " << getId() << ", lang: " << getLanguage() << endl
			<< "sound: " << getSound () << ", wordsounds: " << getWordSounds() << ", picture: " << getPicture() << endl
			<< "Person information:" << endl
			<< getName() << " " << getSurname() << " is " << getAge() << " years old " << getGender() << "." << endl
			<< "Teacher: " << getTeacher() << " on " << getSchool() << " " << getAddress() << " " << getCity() << ", " << getState() << endl
			<< "Headline: '" << getHeadlineText() << "' read with grade " << getGrade()
			<< endl;
}
void 	Xml::PrintStoryText ()
{
	cout << getStoryText ();
}
string 	Xml::GetPersonInfo()
{
	string out = "";
	out += getName() + " " + getSurname() + ", " + getAge() + " rokov, ";
	
		if (getGender() == "girl") out += "dievca";
		else if (getGender() == "boy") out += "chlapec";
		else out += "pohlavie nezname";
	out += ".";
	return out;
}
string 	Xml::GetVersion ()
{
	string out = VERSION_XML;
	return out;
}
string	Xml::GetVersionInfo ()
{
	string out = "Class for XML handling, v" + GetVersion() + "\nWritten by Martin Malar, 2007\n";
	return out;
}
void 	Xml::PrintVersionInfo ()
{
	cout 	<< GetVersionInfo();
}
PersonData* Xml::GetPersonData ()
{
	PersonData* data;
	
	data = g_new (PersonData, 1);
	data->name = g_strdup_printf ("%s", getName().c_str());
	data->surname = g_strdup_printf ("%s", getSurname().c_str());
	data->age = atoi (getAge ().c_str());
	if (g_ascii_strcasecmp (getGender ().c_str(), "girl") == 0) // if gender == "girl"
	{
		data->gender = 2;
	}
	else if (g_ascii_strcasecmp (getGender ().c_str(), "boy") == 0) // else if gender == "boy"
	{
		data->gender = 1;
	}
	else
	{
		data->gender = 0;
	}
	
	if (hasPicture()) data->has_picture = true;
	else data->has_picture = false;
	
	return data;
}
StoryInfo*	Xml::CheckDirectory (gchar* fullPath, gchar* dir_name)
{
	// --------------------------------------------------------------------
	// Returns true if fullPath directory contains valid xml story.
	// --------------------------------------------------------------------
	gchar* full_file;
	StoryInfo *si;
	struct stat st;
	
	si = g_new (StoryInfo, 1);
	si->valid = false;
	
	// check existence of the file
	full_file = g_strconcat (fullPath, G_DIR_SEPARATOR_S, "index_seg.xml", NULL);
	if (g_lstat (full_file, &st) == -1)
	{
		// error accesing the file
		return si;
	}
	
	// open file and check format
	if (!ReadFile (full_file))
	{
		g_error ("XML: Error opening file '%s'", full_file);
		return si;
	}
	// TODO Check format
	
	// fill in info
	si->story_name = g_strdup_printf ("%s", getHeadlineText().c_str());
	si->name = g_strdup_printf ("%s", getName().c_str());
	si->surname = g_strdup_printf ("%s", getSurname().c_str());
	si->file_name = g_strdup_printf ("%s", fullPath); // we remember file to story dir, not to story file
	si->dir_name = g_strdup_printf ("%s", dir_name); // name of directory itself
	si->valid = true;
	
	return si;
}

// properties
string 	Xml::getId ()
{
	return getValue (root, "id");
}
string 	Xml::getLanguage ()
{
	return getValue (root, "lang");
}
string 	Xml::getSound ()
{
	XMLNode node = root.getChildNode ("accompanied-by");
	return getValue (node, "sound");
}
string 	Xml::getWordSounds ()
{
	XMLNode node = root.getChildNode ("accompanied-by");
	return getValue (node, "wordsounds");
}
string 	Xml::getPicture ()
{
	XMLNode node = root.getChildNode ("accompanied-by");
	return getValue (node, "picture");
}
bool 	Xml::hasSound ()
{
	if (getSound() == "yes") return true;
	else return false;
}
bool 	Xml::hasWordSounds ()
{
	if (getWordSounds() == "yes") return true;
	else return false;
}
bool 	Xml::hasPicture ()
{
	if (getPicture() == "yes") return true;
	else return false;
}

string 	Xml::getName ()
{
	return getValue (person, "name");
}
string 	Xml::getSurname ()
{
	return getValue (person, "surname");
}
string 	Xml::getAge ()
{
	return getValue (person, "age");
}
string 	Xml::getGrade ()
{
	return getValue (person, "grade");
}
string 	Xml::getGender ()
{
	return getValue (person, "gender");
}
string 	Xml::getSchool ()
{
	return getValue (person, "school");
}
string 	Xml::getAddress ()
{
	return getValue (person, "address");
}
string 	Xml::getCity ()
{
	return getValue (person, "city");
}
string 	Xml::getState ()
{
	return getValue (person, "state");
}
string 	Xml::getPseudo ()
{
	return getValue (person, "pseudo");
}
string 	Xml::getTeacher ()
{
	return getValue (person, "teacher");
}
string 	Xml::getIdSchool()
{
	return getValue (person, "idschool");
}

string 	Xml::getHeadlineText ()
{
	string out = "";
	int i;
	
	for (i = 0; i < headline.nChildNode("w"); i++)
	{
		out += getText (headline.getChildNode("w", i));
		if (i < headline.nChildNode("w") - 1) 
			{
				out += " ";
			}
	}
	return out;
}
string 	Xml::getStoryText ()
{
	string 	out = "";
	int 	p; // current paragraph
	int		w; // current word
	XMLNode parag;
	
	for (p = 0; p < story.nChildNode ("p"); p++)
	{
		parag = story.getChildNode ("p", p);
		for (w = 0; w < parag.nChildNode ("w"); w++)
		{
			out += getText (parag.getChildNode ("w", w));
			if (w < parag.nChildNode("w") - 1)
				out += " ";
			else
				out += "\n";
		}
	}
	
	return out;
}
sStory	Xml::getStoryWords ()
{
	sStory ss;
	int p; // current paragraph
	int w; // current word
	XMLNode parag;
	bool _DEBUG = false;
	
	// Headline proccessing
	ss.Headline.Words.resize (headline.nChildNode("w"));
	for (w = 0; w < headline.nChildNode("w"); w++)
	{
		ss.Headline.Words[w].Text = getText  (headline.getChildNode ("w", w));
		ss.Headline.Words[w].ind  = getValue (headline.getChildNode ("w", w), "ind");
		ss.Headline.Words[w].isHeadline = true;
		ss.Headline.Words[w].Index = atoi (ss.Headline.Words[w].ind.c_str());
	}

	// Paragraphs proccessing
	ss.Paragraphs.resize ( story.nChildNode ("p") );
	for (p = 0; p < story.nChildNode("p"); p++)
	{
		parag = story.getChildNode ("p", p);
		ss.Paragraphs[p].Words.resize( parag.nChildNode("w") );
		for (w = 0; w < parag.nChildNode("w"); w++)
		{
			if (_DEBUG) cout << "getting text for word: " << w << ", paragraph: " << p << endl;
			ss.Paragraphs[p].Words[w].Text = getText  (parag.getChildNode ("w", w));
			if (_DEBUG) cout << "Done" << endl;
			if (_DEBUG) cout << "Getting value ind for word: " << w << ", paragraph: " << p << endl;
			ss.Paragraphs[p].Words[w].ind  = getValue (parag.getChildNode ("w", w), "ind");
			if (_DEBUG) cout << "Done" << endl; 
			ss.Paragraphs[p].Words[w].isHeadline = false;
			ss.Paragraphs[p].Words[w].Index = atoi (ss.Paragraphs[p].Words[w].ind.c_str());
		}
	}

	return ss;
}

// private
string 	Xml::getValue (XMLNode node, char* attName)
{
	char* buff;
	string result;
	int reqMem, l_char, l_value, sprintf_buff;
	bool memAllocated;
	bool _DEBUG = false;
	
	if (_DEBUG) cout << "Getting value for attribute " << attName << endl;
	
	// if (_DEBUG) cout << " - Checking size of char ... ";
	l_char = sizeof (char);
	if (_DEBUG) cout << l_char << endl;
	if (_DEBUG) cout << " - Checking length of value ... ";
	l_value = strlen (node.getAttribute(attName));
	if (_DEBUG) cout << l_value << endl;
	
	reqMem = l_char * l_value + 1;
	if (_DEBUG) cout << " - Allocating " << reqMem << "B of memory ... ";
	buff = (char*) malloc (reqMem);
	if (buff == NULL) { memAllocated = false; if (_DEBUG) cout << "failed!" << endl; }
	else { memAllocated = true; if (_DEBUG) cout << "done" << endl; }
	
	if (!memAllocated)
	{
		if (_DEBUG) cout << " - Not enough memory available, aborting..." << endl;
		return NULL;
	}
	
	if (_DEBUG) cout << " - Copying string ... ";
	sprintf_buff = sprintf (buff, "%s", node.getAttribute(attName));
	if (sprintf_buff < 0)
	{
		if (_DEBUG) cout << " error #" << sprintf_buff << endl;
	}
	else
	{
		if (_DEBUG) cout << buff << endl;
	}
	result = buff;
	free (buff);
	return result;
}
string 	Xml::getText (XMLNode node)
{
	char* buff;
	string result;
	int reqMem, l_char, l_value, sprintf_buff;
	bool memAllocated;
	bool _DEBUG = false;
	
	if (_DEBUG) cout << "Getting text ..." << endl;
	
	// if (_DEBUG) cout << " - Checking size of char ... ";
	l_char = sizeof (char);
	// if (_DEBUG) cout << l_char << endl;
	if (_DEBUG) cout << " - Checking length of value ... ";
	l_value = strlen (node.getText());
	if (_DEBUG) cout << l_value << endl;
	
	reqMem = l_char * l_value + 1;
	if (_DEBUG) cout << " - Allocating " << reqMem << "B of memory ... ";
	buff = (char*) malloc (reqMem);
	if (buff == NULL) { memAllocated = false; if (_DEBUG) cout << "failed!" << endl; }
	else { memAllocated = true; if (_DEBUG) cout << " - Memory allocated successfully." << endl; }
	
	if (!memAllocated)
	{
		if (_DEBUG) cout << " - Not enough memory available, aborting..." << endl;
		return NULL;
	}
	
	if (_DEBUG) cout << " - Copying string ... ";
	sprintf_buff = sprintf (buff, "%s", node.getText());
	if (sprintf_buff < 0)
	{
		if (_DEBUG) cout << " - error #" << sprintf_buff << " while sprintf-ing" << endl;
	}
	else
	{
		if (_DEBUG) cout << " - Value in buff: " << buff << endl;
	}
	
	if (_DEBUG) cout << " - Assigning buff to result" << endl;
	result = buff;
	if (_DEBUG) cout << " - Trying to free buff..." << endl;
	free (buff);
	if (_DEBUG) cout << " - - Done freeing memory... returning result" << endl;
	return result;
}


// main function
int mainxml(int argc, char* argv[])
{
//	if (argc <= 1)
//	{
//		cout << "Error: No file name provided." << endl; 
//		return 1;
//	}
//	
//	
//	Xml xml(0);
//	xml.ReadFile(argv[1]);
//	xml.PrintInfo ();
//	
//	cout << "Story text:" << endl;
//	xml.PrintStoryText ();
//	cout << endl << endl;
//	xml.PrintVersionInfo();
//	
//	sStory myStory;
//	myStory = xml.getStoryWords();
	
	
	
	
	
//	string s;
//	s = "tralala";
//	cout << s << endl;
//	s = "blablablabla";
//	cout << s << endl;
//	
//	string ss;
//	ss = "tralala";
//	
//	string cat = s + " " + ss; 
//	cout << cat << endl;
	
	
//	// EXAMPLE CODE ///////////////////////////////////////////////////////////
//	  // this open and parse the XML file:
//	  XMLNode xMainNode=XMLNode::openFileHelper("/home/martin/Programs/workspace/PangoTest/Debug/example.xml","PMML");
//	  //XMLNode xMainNode=XMLNode::openFileHelper("example.xml","PMML");
//	
//	  // this prints "RANK For <you>":
//	  XMLNode xNode=xMainNode.getChildNode("Header");
//	  printf("Application Name is: '%s'\n", xNode.getChildNode("Application").getAttribute("name"));
//	  
//	  // this prints "Hello world!":
//	  printf("Text inside Header tag is :'%s'\n", xNode.getText());
//	
//	  // this gets the number of "NumericPredictor" tags:
//	  xNode=xMainNode.getChildNode("RegressionModel").getChildNode("RegressionTable");
//	  int n=xNode.nChildNode("NumericPredictor");
//	
//	  // this prints the "coefficient" value for all the "NumericPredictor" tags:
//	  for (int i=0; i<n; i++)
//	    printf("coeff %i=%f\n",i+1,atof(xNode.getChildNode("NumericPredictor",i).getAttribute("coefficient")));
//	
//	  // this prints a formatted ouput based on the content of the first "Extension" tag of the XML file:
//	  char *t=xMainNode.getChildNode("Extension").createXMLString(true);
//	  printf("%s\n",t);
//	  free(t);


	return 0;
}
